/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     beck_acp.c
* Function:   Example of a ASP like CGI Handler. ACP stands for Active Chip Pages.
*             The beck_acp program installs a CGI handler that can be used to
*             interpret scripts in html pages before serving them to the browser.
*             This enables you to provide the browser with a content that changes
*             according to the state of the I/O.
*
*             The syntax used is described in a PDF file in this directory.
* Notes:      It is advised to set the webserver stack to a minimum of 5000 bytes
*             in the chip.ini file.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\ACP\beck_acp.c, 3, 31.03.2004 11:15:33, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <dir.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define BuffSize 300



/******************************************************************************
* Prototypes
******************************************************************************/
char* Write_func(char myRight[]);
char* Then_Else_func(char myRight[]);
char* Input_func(char myRight[]);
char* Activate_func(char myRight[]);
char* Deactivate_func(char myRight[]);
char* If_func(char myRight[]);
char* GetParams_func(char myRight[]);
char* InterpretRight_func(char myRight[], int ifclause);
char* Input_BC_func(char myRight[]);
char* Activate_BC_func(char myRight[]);
char* Deactivate_BC_func(char myRight[]);
void setLED(int val);



/******************************************************************************
* Global variables
******************************************************************************/
char Str_Buffer[BuffSize] = "";           // some usefull buffer for the functions
char* ACP_Filename = "";
char* LastCommand = "";
char pio7_value = '0';
char led1_value = '0';
int if_func_result = -1;
int BeckModule = -1;
int BC440_As[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int BC440_Act_Buffer = 0;
int SwitchValue = 0;
static unsigned char val602; // the value written to address 0x602
rpCgiPtr CgiRequestGlobal;

// define version number here! 1st the version and then the date
char prg_ver[] = "1.3.3 09-02-04";

static CGI_Entry   cgi_entry;
static char *charbfr = NULL;



/******************************************************************************
*******************************************************************************
* Functions for BC440
*******************************************************************************
******************************************************************************/



/******************************************************************************
* setLED()
******************************************************************************/
void setLED(int val) // set LED. 0: off, 1: green, 2: Red, 3: Yellow
{
   val602 = val;
   outportb(0x602, val602);
}



/******************************************************************************
* Activate_BC_func
******************************************************************************/
char* Activate_BC_func(char myRight[])
{
   int i = 0;
   int j = 0;
   int act_buff = 0;
   int counts = 0;
   int nb_count = 0;
   int new_i = 0;
   int strl = strlen(myRight);

   char num_buff[2] = "";
   strcpy(Str_Buffer, "");

   if (strcmp(LastCommand, myRight) == 0) return 0;
   else strcpy(LastCommand, myRight);

   if (myRight[1] == 'L')
   {
      if ((myRight[2]-'0' >= 1) && (myRight[2]-'0' <= 3))
         setLED(myRight[2]-'0');
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   if (myRight[1] == '+')
      counts++;

   do
   {
      counts++;
      if (myRight[counts] != ',')
      {
         num_buff[nb_count] += myRight[counts];
         nb_count++;
      }

      if ((myRight[counts] == ',') || (counts == strl))
      {
         i = atoi(num_buff);
         new_i = 1;
         for (j=0; j<i; j++) new_i *= 2;
         act_buff += new_i;
         strcpy(num_buff, "");
         nb_count = 0;
         BC440_As[i] = 1;
      }
   } while (counts <= strl);


   if (myRight[1] == '+')
      BC440_Act_Buffer += act_buff;
   else
   {
      outportb(0x601, 0);
      BC440_Act_Buffer = act_buff;
   }

   outportb(0x601, BC440_Act_Buffer);
   strcpy(Str_Buffer, "1");
   return(Str_Buffer);
}



/******************************************************************************
* Deactivate_BC_func()
******************************************************************************/
char* Deactivate_BC_func(char myRight[])
{
   int i = 0;
   int j = 0;
   int deact_buff = 0;
   int counts = 0;
   int nb_count = 0;
   int new_i = 0;
   int strl = 0;
   char num_buff[2] = "";
   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   if (strcmp(LastCommand, myRight) == 0) return 0;
   else strcpy(LastCommand, myRight);

   if (myRight[1] == 'L')
   {
      setLED(myRight[2]-'0');
      val602 = 0;
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   do
   {
      counts++;
      if (myRight[counts] != ',')
      {
         num_buff[nb_count] += myRight[counts];
         nb_count++;
      }

      if ((myRight[counts] == ',') || (counts == strl))
      {
         i = atoi(num_buff);
         if (BC440_As[i] == 1)
         {
            BC440_As[i] = 0;
            new_i = 1;
            for (j=0; j<i; j++) new_i *= 2;
            deact_buff += new_i;
            strcpy(num_buff, "");
            nb_count = 0;
         }
      }
   } while (counts <= strl);

   BC440_Act_Buffer -= deact_buff;
   outportb(0x601, BC440_Act_Buffer);
   strcpy(Str_Buffer, "1");
   return(Str_Buffer);
}



/******************************************************************************
* Input_BC_func()
******************************************************************************/
char* Input_BC_func(char myRight[])
{
   int i = 0;
   int n = 0;
   int m = 0;
   int new_i = 0;

   strcpy(Str_Buffer, "");

   if (myRight[1] == 'L')
   {
      itoa(val602, Str_Buffer, 10);
      return(Str_Buffer);
   }

   if (myRight[1] == 'S')
   {
      outportb(0x602, 4);
      itoa(inportb(0x602) - 16, Str_Buffer, 10);
      setLED(val602);
      return(Str_Buffer);
   }

   if ((myRight[1] == 'W') && (myRight[2]-'E' == 0) && (myRight[3] == '0'))
   {
      itoa(inportb(0x600), Str_Buffer, 10);
      return(Str_Buffer);
   }

   if ((myRight[1] == 'W') && (myRight[2]-'E' == 0) && (myRight[3] == '1'))
   {
      i = inportb(0x603);
      if (hal_read_pio(0) == 0) i += 16;
      if (hal_read_pio(1) == 0) i += 32;
      if (hal_read_pio(4) == 0) i += 64;
      if (hal_read_pio(3) == 0) i += 128;

      itoa(i, Str_Buffer, 10);
      return(Str_Buffer);
   }

   if ((myRight[1] == 'W') && (myRight[2]-'A' == 0) && (myRight[3] == '0'))
   {
      for (i=0; i<8; i++)
      {
         if (i == 0)
         {
            if (BC440_As[i] == 1) m = 1;
         }
         else
         {
            if (BC440_As[i] == 1)
            {
               new_i = 1;
               for (n=0; n<i; n++) new_i *= 2;
               m += new_i;
            }
         }
      }

      itoa(m, Str_Buffer, 10);
      return(Str_Buffer);
   }

   if ((myRight[1] == 'A') && (myRight[2]-'0' == 0) && (myRight[3] == '.'))
   {
      i = myRight[4]-'0';

      if ((i >= 0) && (i <= 7))
      {
         if (BC440_As[i] == 0) strcpy(Str_Buffer , "0");
         else                  strcpy(Str_Buffer , "1");
      }
   }

   if ((myRight[1] == 'E') && (myRight[2]-'0' == 0) && (myRight[3] == '.'))
   {
      i = myRight[4]-'0';

      if ((i >= 0) && (i <= 7))
      {
         n = inportb(0x600) & (1<<i);
         if (n >= 0) Str_Buffer[0] = n ? '1' : '0';
         else        Str_Buffer[0] = 'x';
         Str_Buffer[1]=0;
      }
   }

   if ((myRight[1] == 'E') && (myRight[2]-'0' == 1) && (myRight[3] == '.'))
   {
      i = myRight[4]-'0';

      if ((i >= 0) && (i <= 3))
      {
         n = inportb(0x603) & (1<<i);
         if (n >= 0) Str_Buffer[0] = n ? '1' : '0';
         else        Str_Buffer[0] = 'x';
         Str_Buffer[1]=0;
      }
      else if ((i >= 4) && (i <= 7))
      {
         if (i == 6) n = 4;
         else        n = i - 4;

         if (hal_read_pio(n) == 0) strcpy(Str_Buffer, "1");
         else                      strcpy(Str_Buffer, "0");
      }
   }
   if (strlen(Str_Buffer) == 0) strcpy(Str_Buffer, "x");
   return(Str_Buffer);
}



/******************************************************************************
*******************************************************************************
* Functions for DK40
*******************************************************************************
******************************************************************************/



/******************************************************************************
* Write_func()
******************************************************************************/
char* Write_func(char myRight[])
{
   int cpy = 0;
   int n = 0;
   int i = 0;
   int strl = 0;

   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   for (i=0; i<strl; i++)
   {
      if (cpy == 1)
      {
         if (myRight[i] != '\"')
         {
            Str_Buffer[n] = myRight[i];
            n++;
         }
         else
            cpy = 0;
      }

      if (myRight[i] == '\"') cpy = 1;
   }
   Str_Buffer[n] = '\0';
   return(Str_Buffer);
}



/******************************************************************************
* Then_Else_func()
******************************************************************************/
char* Then_Else_func(char myRight[])
{
   int i = 0;
   int strl = 0;
   char term_to_do[BuffSize] = "";

   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   for (i=5; i<strl; i++)
      term_to_do[i-5] = myRight[i];

   if ((myRight[0] != 'T') && (if_func_result == 1))
      strcpy(Str_Buffer, InterpretRight_func(term_to_do, 0));

   if ((myRight[0] != 'E') && (if_func_result == 0))
      strcpy(Str_Buffer, InterpretRight_func(term_to_do, 0));

   if (if_func_result == -1)
      strcpy(Str_Buffer, "Argument can only be used after an if-clause function!");

   return(Str_Buffer);
}



/******************************************************************************
* Input_func()
******************************************************************************/
char* Input_func(char myRight[])
{
   int i = 0;
   int n = 0;

   strcpy(Str_Buffer, "");

   if (myRight[1] == 'W')                            // input word is requested
   {
      itoa(inportb(0x600), Str_Buffer, 10);          // show in decimal
      return(Str_Buffer);
   }
   if (myRight[1] == 'L')                            // get LED1 status
   {
      Str_Buffer[0] = led1_value;
      Str_Buffer[1] = 0;
   }
   else                                              // input bit is requested
   {
      i = myRight[1]-'0';
      if ((i >= 0) && (i <= 7))
         n = inportb(0x600) & (1<<i);
      else n = -1;                                   // invalid

      if (n >= 0)
         Str_Buffer[0] = n ? '1' : '0';              // 1 or 0
      else
         Str_Buffer[0] = 'x';                        // invalid
      if (i == 7) Str_Buffer[0] = pio7_value;

      Str_Buffer[1]=0;
   }

   return(Str_Buffer);
}



/******************************************************************************
* Activate_func()
******************************************************************************/
char* Activate_func(char myRight[])
{
   int i = 0;
   int j = 0;
   int act_buff = 0;
   int counts = 0;
   int nb_count = 0;
   int new_i = 0;
   int strl = 0;
   char num_buff[2] = "";

   
   if (strcmp(LastCommand, myRight) == 0) return 0;
   else strcpy(LastCommand, myRight);

   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   if (myRight[1] == '+')
      counts++;

   if (myRight[counts+1] == 'L')                         // activate pio3
   {
      hal_write_pio( 3, 0 );
      led1_value = '1';
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   if (myRight[counts+1] == '7')                         // activate pio13
   {
      hal_write_pio( 13, 1 );
      pio7_value = '1';
      outportb(0x600, inportb(0x600));
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   do
   {
      counts++;
      if (myRight[counts] != ',')
      {
         num_buff[nb_count] += myRight[counts];
         nb_count++;
      }

      if ((myRight[counts] == ',') || (counts == strl))
      {
         i = atoi(num_buff);
         new_i = 1;
         for (j=0; j<i; j++) new_i *= 2;
         act_buff += new_i;
         strcpy(num_buff, "");
         nb_count = 0;
      }
   } while (counts <= strl);


   if (myRight[1] == '+')
      if (act_buff != inportb(0x600))
         act_buff += inportb(0x600);
   else outportb(0x600, 0);

   outportb(0x600, act_buff);
   strcpy(Str_Buffer, "1");
   return(Str_Buffer);
}



/******************************************************************************
* Deacivate_func()
******************************************************************************/
char* Deactivate_func(char myRight[])
{
   int i = 0;
   int j = 0;
   int deact_buff = 0;
   int counts = 0;
   int new_i = 0;
   int nb_count = 0;
   int strl = 0;
   char num_buff[2] = "";

   
   if (strcmp(LastCommand, myRight) == 0) return 0;
   else strcpy(LastCommand, myRight);

   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   if (myRight[1] == 'L')
   {
      hal_write_pio( 3, 1 );
      led1_value = '0';
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   if (myRight[1]-'0' == 7)
   {
      hal_write_pio( 13, 0 );
      pio7_value = '0';
      strcpy(Str_Buffer, "1");
      return(Str_Buffer);
   }

   deact_buff = inportb(0x600);

   do
   {
      counts++;
      if (myRight[counts] != ',')
      {
         num_buff[nb_count] += myRight[counts];
         nb_count++;
      }

      if ((myRight[counts] == ',') || (counts == strl))
      {
         i = atoi(num_buff);
         new_i = 1;
         for (j=0; j<i; j++) new_i *= 2;
         deact_buff -= new_i;
         strcpy(num_buff, "");
         nb_count = 0;
      }
   } while (counts <= strl);

   if (deact_buff >= 0) outportb(0x600, deact_buff);

   strcpy(Str_Buffer, "1");
   return(Str_Buffer);
}



/******************************************************************************
* If_func()
******************************************************************************/
char* If_func(char myRight[])
{
   int n = 0;
   int i = 0;
   int a_ = 0;
   int b_ = 0;
   int c_ = 0;
   int cmpres = 0;
   int f_equal = 0;
   int then_else_ok = -1;
   int strl = 0;

   char term1[15] = "";
   char term2[15] = "";
   char term3[BuffSize - 30] = "";

   char term1_ok[15] = "";
   char term2_ok[15] = "";

   strcpy(Str_Buffer, "");
   strl = strlen(myRight);

   for (i=0; i<strl; i++)
   {
      if (myRight[i] == '(') a_ = i;
      if (f_equal == 0)
         if (myRight[i] == '=')
         {
            b_ = i;
            f_equal = 1;
         }

      if (myRight[i] == ')')
      {
         c_ = i;
         if (strl > c_ + 5)
         {
            if (myRight[i+2] == 'T') then_else_ok = 1;
            if (myRight[i+2] == 'E') then_else_ok = 2;
         }
      }
   }

   // copy term1 from the if-clause to a variable
   n = 0;
   i = a_ + 1;
   do
   {
      term1[n] = myRight[i];
      n++;
      i++;
   } while (i < b_);
   term1[n] = '\0';

   // copy term2 from the if-clause to a variable
   n = 0;
   i = b_ + 2;
   do
   {
      term2[n] = myRight[i];
      n++;
      i++;
   } while (i < c_);
   term2[n] = '\0';

   // copy term3 from the if-clause to a variable
   n = 0;
   i = c_ + 2 + 5;
   do
   {
      term3[n] = myRight[i];
      n++;
      i++;
   } while (i < strl);
   term3[n] = '\0';

   strcpy(term1_ok, InterpretRight_func(term1, 1));
   strcpy(term2_ok, InterpretRight_func(term2, 1));

   strl = strlen(term2_ok);

   if (term2_ok[0] == '\"')
   {
      for (i=0; i<strl-1; i++)
         term2_ok[i] = term2_ok[i+1];
      term2_ok[strl-2] = '\0';
   }

   cmpres = strcmp(term1_ok, term2_ok);

   if (cmpres == 0)
   {
      if_func_result = 0;
      if (then_else_ok == 1) strcpy(Str_Buffer, InterpretRight_func(term3, 0));
      else strcpy(Str_Buffer, "");
   }

   if (cmpres != 0)
   {
      if_func_result = 1;
      if (then_else_ok == 2) strcpy(Str_Buffer, InterpretRight_func(term3, 0));
      else strcpy(Str_Buffer, "");
   }

   if (then_else_ok == 0) strcpy(Str_Buffer, "");

   return(Str_Buffer);
}



/******************************************************************************
* GetParams_func()
******************************************************************************/
char* GetParams_func(char myRight[])
{
   char Parameters[10][30];
   int paramlen = 0;
   int m = 0;
   int n = 0;
   int i = 0;
   int paramwant = myRight[3]-'0';
   int strl = 0;

   strcpy(Str_Buffer, "");
   strl = strlen(ACP_Filename);

   for (i=0; i<10; i++)
      strcpy(Parameters[i], "0");

   for (i=0; i<strl; i++)
   {
      if (ACP_Filename[i] == '?')
      {
         paramlen++;       // counts parameters
         do
         {
            i++;
            if (ACP_Filename[i] != '&') Parameters[n][m] = ACP_Filename[i];
            m++;
            if (ACP_Filename[i] == '&')
            {
               Parameters[n][m] = '\0';
               paramlen++;
               m = 0;
               n++;
            }
         } while (i <= strl);
         Parameters[n][m] = '\0';
      }
   }

   if ((paramwant > 0) && (paramwant <= 10))
      strcpy(Str_Buffer, Parameters[paramwant-1]);
   else
      strcpy(Str_Buffer, "Invalid parameter request!");
   return(Str_Buffer);
}



/******************************************************************************
* InterpretRight_func()
******************************************************************************/
char* InterpretRight_func(char myRight[], int ifclause)
{
   strcpy(Str_Buffer, "");

   switch(myRight[0])
   {
      case 'A':                                                // activate
         if (BeckModule == 0)
               strcpy(Str_Buffer, Activate_func(myRight));
            else
               strcpy(Str_Buffer, Activate_BC_func(myRight));
         break;

      case 'D':                                                // deactivate
         if (BeckModule == 0)
               strcpy(Str_Buffer, Deactivate_func(myRight));
            else
               strcpy(Str_Buffer, Deactivate_BC_func(myRight));
         break;

      case 'E':                                                // else
         if ((myRight[1] == 'L') && (myRight[2] == 'S'))
            strcpy(Str_Buffer, Then_Else_func(myRight));
         break;

      case 'P':                                                // parameters
         if ((myRight[1] == 'A') && (myRight[2] == 'R'))
            strcpy(Str_Buffer, GetParams_func(myRight));
         break;

      case 'I':
         if (myRight[1] == 'F')                                // if-clause
            strcpy(Str_Buffer, If_func(myRight));
         else                                                  // input
            if (BeckModule == 0)
               strcpy(Str_Buffer, Input_func(myRight));
            else
               strcpy(Str_Buffer, Input_BC_func(myRight));
         break;

      case 'T':                                                // then
         if ((myRight[1] == 'H') && (myRight[2] == 'E'))
            strcpy(Str_Buffer, Then_Else_func(myRight));
         break;

       case 'V':                                               // version
         if ((myRight[1] == 'E') && (myRight[2] == 'R'))
            strcpy(Str_Buffer, prg_ver);
         break;

       case 'W':                                               // write
         if ((myRight[1] == 'R') && (myRight[2] == 'I'))
            strcpy(Str_Buffer, Write_func(myRight));
         break;

       default:                                                // invalid
         if (ifclause == 0)
            strcpy(Str_Buffer, "Invalid right hand");
         else
            strcpy(Str_Buffer, myRight);
         break;
      }
      return(Str_Buffer);
}



/******************************************************************************
*******************************************************************************
* Interpreter- and CGI-function
*******************************************************************************
******************************************************************************/



/******************************************************************************
* acpInterpreter()
******************************************************************************/
int near acpInterpreter(char *target, char *source)
{
   char left[120], right[BuffSize];  // what is originally left and right of '=' char
   char newRight[BuffSize];          // interpreted string at the right example: "I1" becomes "1"
   // Warning: Range checking is still missing !!!
   char *pch;
   int  i,n;                    // some integer variables


   target[0]=0;

   n=strlen(source);
   if (n<3) return 0;                              // can not be serious

   pch=strchr(source,'=');                         // find the '='
   if (!pch) return 0;

   // find the string at the left of the '=' char
   n=i=0;
   while (source[i]==32) i++;                      // skip spaces
   while ((source[i]!='=') && (source[i]!=0))
   {
      left[n++]=source[i++];
   }
   while ((left[n-1]==32) && (n>0)) n--;           // remove trailing spaces
   left[n]=0;                                      // terminate string

   // find the string at the right of the '=' char
   if (source[i]=='=') i++;                        // skip the '='
   while (source[i]==32) i++;                      // skip spaces
   strcpy(right,&source[i]);
   n=strlen(right);
   while ((right[n-1]==32) && (n>0)) n--;          // remove trailing spaces
   right[n]=0;                                     // terminate string

   strcpy(newRight, InterpretRight_func(right, 0));   // interpret the right hand

   // now interpret the left
   // currently, only the empty string is supported
   switch(left[0])
   {
      case 0:                                      // empty string
         strcpy(target, newRight);
         break;
      case '!':                                    // don't show return result
         strcpy(target, "");
         break;
      default:                                     // invalid
         strcpy(target,"Invalid left hand");
         break;
   }
   return strlen(target);
}



/******************************************************************************
* ACP_Func()
******************************************************************************/
void huge _pascal ACP_Func(rpCgiPtr CgiRequest)
{
   FILE *f;
   int i, n, j, stop;
   char ch1, ch2, acpBfr[BuffSize];  // warning: must add range checking !!!
   char *WebRoot = "";
   char CheckRoot[BuffSize] = "";

   
   // Make CGI Structure global in order to call the function without a file browsing
   // CgiRequestGlobal = CgiRequest;

   // set if function settings to start value
   if_func_result = -1;

   // check arguments
   if(CgiRequest->fArgumentBufferLength==0L)  // no arguments
   {
      //we send the page
      CgiRequest->fHttpResponse         = CgiHttpOk;
      CgiRequest->fResponseBufferPtr    = "missing argument";
      CgiRequest->fResponseBufferLength = strlen(CgiRequest->fResponseBufferPtr);
      return;
   }

   // fill global variable with the filename (parameters enclosed)
   ACP_Filename = CgiRequest->fArgumentBufferPtr;

   // first check for slashes and replace them with backslashes!
   j = strlen(ACP_Filename);
   for (i=0; i<j; i++)
      if (ACP_Filename[i] == '/') ACP_Filename[i] = '\\';
   ACP_Filename[strlen(ACP_Filename)] = '\0';

   // Get Web Root Directory
   strcpy(WebRoot, "");

   CGI_GetRootDir ( &WebRoot );

   j = 0;
   n = strlen(WebRoot);
   for (i=0; i<n; i++)
   {
      if (WebRoot[i] == ':') j = 1;
   }

   if (j == 0)
   {
      // Get Current Drive Letter
      Str_Buffer[0] = 'A' + getdisk();
      strcat(Str_Buffer, ":\\");
      strcat(Str_Buffer, WebRoot);
      strcpy(WebRoot, Str_Buffer);
   }

   // Security check! Only use filenames in the WebRoot directory or above
   strcpy(Str_Buffer, "");

   n = strlen(ACP_Filename);
   j = -1;
   for (i=0; i<n; i++)
   {
      if (ACP_Filename[i] == '\\')
      {
         if (i > 0)
         {
            if (ACP_Filename[i - 1] == '.') j = i + 1;
         }
         else if (i == 0) j = i + 1;
      }
   }

   if (j > -1)
      for (i=0; i<n; i++)
         Str_Buffer[i] = ACP_Filename[i + j];
   else
      strcpy(Str_Buffer, ACP_Filename);

   strcpy(CheckRoot, WebRoot);
   strcat(CheckRoot, "\\");
   strcat(CheckRoot, Str_Buffer);
   WebRoot[strlen(CheckRoot)] = '\0';

   // release old buffer if needed
   helper_release_rtos_mem( FP_SEG (charbfr ));
   charbfr = NULL;


//   printf("\nACP Filename: %s", CheckRoot);         //Some debug information


   f = fopen(CheckRoot,"rb");
   if (!f) // open failed
   {
      //we send the page
      CgiRequest->fHttpResponse         = CgiHttpOk;
      CgiRequest->fResponseBufferPtr    = "File open failed";
      CgiRequest->fResponseBufferLength = strlen(CgiRequest->fResponseBufferPtr);
      return;
   }

   // how long should the buffer be ?
   n = (int)filelength(fileno(f));
   charbfr = MK_FP( helper_alloc_rtos_mem  (n+1024), 0 ); // add some extra
   if (!charbfr)
   {
      CgiRequest->fHttpResponse         = CgiHttpOk;
      CgiRequest->fResponseBufferPtr    = "Not enough memory";
      CgiRequest->fResponseBufferLength = strlen(CgiRequest->fResponseBufferPtr);
      fclose(f);
      return;
   }

   // now read char by char, checking for <%
   n=0;
   while (!feof(f))
   {
      if (1==fread(&ch1,1,1,f))
      {
         if (ch1=='<')
         {
            fread(&ch2,1,1,f);
            if (ch2=='%')      // we found a '<%' sequence !!!
            {
               stop = i = 0;
            do
            {
               fread(&ch1,1,1,f);
               if (ch1=='%')
               {
                  fread(&ch2,1,1,f);
                  if (ch2=='>') stop = 1;      // we found the '%>'
                  else
                  {
                     acpBfr[i++] = ch1;
                     acpBfr[i++] = ch2;
                  }
               }
               else acpBfr[i++] = ch1;
            } while (!stop);
            acpBfr[i] = 0;
            n += acpInterpreter(&charbfr[n],acpBfr);
         }
         else // started with '<' but no '%' followed
         {
            charbfr[n++] = ch1;
            charbfr[n++] = ch2;
         }
         }
         else charbfr[n++] = ch1;      // all other chars, add to output buffer
      }
   } // eof
   charbfr[n] = 0;                     // terminate string

   fclose(f);
   CgiRequest->fHttpResponse         = CgiHttpOk;
   CgiRequest->fResponseBufferPtr    = charbfr;
   CgiRequest->fResponseBufferLength = n;

   fclose(f);
}



/******************************************************************************
* acp_install()
******************************************************************************/
int acp_install(char Par[], char Par2[])
{

   if (strcmp(Par, "dk40") == 0)
   {
      printf("\nACP Interpreter for DK40 activated!\n");
      BeckModule = 0;
   }

   if (strcmp(Par, "bc440") == 0)
   {
      printf("\nACP Interpreter for BC440-WEB activated!\n");
      BeckModule = 1;
   }

   if (BeckModule == -1)
   {
      printf("\nWrong parameter!\n\n");
      printf("\nusage: beck_acp.exe <device>");
      printf("\ndevice = dk40 or bc440\n");
      exit(1);
   }

   // enable hardware
   pfe_enable_pcs(6);

   if (BeckModule == 1)                   // BC440 settings
   {
      pfe_enable_pio(0, 1);
      pfe_enable_pio(1, 1);
      pfe_enable_pio(3, 1);
      pfe_enable_pio(4, 1);
      pfe_enable_bus(0xFFFF, 1);
      outportb(0x601, 0);
   }
   else                                    // DK40 settings
   {
      pfe_enable_pio(13,5);                // use pio 13 (input 7 on DK40) as output
      pfe_enable_pio(3,4);                 // use pio 03 (LED1) as output
   }

   // set up cgi handler
   cgi_entry.PathPtr    = Par2;            // name of the page, case sensitive !!!
   cgi_entry.method     = CgiHttpGet;      // http method
   cgi_entry.CgiFuncPtr = ACP_Func;        // function

   return CGI_Install(&cgi_entry);
}



// End of file

