/****************************************************************************
*
* (C) 1999-2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module      : CGIAPI.C
* Function    : cgi api functions
* ---------------------------------------------------------------------------

$Header: CGIAPI.C, 7, 08.03.2004 18:01:26, Andre Pribil$

*****************************************************************************/

#include <STDLIB.H>
#include <DOS.H>
#include "CGIAPI.H"

/*************************************************************************/

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 CGI API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces CGIAPI\.H><efont> |

 @libcat1  CGI | cgi01 | CGI_Install | Install a CGI function |
 @libcat1  CGI | cgi02 | CGI_Delete | Remove a CGI function |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI | cgi03 | CGI_SetMainPage | Select a new main page |
 @libcat1  CGI | cgi06 | CGI_GetMainPage | Get main page name |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI | cgi04 | CGI_SetRootDir | Set Web server's root directory |
 @libcat1  CGI | cgi05 | CGI_GetRootDir | Get Web server's root directory |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI |  |  |  |
 @libcat1  CGI | cgi07 | CGI_GetFormItem | Split a formular into name and value |
 @libcat1  CGI | cgi08 | CGI_GetNextItem | Return pointer to the next formular tag |

*/

#define CGIVECT       0xAB
/*************************************************************************/
//install cgi function
/*************************************************************************/
#ifdef _cgi01_
/*
 @doc DOC_cgi01_
 ---------------------------------------------------------------
 Install a CGI function
 ---------------------------------------------------------------
 @clib  CGI_Install | Install a CGI function. |
        <cfont>int CGI_Install ( CGI_Entry far  *cgiptr );<efont>  |
 @parm  int  | cgiptr  | Pointer to a temporary <code>CGI_Entry<codee>
        <linkto>../../cgistruc.htm#CGI_Entry<linkmid>type<linktail> structure.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_NO_FREE_ENTRY or CGI_INVALID_METHOD
 @comm  This API function makes a copy of the information in the
        provided <code>CGI_Entry<codee> structure, so your structure
        at <code>cgiptr<codee> need not be persistent.
 @Also  cgi02.htm | CGI_Delete  | | | |
 @related <cfont>cgistat<codee> | ../../command.htm#cgistat | command | line
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x01<linkmid>documentation<linktail>
        for more details.
*/
int CGI_Install (CGI_Entry far  * cgiptr)
{
    _DX = FP_SEG(cgiptr);
    _SI = FP_OFF(cgiptr);

    _AH = CGI_INSTALL;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif

/*************************************************************************/
// delete cgi function
/*************************************************************************/
#ifdef _cgi02_
/*
 @doc DOC_cgi02_
 ---------------------------------------------------------------
 Remove a CGI function
 ---------------------------------------------------------------
 @clib  CGI_Delete | Remove a CGI function. |
        <cfont>int CGI_Delete ( char far * PathPtr );<efont>  |
 @parm  int  | PathPtr  | Pointer to the null terminated URL path name.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_NOT_FOUND
 @comm  The CGI function to be deleted is identified by the provided
        URL string.<nl>
        <nl>
        It is also possible to remove the two predefined
        cgi functions main.htm and ChipCfg from the table.<nl>
 @Also  cgi01.htm | CGI_Install  |
        cgi03.htm | CGI_SetMainPage | |
 @related <cfont>cgistat<codee> | ../../command.htm#cgistat | command | line
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x02<linkmid>documentation<linktail>
        for more details.
*/
int CGI_Delete (char far * PathPtr)
{
    _DX = FP_SEG(PathPtr);
    _SI = FP_OFF(PathPtr);

    _AH = CGI_REMOVE;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
//Set main page
/*************************************************************************/
#ifdef _cgi03_
/*
 @doc DOC_cgi03_
 ---------------------------------------------------------------
 Set a new main page
 ---------------------------------------------------------------
 @clib  CGI_SetMainPage | Set a new main page. |
        <cfont>int CGI_SetMainPage ( const char far *mainPageName );<efont> |
 @parm  int  | mainPageName  | Pointer to name of new main page.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_INVALID_NAME
 @comm  The string at <code>mainPageName<codee> is null terminated
        with a maximum length of 64 characters (not counting the
        terminating zero).<nl>
 @Also  cgi06.htm | CGI_GetMainPage  |
        cgi04.htm | CGI_SetRootDir   | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x03<linkmid>documentation<linktail>
        for more details.
*/
int CGI_SetMainPage (const char far *mainPageName)
{
    _DX = FP_SEG(mainPageName);
    _SI = FP_OFF(mainPageName);

    _AH = CGI_SETMAIN;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
//Set root dir
/*************************************************************************/
#ifdef _cgi04_
/*
 @doc DOC_cgi04_
 ---------------------------------------------------------------
 Set Web server's root directory
 ---------------------------------------------------------------
 @clib  CGI_SetRootDir | Set Web server's root directory. |
        <cfont>int CGI_SetRootDir ( const char far *rootDirName );<efont> |
 @parm  int  | rootDirName  | Pointer to the name of new root directory
        for Web server.
 @rdesc 0:  success <nl>
        non-zero:  Failure, return value contains error code
        CGI_INVALID_DIR
 @comm  The string at <code>rootDirName<codee> is null terminated with a maximum
        length of 64 characters (not counting the terminating zero).
 @Also  cgi05.htm | CGI_GetRootDir  |
        cgi03.htm | CGI_SetMainPage | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x04<linkmid>documentation<linktail>
        for more details.
*/
int CGI_SetRootDir (const char far *rootDirName)
{
    _DX = FP_SEG(rootDirName);
    _SI = FP_OFF(rootDirName);

    _AH = CGI_SETROOTDIR;
    asm { int CGIVECT  } ;

    return _AX ;
}
#endif


/*************************************************************************/
// Get root dir
/*************************************************************************/
#ifdef _cgi05_
/*
 @doc DOC_cgi05_
 ---------------------------------------------------------------
 Get Web server's root directory
 ---------------------------------------------------------------
 @clib  CGI_GetRootDir | Get Web server's root directory. |
        <cfont>int CGI_GetRootDir ( char far **rootDirName );<efont> |
 @parm  char far **  | rootDirName  | Contains pointer to root directory name.
 @rdesc 0:<tab>The variable rootDirName contains a pointer to the Web server's root directory name.
 @comm  The referenced root directory name string is null terminated and is in
        the  RTOS's data space.<nl>
        <nl>
        The caller must treat this string as a
        constant and must not modify it.
 @Also  cgi04.htm | CGI_SetRootDir  |
        cgi06.htm | CGI_GetMainPage | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x05<linkmid>documentation<linktail>
        for more details.
*/
int CGI_GetRootDir(char far **rootDirName)
{
    unsigned int tmp_es;

    _AH = CGI_GETROOTDIR;
    asm { int CGIVECT  } ;

    tmp_es = _ES;

    *rootDirName = (char far *)MK_FP( tmp_es, _DI );

    return _DX;
}
#endif



/*************************************************************************/
//Get main page name
/*************************************************************************/
#ifdef _cgi06_
/*
 @doc DOC_cgi06_
 ---------------------------------------------------------------
 Get main page name
 ---------------------------------------------------------------
 @clib  CGI_GetMainPage | Get main page name. |
        <cfont>int CGI_GetMainPage ( char far **mainPageName );<efont> |
 @parm  char far **  | mainPageName  | Contains pointer to current main page name.
 @rdesc 0:<tab>The variable mainPageName contains a pointer to a string containing name of current main page
 @comm  The referenced main page name string is null terminated and is in
        the  RTOS's data space.<nl>
        <nl>
        The caller must treat this string as a constant and must not modify it.
 @Also  cgi03.htm | CGI_SetMainPage |
        cgi05.htm | CGI_GetRootDir  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x06<linkmid>documentation<linktail>
        for more details.
*/
int CGI_GetMainPage(char far **mainPageName)
{
   unsigned int tmp_es;

   _AH = CGI_GETMAIN;
   asm { int CGIVECT  } ;

   tmp_es = _ES;

	*mainPageName = (char far *)MK_FP( tmp_es, _DI );

   return _DX;
}
#endif

/*************************************************************************/
// Get formitem
/*************************************************************************/
#ifdef _cgi07_
/*
 @doc DOC_cgi07_
 ---------------------------------------------------------------
 Split a formular into name and value
 ---------------------------------------------------------------
 @clib  CGI_GetFormItem | Split a formular into name and value. |
        <cfont>void CGI_GetFormItem ( const char far *ArgumentBufferPtr,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp; FormItem far *formitem );<efont> |
 @parm  int  | ArgumentBufferPtr  | Input Parameter:<s2>
        Pointer to argument buffer to  be parsed.
 @parm  int  | formitem  | Input/Output Parameters:<s2>
        Pointer to a <code>FormItem<codee>
        <linkto>../../cgistruc.htm#FormItem<linkmid>type<linktail> structure
 @rdesc  -- none --
 @comm  On initial call, the <code>ArgumentBufferPtr<codee> provided by the caller
        is a copy of the <code>fArgumentBufferPtr<codee> member of
        the <code>rbCgi<codee>
        <linkto>../../cgistruc.htm#rpCgi<linkmid>structure<linktail> passed by the
        Web server to the CGI callback function.<s2> On subsequent calls here
        to pick up additional formular, the pointer returned from the
        <linkto>cgi08.htm<linkmid>CGI_GetNextItem<linktail>
        function can be used here.<nl>
        <nl>
        Prior to calling the two members of the <code>FormItem<codee>
        <linkto>../../cgistruc.htm#FormItem<linkmid>structure<linktail>
        must be set by the caller.<s2>
        Both member pointers in the <code>FormItem<codee> data structure
        reference buffers allocated by the
        user, which will receive strings produced by this API call.<nl>
 @Also  cgi08.htm | CGI_GetNextItem  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x07<linkmid>documentation<linktail>
        for more details.
*/
void CGI_GetFormItem (const char far * ArgumentBufferPtr,
                      FormItem far * formitem)
{
    _BX = FP_SEG(ArgumentBufferPtr);
    _SI = FP_OFF(ArgumentBufferPtr);

    _ES = FP_SEG(formitem);
    _DI = FP_OFF(formitem);

    _AH = CGI_GETFORMITEM;
    asm { int CGIVECT  } ;
}
#endif

/*************************************************************************/
//Get next formitem
/*************************************************************************/
#ifdef _cgi08_
/*
 @doc DOC_cgi08_
 ---------------------------------------------------------------
 Return pointer to the next formular tag
 ---------------------------------------------------------------
 @clib  CGI_GetNextItem | Get the next item formular tag. |
        <cfont>int CGI_GetNextItem ( char far *ArgumentBufferPtr,<nl>
        <tab><tab><tab><tab><tab>&nbsp;&nbsp;char far **nextItemString );<efont> |
 @parm  char far *  | ArgumentBufferPtr  | CGI request argument pointer.
 @parm  char far **  | nextItemString  | Pointer to the found item.
 @rdesc 0:<tab>Item was found<nl>
        -1:<tab>No more items found
 @comm  Most formulars have more than one item.<s2> This function searches for
        the next form item in a CGI request argument string.<s2> It can
        only be used after a
        <linkto>cgi07.htm<linkmid>CGI_GetFormItem<linktail> call.<nl>
        <nl>
        The string referenced by <code>ArgumentBufferPtr<codee> is not
        modified in any way by this function.
 @Also  cgi07.htm | CGI_GetFormItem  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../cgi.htm#0xAB0x08<linkmid>documentation<linktail>
        for more details.
*/
int CGI_GetNextItem (char far * ArgumentBufferPtr, char far **nextItemString)
{
    unsigned int tmp_es;

    _BX = FP_SEG(ArgumentBufferPtr);
    _SI = FP_OFF(ArgumentBufferPtr);

    _AH = CGI_FINDNEXTITEM;
    asm { int CGIVECT  } ;

    tmp_es = _ES;

    if (_DX == 0)
        *nextItemString = MK_FP( tmp_es, _DI );
    else
        *nextItemString = NULL;

    return _DX;
}
#endif


/*************************************************************************/
//end cgiapi.c