/****************************************************************************
*
* (C) 1999-2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module        : RTXAPI.H
* Function      : Constants, protoypes for RTX

$Header: RTXAPI.H, 5, 02.12.2003 15:32:28, Andre Pribil$

******************************************************************************/

#ifndef RTX_API_H_
#define RTX_API_H_

/**************************************************************************************************/
//returned common API errorcodes at DX register, special RTOS errorcodes are listed at the HTML-Docs
/**************************************************************************************************/
#define RTX_ENOERROR        0
#define RTX_ERROR          -1
#define RTX_NOT_SUPPORTED  -2


/*****************************************************************************/
//typedefs
/*****************************************************************************/
/* ID type for tasks ,semaphores */

typedef unsigned int RTOS_ID ;


typedef void huge (far *PROC_VECTOR)(void) ;

/*Task Definition Structure */
typedef struct tag_taskdefblock
{
        PROC_VECTOR   proc ;            /* task procedure pointer                   */
        char    name[4];                /* name 4 characters not null terminated    */
        unsigned int far * stackptr;    /* task stack pointer                       */
        unsigned int stacksize;         /* size of task stack (bytes)               */   
        unsigned short int attrib;      /* task attributes, not supported by the RTOS API   */
        short int priority;             /* task priority,        range: 20<=priority<=127   */
        unsigned short int time_slice;  /* 0: none, !=0: number of milliseconds before task force to relinquish processor */
        short int mailboxlevel1;        /* depth of mailbox i: 0 not used, else 1 - 8 mailboxdepth */
        short int mailboxlevel2;        /* max. number of messages resides inside a task mailbox   */
        short int mailboxlevel3;
        short int mailboxlevel4;
}TaskDefBlock;



typedef struct tagtasklist
{
     unsigned int taskID;         /*task identifier*/
     char taskname[5];            /* unique name 4 characters , 0 terminated */
}TaskList;



typedef struct tag_task_statedata{
        unsigned int taskID;
        unsigned int taskPrio;
        unsigned int taskState;     /*taskstate see below*/
        unsigned int taskCount;     /*count of the taskmonitor, if taskmon is active*/
        unsigned int stackused;     /*used stack in percent*/
        unsigned int stacksize;     /*whole stacksize*/
}Task_StateData;

/*taskstates (16Bit value)
Bit0 timer wait (used with other bits)
Bit1 trigger wait (i.e. idle)
Bit2 semaphore wait
Bit3 event group wait
Bit4 message exchange wait
Bit5 message send wait
Bit6 suspended (waiting for resume)
Bit7 waiting for wake
>Bit7 internal use only
*/


/*Time/Date Structure*/
typedef struct tag_time
{
        unsigned char sec;      /* seconds      (0-59)  */
        unsigned char min;      /* minutes      (0-59)  */
        unsigned char hr;       /* hours        (0-23)  */
        unsigned char dy;       /* day          (1-31)  */
        unsigned char mn;       /* month        (1-12)  */
        unsigned char yr;       /* year         (0-99)  */
        unsigned char dow;      /* day of week  (Mon=1 to Sun=7) */
        unsigned char dcen;     /* century if time/date is correct */
}TimeDate_Structure;

/*Timer procedure structure*/
typedef struct tag_timer_proc
{
   unsigned int  far * timerID;     /*pointer to storage the unique timerID*/
   PROC_VECTOR  proc ;              /*pointer to the procedure to be executed*/
   void  far   *dummyptr;           /*dummypointer, not used at the moment */
   char         name[4];            /*pointer to a unique 4 character name*/
   long         interval;           /*timer execution interval*/
}TimerProc_Structure;


/*Event manager: Wait for event structure*/
typedef struct tag_rtx_event_wait
{
  unsigned int mask;    // 16-Bit mask identifying the flags of interest of the group.
  unsigned int value;   // 16 Bit value, which specfies the states of interest
                        //  for each flag selected by the mask.
  int match;            // event match requirements, 0:only one flag must match with value,
                        // !=0: all by mask specified flags must match
  long timeout;         // Maximum time (milliseconds) for waiting for an event match
}RTX_Wait_Event;


/*Message exchange create structure*/
typedef struct tag_rtx_msg{
  unsigned int  msgID;  // for storage the unique message exchange ID
  char name[4];         // name 4 characters, not null terminated
  int  mb0;             // numbers of message envelopes, which can reside in
                        // each of the exchange mailbox, max. 8
  int  mb1;
  int  mb2;
  int  mb3;
}RTX_Msg;


/*Message exchange wait structure*/
typedef struct tag_rtx_wait_msg{
  unsigned int   msgID;         // ID of the message exchange
  int   prio;                   // priority for wait (0-3),  0=highest
  char far *msg;                // pointer ot userbuffer to storage the arrived message
  long timeout;                 // Maximum time (milliseconds) for waiting for a message
}RTX_Wait_Msg;




/*****************************************************************************/
#endif  /*RTX_API_H_*/
/*****************************************************************************/
//rtxapi.h
