/****************************************************************************
*
* (C) 2000-2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module         : TCPIP.C
* Function       : TCPIP API socket functions
* ---------------------------------------------------------------------------

$Header: c:\prj\IPC@CHIP\CLib\TCPIP\TCPIP.C, 22, 14.04.2004 13:30:34, Markus Bartat$

*****************************************************************************/

#include <STDLIB.H>
#include <DOS.H>
#include <TCPIPAPI.H>
#include <TCPIP.H>
#include "TCPPRIV.H"


/*************************************************************************/
//HTONS
/*************************************************************************/
#define HTONS(value)  ( (unsigned)(value<<8)|(unsigned)(value>>8) )

#define BIOS_VECT  (0xA0)       // Software Interrupt number for BIOS API
// Services

// TCPIP Services
#define TCPIPVECT  (0xAC)

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 TCP/IP API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces TCPIP\.H><efont><nl>
  <tab><tab><tab><tab><tab><arial>which includes<ariale> <cfont>
 <InBraces TCPIPAPI\.H><efont> | 
    <tab><linkto>#TCP<linkmid>TCP/IP<linktail><tab>
    <linkto>#Client<linkmid>PPP Client<linktail><tab>
    <linkto>#Server<linkmid>PPP Server<linktail><tab>
    <linkto>#Other<linkmid>Other<linktail><tab>
 @libcat1a  TCP | TCP/IP | tcpIPR | Get_IPConfig | Get IP, subnet mask and/or gateway |
 @libcat1a  TCP | TCP/IP | tcpIPW | Set_IPConfig | Set IP, subnet mask and/or gateway |
 @libcat1a  TCP | TCP/IP | tcpPSIp | PPP_Server_GetIP | Get IP Address of the PPP Server |
 @libcat1a  TCP | TCP/IP | tcpPCIp | PPP_Client_GetIP | Get IP Address of the PPP Client |
 @libcat1a  TCP | TCP/IP | tcp11 | InetToAscii | Convert an IP address to an IP string |
 @libcat1a  TCP | TCP/IP | tcp08 | inet_addr | Converts a string containing an IP address to dot notation  |
 @libcat1a  TCP | TCP/IP |   |   |  | <nl><tab><tab>--- Socket Functions ---
 @libcat1a  TCP | TCP/IP | tcp01  | opensocket   | Open TCP or UDP socket |
 @libcat1a  TCP | TCP/IP | tcp02  | closesocket  | Close socket |
 @libcat1a  TCP | TCP/IP | tcp03  | bind         | Associates a local address with a socket |
 @libcat1a  TCP | TCP/IP | tcp04  | connect | Establish a connection to a specifed socket |
 @libcat1a  TCP | TCP/IP | tcp0C | GetWaitingBytes | Report the number of bytes on a socket |
 @libcat1a  TCP | TCP/IP | tcp15 | Set_IP_Type_Of_Service | Set socket's Type-Of-Service |
 @libcat1a  TCP | TCP/IP | tcp16 | setsockopt | Set options on a socket |
 @libcat1a  TCP | TCP/IP | tcp17 | getsockopt | Get options on a socket |
 @libcat1a  TCP | TCP/IP | tcp18 | Set_Blocking_Mode | Set a socket in blocking or non-blocking mode |
 @libcat1a  TCP | TCP/IP | tcp19 | RegisterCallbackFunction | Register a user callback function |
 @libcat1a  TCP | TCP/IP | tcp21 | get_socketerror | Get last socket error |
 @libcat1a  TCP | TCP/IP |   |   |  | <nl><tab><tab>--- TCP/IP Sockets ---
 @libcat1a  TCP | TCP/IP | tcpcon | tcp_connect  | Open TCP/IP socket and connect |
 @libcat1a  TCP | TCP/IP | tcp0D | accept | Accept the next incoming connection |
 @libcat1a  TCP | TCP/IP | tcp0E | listen | Listen for incoming connections |
 @libcat1a  TCP | TCP/IP | tcphton | htons | Converts a 16 bit word from host to TCP/IP network byte order |
 @libcat1a  TCP | TCP/IP | tcp0F | send | Transmit a message (TCP) |
 @libcat1a  TCP | TCP/IP | tcp10 | recv | Receive message (TCP) |
 @libcat1a  TCP | TCP/IP | tcp12 | ResetConnection | Abort a connection on a socket |
 @libcat1a  TCP | TCP/IP | tcp13 | setlinger | Set linger time on close (TCP) |
 @libcat1a  TCP | TCP/IP | tcp14 | setreuse | Set reuse option on a listening socket |
 @libcat1a  TCP | TCP/IP | tcp22 | Get_TCP_Socket_State | Find TCP socket and return state |
 @libcat1a  TCP | TCP/IP | tcp23 | FindAllOpenSockets | Get information about all open sockets |

 @libcat1a  TCP | TCP/IP |   |   |  | <nl><tab><tab>--- UDP Sockets ---
 @libcat1a  TCP | TCP/IP | tcp05 | recvfrom | Receive a datagram and store the source address |
 @libcat1a  TCP | TCP/IP | tcp06 | sendto | Transmit a datagram |

 @libcat2y  PPP | Client | tcp40 | PPP_Client_Installed | Check if PPP client is installed |
 @libcat2y  PPP | Client | tcp41 | PPP_Client_Open | Open a PPP connection |
 @libcat2y  PPP | Client | tcp42 | PPP_Client_Close | Close a PPP client connection |
 @libcat2y  PPP | Client | tcp43 | PPP_Client_GetStatus | Get PPP client status |
 @libcat2y  PPP | Client | tcp44 | PPP_Client_Get_DNSIP | Get DNS IP |
 @libcat2y  PPP | Client | tcp45 | PPP_Client_SetOptions | Set PPP options for the PPP client |
 @libcat2y  PPP | Client | tcpPCIp | PPP_Client_GetIP | Get IP Address of the PPP Client |

 @libcat3y  PPP | Server | tcp50 | PPP_Server_Installed | Check if PPP server is installed |
 @libcat3y  PPP | Server | tcp51 | PPP_Server_Suspend | Suspend the PPP server task |
 @libcat3y  PPP | Server | tcp52 | PPP_Server_Activate | Activate the PPP server |
 @libcat3y  PPP | Server | tcp53 | PPP_Server_GetStatus | Get the current state of the PPP server |
 @libcat3y  PPP | Server | tcp54 | PPP_Server_Get_Cfg | Get configuration data of the PPP server |
 @libcat3y  PPP | Server | tcp55 | PPP_Server_SetOptions | Set PPP options for the PPP server |
 @libcat3y  PPP | Server | tcpPSIp | PPP_Server_GetIP | Get IP Address of the PPP Server |

 @libcat4  Other |    |   |  | <tab><tab>--- Monitoring ---
 @libcat4  Other | tcp65 | Get_FTP_Login_Counters | Access the FTP server login counters |
 @libcat4  Other | tcp66 | Get_Telnet_Login_Counters | Access the Telnet server login counters |
 @libcat4  Other | tcp67 | Get_Telnet_State | Check if have an active telnet session |
 @libcat4  Other | tcp70 | Get_Installed_Servers | Get information about running servers |
 @libcat4  Other | tcp78 | Get_TCPIP_Memory_Status | Get information about TCP/IP stack memory usage |
 @libcat4  Other | tcp74 | Get_TCPIP_Statistics | Access the internal network packet counter |
 @libcat4  Other | tcp60 | Get_SNMP_Data | Get internal TCP/IP SNMP variables |
 @libcat4  Other | tcp60a | Get_SNMP_DataEx | Get internal TCP/IP SNMP variables (Extended) |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- DHCP ---
 @libcat4  Other | tcp72 | DHCP_use | Set/Reset DHCP usage of the Ethernet interface |
 @libcat4  Other | tcp73 | Get_DHCP_Status | Get DHCP status of the Ethernet interface |
 @libcat4  Other | tcp73a | Get_DHCP_Status_Ext | Get DHCP status of the Ethernet interface |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- ping ---
 @libcat4  Other | tcp75 | Ping_Open | Open and start ICMP echo requests |
 @libcat4  Other | tcp76 | Ping_Close | Finish ICMP echo requests |
 @libcat4  Other | tcp77 | Ping_Statistics | Retrieve ping information |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- Server Time-outs ---
 @libcat4  Other | tcp79 | Set_FTPServer_Idle_Timeout | Set FTP server time-out  |
 @libcat4  Other | tcp79a | Set_TelnetServer_Idle_Timeout | Set Telnet server time-out  |
 @libcat4  Other | tcp79b | Get_FTPServer_Idle_Timeout | Get FTP server time-out value  |
 @libcat4  Other | tcp79c | Get_TelnetServer_Idle_Timeout | Get Telnet server time-out value  |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- Gateway ---
 @libcat4  Other | tcp80 | AddDefaultGateway | Add the default gateway |
 @libcat4  Other | tcp80a | AddDefaultGatewayEx | Add the default gateway (Extended) |
 @libcat4  Other | tcp81 | DelDefaultGateway | Delete the default gateway |
 @libcat4  Other | tcp82 | GetDefaultGateway | Get the current default gateway |
 @libcat4  Other | tcp83 | AddStaticRoute | Add a route for a interface |
 @libcat4  Other | tcp84 | DelStaticRoute | Delete a route from an interface |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- ARP cache access ---
 @libcat4  Other | tcp88 | DelArpEntryByPhys | Delete ARP entry by physical address |
 @libcat4  Other | tcp89 | AddArpEntry | Add an entry in the ARP table |
 @libcat4  Other | tcp8A | GetARPRouteCacheEntries | Read ARP cache table|
 @libcat4  Other |    |   |  | <nl><tab><tab>--- IP Multicast ---
 @libcat4  Other | tcp90 | Add_IGMP_Membership | Install an IP multicast address entry |
 @libcat4  Other | tcp91 | Drop_IGMP_Membership | Delete an IP multicast address entry |
 @libcat4  Other | tcp92 | IPMulticast_to_MacAddr | Map IP multicast address to ethernet address |
 @libcat4  Other |    |   |  | <nl><tab><tab>--- TCPIP Device driver ---
 @libcat4  Other | tcpA0  | Dev_Open_Interface |  Install user device driver |
 @libcat4  Other | tcpA1  | Dev_Close_Interface |  Close user device driver/interface |
 @libcat4  Other | tcpA2  | Dev_Recv_Interface |  Receive and process incoming data |
 @libcat4  Other | tcpA3  | Dev_Recv_Wait | Waits for data received signal |
 @libcat4  Other | tcpA4  | Dev_Notify_ISR | Signal from Interrupt Service Routine |
 @libcat4  Other | tcpA5  | Dev_Get_Buffer | Get a buffer from TCP/IP stack        |
 @libcat4  Other | tcpA6  | Dev_Send_Complete | Signal message send complete     |
 @libcat4  Other | tcpA7  | Dev_Wait_DHCP_Complete | Waits for DHCP completion   |


 @libcat4  Other |    |   |  | <nl><tab><tab>--- Miscellaneous ---
 @libcat4  Other | tcp09 | api_sleep | Sleep for specified number of milliseconds |
 @libcat4  Other | tcp71 | Reconfigure_ethernet | Reconfigure Ethernet interface |
 @libcat4  Other | tcp7a | RegisterIPUserCallbackHandler | Register an User callback handler for IP |
 @libcat4  Other | tcp7b | Register_ARP_User_CB | Register an ARP User callback handler. |
 @libcat4  Other | tcp8D | Get_Iface_Entries | Read table of TCP/IP device interfaces |
*/

/*************************************************************************/
// Get IP of the ethernet interface
/*************************************************************************/
#ifdef _tcpIPR_
/*
 @doc DOC_tcpIPR_
 ---------------------------------------------------------------
 Get IP, Net Mask of the default ethernet interface and the Gateway
 ---------------------------------------------------------------
 @clib  Get_IPConfig | Get IP, subnet mask of the default ethernet interface and/or gateway. |
        <cfont>void Get_IPConfig ( char far *IP,<nl>
        <tab><tab><tab><tab><tab>char far *Mask, char far *Gateway );<efont> |
 @parm  int  | IP  | Output parameter:<s2> Pointer to a 16 byte memory area
        where the IP address is to be stored as a null terminated string.
 @parm  int  | Mask  | Output parameter:<s2> Pointer to a 16 byte memory area
        where the subnet mask is to be stored as a null terminated string.
 @parm  int  | Gateway | Output parameter:<s2> Pointer to a 16 byte memory
        area where the gateway address is to be stored as a null terminated
        string.
 @rdesc -- none --
 @comm  Any of the three output parameters can be set to NULL pointers if
        the respective output is not desired.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x01<linkmid>documentation<linktail>
        for more details.

 @Also  tcpIPW.htm | Set_IPConfig  |  |  |  |
*/
void Get_IPConfig (char far *IP, char far * Mask, char far *Gateway)
{
    if (FP_SEG(IP))
    {
        _DX = FP_OFF(IP);
        _ES = FP_SEG(IP);
        _AH = 0x01;                         // Get IP Address in [ES:DX]
        asm { int BIOS_VECT  } ;
    }

    if (FP_SEG(Mask))
    {
        _DX = FP_OFF(Mask);
        _ES = FP_SEG(Mask);
        _AH = 0x03;                         // Get Net Mask in [ES:DX]
        asm { int BIOS_VECT  } ;
    }

    if (FP_SEG(Gateway))
    {
        _DX = FP_OFF(Gateway);
        _ES = FP_SEG(Gateway);
        _AH = 0x05;                         // Get Gateway in [ES:DX]
        asm { int BIOS_VECT  } ;
    }
}
#endif

/*************************************************************************/
// Set IP configuration,becomes valid after call of reconfigure Ethernet
/*************************************************************************/
#ifdef _tcpIPW_
/*
 @doc DOC_tcpIPW_
 ---------------------------------------------------------------
 Set IP, Net Mask of the default ethernet interface and/or Gateway
 ---------------------------------------------------------------
 @clib  Set_IPConfig | Set IP, subnet mask of the default ethernet interface and/or gateway. |
        <cfont>void Set_IPConfig ( const char far *IP,<nl>
        <tab><tab><tab><tab><tab>const char far *Mask, const char far *Gateway );<efont> |
 @parm  int  | IP  | Pointer to new IP address expressed as a null
        terminated ASCII string in network address dot notation.
 @parm  int  | Mask  | Pointer to new net submask expressed as a null
        terminated ASCII string in dot notation.
 @parm  int  | Gateway | Pointer to new gateway address expressed as a null
        terminated ASCII string in dot notation.
 @rdesc -- none --
 @comm  Any of the three parameters can be set to NULL pointers if
        the respective value is not to be set.<nl>
        <nl>
        <bold>Important:<bolde><nl>
        This API function writes to chip.ini and is not reentrant.<s2>
        Don't use in different tasks or in combination with @CHIP-RTOS commands,<s2>
        which are writing to chip.ini, e.g. DHCP. Avoid race conditions with
        any other API call, which writes or read also to/from chip.ini
        e.g. <linkto>../../BIOSINTS.HTM#0xA00x23<linkmid>BIOS service 0x23<linktail>.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x02<linkmid>documentation<linktail>
        for more details.

 @Also  tcpPSIp.htm | Get PPP Server IP Address
        tcpPCIp.htm | Get PPP Client IP Address
        tcpIPR.htm  | Get_IPConfig  |
        tcp71.htm   | Reconfigure_ethernet
*/
void Set_IPConfig(const char far *IP,
                  const char far *Mask,
                  const char far *Gateway)
{
    if (FP_SEG(IP))
    {
        _DX = FP_OFF(IP);
        _ES = FP_SEG(IP);
        _AH = 0x02;                         // Set IP Address in [ES:DX]
        asm { int BIOS_VECT  } ;
    }

    if (FP_SEG(Mask))
    {
        _DX = FP_OFF(Mask);
        _ES = FP_SEG(Mask);
        _AH = 0x04;                         // Set Net Mask in [ES:DX]
        asm { int BIOS_VECT  } ;
    }

    if (FP_SEG(Gateway))
    {
        _DX = FP_OFF(Gateway);
        _ES = FP_SEG(Gateway);
        _AH = 0x06;                         // Set Gateway in [ES:DX]
        asm { int BIOS_VECT  } ;
    }
}
#endif

/*************************************************************************/
//PPP Server IP
/*************************************************************************/
#ifdef _tcpPSIp_
/*
 @doc DOC_tcpPSIp_
 ---------------------------------------------------------------
 Reads the IP Address of the PPP Server
 ---------------------------------------------------------------
 @clib  PPP_Server_GetIP | Reads the IP Address of the PPP Server |
        <cfont>void PPP_Server_GetIp (char far *IP );<efont> |
 @parm  int  | IP  | Output parameter:<s2> Pointer to a 16 byte memory area
        where the IP address is to be stored as a null terminated string.
 @rdesc -- none --
 @comm  Copies the current IP Address of the PPP Server into the comitted
        string. If no IP Address of the PPP Server is available the string
        will be empty.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x50<linkmid>documentation<linktail>
        for more details.

 @Also  tcpIPR.htm | Get IP Config of the Ethernet interface |
        tcpPCIp.htm | Get IP Address of the PPP Client | |
*/
void PPP_Server_GetIP (char far *IP)
{
    _DX = FP_OFF(IP);
    _ES = FP_SEG(IP);
    _AH = 0x50;
    asm { int BIOS_VECT  } ;
}
#endif

/*************************************************************************/
//PPP Client IP
/*************************************************************************/
#ifdef _tcpPCIp_
/*
 @doc DOC_tcpPCIp_
 ---------------------------------------------------------------
 Reads the IP Address of the PPP Client
 ---------------------------------------------------------------
 @clib  PPP_Client_GetIP | Reads the IP Address of the PPP Client |
        <cfont>void PPP_Client_GetIP ( const char far *IP );<efont> |
 @parm  int  | IP  | Output parameter:<s2> Pointer to a 16 byte memory area
        where the IP address is to be stored as a null terminated string.
 @rdesc -- none --
 @comm  Copies the current IP Address of the PPP Client into the comitted
        string. If no IP Address of the PPP Client is available the string
        will be empty.

 @Also  tcpIPR.htm | Get IP Config of the Ethernet interface |
        tcpPSIp.htm | Get IP Address of the PPP Server | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x55<linkmid>documentation<linktail>
        for more details.

*/
void PPP_Client_GetIP (const char far *IP)
{
    _DX = FP_OFF(IP);
    _ES = FP_SEG(IP);
    _AH = 0x55;
    asm { int BIOS_VECT  } ;
}
#endif


/*****************************************************************************
*
*  open a tcp connection
*
*  DestIPStr: destination IP address format xxx.xxx.xxx.xxx
*  ClientPort: local port (0=random port)
*  HostPort: remote port
*  error: TCP/IP error
*
*
*  return: socket descriptor (-1 = error)
*/

#ifdef _tcpcon_
/*
 @doc DOC_tcpcon_
 ---------------------------------------------------------------
 open a tcp connection
 ---------------------------------------------------------------
 @clib  tcp_connect | Open a client TCP/IP socket connection to a specified
        host. |
        <cfont>int tcp_connect ( const char far *DestIPStr,<nl>
        <tab><tab><tab><tab>&nbsp; unsigned int ClientPort, unsigned int HostPort,<nl>
        <tab><tab><tab><tab>&nbsp; int *error );<efont> |
 @parm  int  | DestIPStr  | Pointer to server's IP address, expressed
        as a null terminated ASCII string in network dot notation.
 @parm  int  | ClientPort  | Optional local port number to assign to socket.<s2>
        Set this to zero if a bind operation is not desired (random port).
 @parm  int  | HostPort    | Remote port number to which connection is desired.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1 = Failure (see <code>error<codee> output parameter) <nl>
        else new socket descriptor
 @comm  This function calls the required set of TCP/IP functions to establish
        a client connection to a host.<nl>
        <nl>
        Pseudo code:<nl>
        <nl>
        <code>
        struct sockaddr_in addr;<nl>
        int sd = opensocket(SOCK_STREAM, error) ;<nl>
        addr.sin_family = PF_INET ;<nl>
        addr.sin_addr.s_addr =  0 ;<nl>
        IF ClientPort != 0 THEN<nl>
        <tab>addr.sin_port = htons(ClientPort) ;<nl>
        <tab>bind(sd, &addr, error) ;<nl>
        ENDIF<nl>
        addr.sin_port = htons(HostPort) ;<nl>
        error = inet_addr(DestIPStr, &addr.sin_addr.s_addr) ;<nl>
        error = connect (sd, &addr) ;<nl>
        return sd ;<nl>
        <codee><nl>
        <nl>
        The error handling is not shown in the above simplified pseudo code.<s2>
        In case of errors, the socket is closed by this API function and a return
        is made from the point of the error with return value -1.<s2>
        On success, the new socket descriptor is returned.<s2> The caller is then
        responsible for calling <linkto>tcp02.htm<linkmid>closesocket<linktail> when
        finished with the connection.
 @Also  tcp01.htm | opensocket  |
        tcphton.htm | htons  |
        tcp03.htm | bind
 @Also  tcp08.htm | inet_addr  |
        tcp04.htm | connect  |
        tcp02.htm | closesocket
*/
int tcp_connect (const char far *DestIPStr,
                 unsigned int ClientPort,
                 unsigned int HostPort,
                 int *error)
{
    int local_error;
    // sd = socket descriptor, open socket
    int sd = opensocket (SOCK_STREAM, &local_error) ;
    if (local_error == 0)                       // Open successful?
    {
        struct sockaddr_in addr;

        addr.sin_family = PF_INET ;
        addr.sin_addr.s_addr =  0 ;
        if (ClientPort != 0)
        {   // force client port
            addr.sin_port = HTONS(ClientPort);  // convert byte order
            bind (sd, &addr, &local_error ) ;
        }
        if (local_error == 0)                   // No bind failure?
        {
            addr.sin_port = HTONS(HostPort); // convert byte order

            // convert server IP address string to binary
            local_error = inet_addr (DestIPStr, &addr.sin_addr.s_addr);
            if (local_error == 0)               // Valid IP address?
            {
                // establish a connection to the server
                connect (sd, &addr, &local_error) ;
            }
        }
        if (local_error)                        // Failure?
        {
            closesocket (sd, &local_error) ;                  // Give up.
            sd = -1 ;
        }
    }
    *error = local_error ;                      // Set output parameter

    return sd ;
}  // tcp_connect()

#endif

/*************************************************************************/
// open socket
/*************************************************************************/
#ifdef _tcp01_
/*
 @doc DOC_tcp01_
 ---------------------------------------------------------------
 open socket
 ---------------------------------------------------------------
 @clib  opensocket | Open a socket. |
        <cfont>int opensocket ( unsigned char type, int *error );<efont> |
 @parm  int  | type  | SOCK_STREAM = 1 for TCP socket<nl>
        SOCK_DGRAM = 2 for UDP datagram socket.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1: Failure (see <code>error<codee> output parameter)<nl>
        else socket descriptor
 @comm  Creates an endpoint for communication and returns a socket descriptor.
        <s2>This function provides the BSD socket() functionality.

 @Also  tcp02.htm | closesocket  |
        tcp03.htm | bind |
        tcp04.htm | connect

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x01<linkmid>documentation<linktail>
        for more details.

*/
int opensocket (unsigned char type, int *error)
{
    int sd ;
    int error_code = 0 ;
    _AL = type ;
    _AH = API_OPENSOCKET;
    asm { int TCPIPVECT  } ;
    sd = _AX ;
    if (_DX != 0)
    {
        error_code = sd ;       // Error code was returned in AX.
        sd = -1 ;
    }
    *error = error_code ;
    return sd ;
}
#endif


/*************************************************************************/
// close socket
/*************************************************************************/
#ifdef _tcp02_
/*
 @doc DOC_tcp02_
 ---------------------------------------------------------------
 close socket
 ---------------------------------------------------------------
 @clib  closesocket | Close a socket. |
        <cfont>int closesocket ( int sd, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | error | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1: Failure (see <code>error<codee> output parameter)<nl>
         0: success
 @comm  Closes the socket indicated by <code>sd<codee> and releases all of
        its associated resources.

 @Also  tcp01.htm | opensocket  | | | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x02<linkmid>documentation<linktail>
        for more details.

*/
int closesocket (int sd, int *error)
{
    _BX = sd ;
    _AH = API_CLOSESOCKET;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
// bind
/*************************************************************************/
#ifdef _tcp03_
/*
 @doc DOC_tcp03_
 ---------------------------------------------------------------
 Bind socket
 ---------------------------------------------------------------
 @clib  bind | Bind an unnamed socket to an address and port number. |
        <cfont>int bind ( int sd, const struct sockaddr_in far *addressPtr,<nl>
        <tab><tab>&nbsp;&nbsp; int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | addressPtr  | Pointer to a <code>sockaddr_in<codee> structure
        (see tcpipapi.h) preset by caller.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  It is only necessary to use the bind call in server applications.<s2>
        If you use the bind call in a client application, the client uses the
        given port number as its own source port address.<s2> Otherwise a random
        16-bit source port number will be used when no bind call is made.<nl>
        <nl>
        The <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>sockaddr_in<linktail>
        structure at <cfont>addressPtr<efont> must be filled in by
        the caller prior to making this API call.<s2>An example of how this
        can be done follows.<nl>
        <nl>
        @raw
        // Example
        int ClientPort = 3000 ;
        struct sockaddr_in addr ;
        int error_code ;

        addr.sin_family = PF_INET ;
        addr.sin_addr.s_addr =  0 ;
        addr.sin_port = htons(ClientPort);   // convert byte order
        error_code = bind (sd, &addr ) ;

 @Also  tcp01.htm | opensocket  |
        tcp04.htm | connect | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x03<linkmid>documentation<linktail>
        for more details.

*/
int bind (int sd, const struct sockaddr_in far * addressPtr, int *error)
{
    _DX = FP_SEG(addressPtr);
    _SI = FP_OFF(addressPtr);
    _BX = sd ;
    _AH = API_BIND;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;
}
#endif

/*************************************************************************/
// connect
/*************************************************************************/
#ifdef _tcp04_
/*
 @doc DOC_tcp04_
 ---------------------------------------------------------------
 Connect socket
 ---------------------------------------------------------------
 @clib  connect | Connect to another socket. |
        <cfont>int connect ( int sd,<nl>
        <tab><tab><tab>&nbsp; const struct sockaddr_in far *addressPtr, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | addressPtr  | Pointer to a sockaddr_in
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>structure<linktail>
        containing host's IP address and port number.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  The caller must fill in the sockaddr_in data
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>structure<linktail> at
        <code>addressPtr<codee> prior to calling here.<s2> An example of how
        this normally would be done follows.<s2> The PF_INET setting for the
        <code>sin_family<codee> is required.<nl>
        <nl>
        @raw
        // Example
        char szHostIPStr[] = "172.30.1.68" ;
        int HostPort = 3000 ;
        struct sockaddr_in addr ;
        int error_code ;

        addr.sin_family = PF_INET ;
        addr.sin_addr.s_addr =  0 ;
        addr.sin_port = htons (HostPort) ;   // convert byte order

        // convert server IP address string to binary
        inet_addr (szHostIPStr, &addr.sin_addr.s_addr);
        // establish a connection to the server
        error_code = connect (sd, &addr) ;

 @Also  tcp01.htm | opensocket |
        tcpcon.htm | tcp_connect  |
        tcphton.htm | htons
 @Also  tcp08.htm | inet_addr |
        tcp03.htm | bind | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x04<linkmid>documentation<linktail>
        for more details.


*/
int connect (int sd, const struct sockaddr_in far * addressPtr, int *error)
{
    _DX = FP_SEG(addressPtr);
    _SI = FP_OFF(addressPtr);
    _BX = sd ;
    _AH = API_CONNECT;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
//recvfrom
/*************************************************************************/
#ifdef _tcp05_
/*
 @doc DOC_tcp05_
 ---------------------------------------------------------------
 UDP only, receive message from another socket
 ---------------------------------------------------------------
 @clib  recvfrom | Receive message from another socket. |
        <cfont>int recvfrom ( int sd, char far *bufptr, int bufLen,<nl>
        <tab><tab><tab>&nbsp;&nbsp; int flags, unsigned long timeout,<nl>
        <tab><tab><tab>&nbsp;&nbsp; struct sockaddr_in far *fromPtr,<nl>
        <tab><tab><tab>&nbsp;&nbsp; int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | bufptr  | Pointer to input buffer where received
        characters will be output.
 @parm  int  | bufLen  | Maximum characters to store in
        buffer at <code>bufptr<codee>.
 @parm  int  | flags  | Wait options:<nl>
        <s2>MSG_BLOCKING  - Sleep until data comes in<nl>
        <s2>MSG_TIMEOUT  - The caller wakes up after timeout or if <code>bufLen<codee>
        data comes in<nl>
        <s2>MSG_DONTWAIT - Return immediately after collecting what ever data is ready<nl>
 @parm  int  | timeout  | Maximum milliseconds to wait if <code>flags<codee> is set
        to MSG_TIMEOUT:<nl>
 @parm  int  | fromPtr  | Output parameter:<s2>Pointer to a sockaddr_in
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>structure<linktail>
        which will be set to indicate from where the
        received data originated.<s2>This pointer can be set to NULL if this
        information is not desired.<nl>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1:  Failure, see error value reported<nl>
        else count of bytes received and placed in <code>bufptr<codee><s2>
        (0 if time-out)
 @comm  This API function applies only to UDP sockets.<nl>
        <nl>
        The MSG_BLOCKING mode will not work if the socket has
        been put into non-blocking mode using the
        <linkto>tcp18.htm<linkmid>Set_Blocking_Mode<linktail> API.

 @Also  tcp06.htm | sendto  |
        tcp01.htm | opensocket |
        tcp18.htm | Set_Blocking_Mode

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x05<linkmid>documentation<linktail>
        for more details.

*/
int recvfrom(int sd, char far * bufptr, int bufLen, int flags,
             unsigned long timeout,
             struct sockaddr_in far * fromPtr,
             int *error)
{
    int bytes_received ;
    int the_error_code = 0 ;
    int fromLen;
    struct recv_params R;

    //fill the struct recv_param R
    fromLen          =   sizeof(struct sockaddr_in);
    R.bufferPtr      =   bufptr;
    R.bufferLength   =   bufLen;
    R.flags          =   flags;
    R.fromPtr        =   (struct sockaddr *)fromPtr;
    R.fromlengthPtr  =   &fromLen;
    R.timeout        =   timeout; /*milliseconds*/

    //API
    _BX = sd;
    _DX = FP_SEG(&R);
    _SI = FP_OFF(&R);
    _AH = API_RECVFROM;
    asm { int TCPIPVECT  } ;
    bytes_received = _AX ;
    if (_DX != 0)
    {
        the_error_code = bytes_received ;       // AX value
        bytes_received = API_ERROR ;
    }
    *error = the_error_code ;
    return bytes_received ;

}
#endif

/*************************************************************************/
//sendto
/*************************************************************************/
#ifdef _tcp06_
/*
 @doc DOC_tcp06_
 ---------------------------------------------------------------
 Transmit a datagram
 ---------------------------------------------------------------
 @clib  sendto | Transmit a datagram. |
        <cfont>int sendto ( int sd, const char far *bufptr, int bufLen,<nl>
        <tab><tab><tab> int flags, const struct sockaddr_in far *toPtr,<nl>
        <tab><tab><tab> int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | bufptr  | Pointer to output buffer containing
        characters to be transmitted.
 @parm  int  | bufLen  | Maximum characters to output from <code>bufptr<codee>.
 @parm  int  | flags  | Wait options:<nl>
        <s2>MSG_BLOCKING  - Wait for data send to complete<nl>
        <s2>MSG_DONTWAIT - Don't wait for data send to complete<nl>
 @parm  int  | toPtr  | Data destination
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1:  Failure, see error value reported<nl>
        else count of bytes transfered into the socket send queue.
 @comm  This API function applies only to UDP sockets.<s2>
        This function will output up to <code>bufLen<codee> bytes from the buffer
        at <code>bufptr<codee> to the IP address specified by the sockaddr_in
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>structure<linktail>
        referenced by the <code>toPtr<codee> pointer.<s2> The return
        value indicates the actual number of bytes sent.
        <nl><nl>
        The MSG_BLOCKING mode will not work if the socket has
        been put into non-blocking mode using the
        <linkto>tcp18.htm<linkmid>Set_Blocking_Mode<linktail> API.

 @Also  tcp05.htm | recvfrom  |
        tcp01.htm | opensocket |
        tcp18.htm | Set_Blocking_Mode

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x06<linkmid>documentation<linktail>
        for more details.

*/
int sendto (int sd, const char far * bufptr, int bufLen, int flags,
            const struct sockaddr_in far * toPtr, int *error)
{
    int bytes_sent ;
    int the_error_code = 0 ;
    int toLen        = sizeof(struct sockaddr_in);

    //Init the struct send_param S for API-Call
    struct send_params S;
    S.bufferPtr      = bufptr;
    S.bufferLength   = bufLen;
    S.flags          = flags;
    S.toPtr          = (struct sockaddr *) toPtr;
    S.tolengthPtr    = &toLen;

    //API
    _BX = sd;
    _DX = FP_SEG(&S);
    _SI = FP_OFF(&S);
    _AH = API_SENDTO;
    asm { int TCPIPVECT  } ;
    bytes_sent = _AX ;
    if (_DX != 0)
    {
        the_error_code = bytes_sent ;       // AX value
        bytes_sent = API_ERROR ;
    }
    *error = the_error_code ;
    return bytes_sent ;
}
#endif

/*************************************************************************/
//
/*************************************************************************/
#ifdef _tcphton_
/*
 @doc DOC_tcphton_
 ---------------------------------------------------------------
 Converts a 16 bit word from host to TCP/IP network byte order
 ---------------------------------------------------------------
 @clib  htons | Converts a 16 bit word from host to TCP/IP network byte order. |
        <cfont>unsigned short htons ( unsigned short hostshort );<efont> |
 @parm  int  | hostshort  | Port number in little endian (PC) byte order
 @rdesc input value returned in network byte order (big endian)
 @comm  This function converts the provided 16 bit value from Intel byte
        ordering (little endian) to network byte order (big endian).<s2>
        This is commonly needed for TCP/IP port number specifications.
 @Also  tcp04.htm | connect  |
        tcp03.htm | bind     | |
*/
unsigned short htons (unsigned short hostshort )
{
    _AX = hostshort ;
    asm { XCHG AH,AL } ;
    return _AX ;
}
#endif


/*************************************************************************/
// inet_addr
/*************************************************************************/
#ifdef _tcp08_
/*
 @doc DOC_tcp08_
 ---------------------------------------------------------------
 Converts a dotted decimal IP address string to an unsigned long
 ---------------------------------------------------------------
 @clib  inet_addr | Converts a dotted decimal IP address string to an unsigned long. |
        <cfont>int inet_addr ( const char far *IPAddressStringPtr,<nl>
        <tab><tab><tab><tab>unsigned long far *IPAddress );<efont> |
 @parm  int  | IPAddressStringPtr  | Pointer to null terminated
        ASCII string of network address in dot notation.
 @parm  int  | IPAddress  | Output parameter:<s2>Pointer to a
        32 bit location where binary equivalent to the provided ASCII
        IP address will be output.
 @rdesc 0:  success <nl>
        non-zero:  Invalid IP address.

 @Also  tcpcon.htm | tcp_connect |
        tcp11.htm  | InetToAscii | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x08<linkmid>documentation<linktail>
        for more details.

*/
int inet_addr (const char far * IPAddressStringPtr,
               unsigned long far * IPAddress)
{
    _BX = FP_SEG(IPAddressStringPtr);
    _SI = FP_OFF(IPAddressStringPtr);
    _ES = FP_SEG(IPAddress);
    _DI = FP_OFF(IPAddress);

    _AH = API_INETADDR;
    asm { int TCPIPVECT  } ;
    return _DX ;
}
#endif

/*************************************************************************/
//sleep
/*************************************************************************/
#ifdef _tcp09_
/*
 @doc DOC_tcp09_
 ---------------------------------------------------------------
 The application sleeps for given number of milliseconds
 ---------------------------------------------------------------
 @clib  api_sleep | The calling task sleeps for specified
        number of milliseconds. |
        <cfont>void api_sleep ( unsigned int howlong );<efont> |
 @parm  int  | howlong  | Sleep period in milliseconds
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x09<linkmid>documentation<linktail>
        for more details.

*/
void api_sleep (unsigned int howlong)
{
    _BX = howlong ;
    _AH = API_SLEEP ;
    asm { int TCPIPVECT  } ;
}
#endif


/*************************************************************************/
//get waiting bytes
/*************************************************************************/
#ifdef _tcp0C_
/*
 @doc DOC_tcp0C_
 ---------------------------------------------------------------
 Get the number of bytes on a socket, waiting for read
 ---------------------------------------------------------------
 @clib  GetWaitingBytes | Report the number of bytes available
        on a socket, waiting to be read. |
        <cfont>int GetWaitingBytes ( int sd, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1 on failure, see error reported<nl>
        else count of received bytes waiting on socket.

 @Also  tcp10.htm | recv  |
        tcp05.htm | recvfrom |
        tcp01.htm | opensocket

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x0C<linkmid>documentation<linktail>
        for more details.

*/
int GetWaitingBytes (int sd, int *error)
{
    int bytes_ready ;
    int the_error_code = 0 ;

    _BX = sd ;
    _AH = API_GETRCV_BYTES ;
    asm { int TCPIPVECT  } ;
    bytes_ready = _AX ;
    if (_DX != 0)
    {
        the_error_code = bytes_ready ;
        bytes_ready = API_ERROR ;
    }

    *error = the_error_code ;
    return bytes_ready ;
}
#endif

/*************************************************************************/
//accept
/*************************************************************************/
#ifdef _tcp0D_
/*
 @doc DOC_tcp0D_
 ---------------------------------------------------------------
 accept the next incoming connection
 ---------------------------------------------------------------
 @clib  accept | Accept the next incoming connection. |
        <cfont>int accept ( int sd, struct sockaddr_in far *addressPtr,<nl>
        <tab><tab><tab> int *error );<efont> |
 @parm  int  | sd  | Socket descriptor (must be switched into
        listen mode using <linkto>tcp0E.htm<linkmid>listen<linktail> API).
 @parm  int  | addressPtr  | Optional output parameter:<s2>Pointer to a
        sockaddr_in
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>structure<linktail>
        (see tcpipapi.h).<s2>On success, this function fills in
        this sockaddr_in structure with the IP address and port number of
        the accepted connection.<s2>This pointer can be set to NULL if
        this address and port number report is not desired.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1: Failure, error code output to <code>error<codee><nl>
        else contains new socket descriptor for the connection.
 @comm  This API function extracts the first connection on the queue
        of pending connections (client calls to
        <linkto>tcp0E.htm<linkmid>listen<linktail> API).<s2>  It then creates
        a new socket for this connection.<s2>This new socket will have
        the same socket options as the listening socket, <code>sd<codee>.
        <nl>

        This call is used by a TCP server.
 @Also  tcp0E.htm | listen     |
        tcp01.htm | opensocket | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x0D<linkmid>documentation<linktail>
        for more details.

*/
int accept (int sd,
            struct sockaddr_in far * addressPtr,
            int *error)
{
    int sd_accepted ;
    int the_error_code = 0 ;

    _BX = sd ;
    _DX = FP_SEG(addressPtr);
    _SI = FP_OFF(addressPtr);

    _AH = API_ACCEPT ;
    asm { int TCPIPVECT  } ;
    sd_accepted = _AX ;
    if (_DX != 0)
    {
        the_error_code = sd_accepted ;
        sd_accepted = API_ERROR ;
    }

    *error = the_error_code ;
    return sd_accepted ;
}
#endif


/*************************************************************************/
//listen
/*************************************************************************/
#ifdef _tcp0E_
/*
 @doc DOC_tcp0E_
 ---------------------------------------------------------------
 Listen for incoming connections
 ---------------------------------------------------------------
 @clib  listen | Listen for incoming connections. |
        <cfont>int listen ( int sd, int backlog, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | backlog  | The maximum number (limited to 5) of allowed
        outstanding connections
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function places the socket in passive mode and sets the
        number of incoming TCP connections the system will queue
        (<code>backlog<codee>).<nl>
        <nl>
        This call is used by a TCP server.

 @Also  tcp0D.htm | accept  |
        tcp01.htm | opensocket | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x0E<linkmid>documentation<linktail>
        for more details.

*/
int listen (int sd, int backlog, int * error)
{
    _BX = sd ;
    _CX = backlog ;

    _AH = API_LISTEN ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX;
}
#endif


/*************************************************************************/
//send
/*************************************************************************/
#ifdef _tcp0F_
/*
 @doc DOC_tcp0F_
 ---------------------------------------------------------------
 Transmit a message
 ---------------------------------------------------------------
 @clib  send | Transmit a message. |
        <cfont>int send ( int sd, const char far *bufptr, int bufLen,<nl>
        <tab><tab>&nbsp;&nbsp; int flags, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | bufptr  | Pointer to output buffer containing
        data to be transmitted.
 @parm  int  | bufLen  | Maximum characters to output from <code>bufptr<codee>.
 @parm  int  | flags  | Wait options:<nl>
        <s2>MSG_BLOCKING  - Wait for data send to complete<nl>
        <s2>MSG_DONTWAIT - Don't wait for data send to complete<nl>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1:  Failure, see error value reported<nl>
        else count of bytes transfered into the socket send queue.
 @comm  This API function applies only to TCP sockets.<s2>
        It transmits a message to another transport end-point.<s2>
        It may be used only if the socket is in a
        connected state.<nl>
        <nl>
        The MSG_BLOCKING mode will not work if the socket has
        been put into non-blocking mode using the
        <linkto>tcp18.htm<linkmid>Set_Blocking_Mode<linktail> API.

 @Also  tcp10.htm | recv  |
        tcp06.htm | sendto |
        tcp01.htm | opensocket
 @Also  tcp18.htm | Set_Blocking_Mode  | | | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x0F<linkmid>documentation<linktail>
        for more details.

*/
int send(int sd, const char far * bufptr, int bufLen,
         int flags, int *error)
{
    int bytes_sent ;
    int the_error_code = 0 ;

    //Init the struct send_param S for API-Call
    struct send_params S;
    S.bufferPtr      = bufptr;
    S.bufferLength   = bufLen;
    S.flags          = flags;
    S.toPtr          = NULL ;
    S.tolengthPtr    = NULL ;

    //API
    _BX = sd;
    _DX = FP_SEG(&S);
    _SI = FP_OFF(&S);
    _AH = API_SEND;
    asm { int TCPIPVECT  } ;
    bytes_sent = _AX ;
    if (_DX != 0)
    {
        the_error_code = bytes_sent ;       // AX value
        bytes_sent = API_ERROR ;
    }
    *error = the_error_code ;
    return bytes_sent ;
}
#endif


/*************************************************************************/
//recv
/*************************************************************************/
#ifdef _tcp10_
/*
 @doc DOC_tcp10_
 ---------------------------------------------------------------
 Receive a message
 ---------------------------------------------------------------
 @clib  recv | Receive a message from another socket. |
        <cfont>int recv ( int sd, char far *bufptr, int bufLen,<nl>
        <tab><tab>&nbsp;&nbsp; int flags, unsigned long timeout,<nl>
        <tab><tab>&nbsp;&nbsp; int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | bufptr  | Pointer to input buffer where received
        characters will be stored.
 @parm  int  | bufLen  | Maximum characters to store in
        buffer at <code>bufptr<codee>.
 @parm  int  | flags  | Wait options:<nl>
        <s2>MSG_BLOCKING  - Sleep until data comes in<nl>
        <s2>MSG_TIMEOUT  - The caller wakes up after timeout or if <code>bufLen<codee>
        data comes in<nl>
        <s2>MSG_DONTWAIT - Return immediately after collecting what ever data is ready<nl>
 @parm  int  | timeout  | Maximum milliseconds to wait if <code>flags<codee> is set
        to MSG_TIMEOUT:<nl>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -1:  Failure, see error value reported<nl>
        else count of received bytes stored in buffer at <code>bufptr<codee>.
 @comm  This API function applies only to TCP sockets.<s2>
        It receives a message from another socket.<s2>
        This API function may be used only if the socket is in a
        connected state.<nl>
        <nl>
        If <code>flags<codee> parameter is set to MSG_DONTWAIT this API call returns
        immediately.<s2>If no data is available -1 is returned and output parameter
        <code>error<codee> is set to 235.<nl>
        <nl>If <code>flags<codee> parameter is set to MSG_BLOCKING, the <code>recv<codee>
        call waits for a message to arrive.<s2>
        By default the blocking mode is set for all sockets at the
        <linkto>tcp01.htm<linkmid>opensocket<linktail> call.<s2> If however
        a socket was subsequently set to non blocking mode with the
        <linkto>tcp18.htm<linkmid>Set_Blocking_Mode<linktail> API, the MSG_BLOCKING flag
        will then not work here.
 @Also  tcp0F.htm | send  |
        tcp05.htm | recvfrom |
        tcp01.htm | opensocket
 @Also  tcp18.htm | Set_Blocking_Mode  | | | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x10<linkmid>documentation<linktail>
        for more details.

*/
int recv (int sd, char far * bufptr, int bufLen, int flags,
          unsigned long timeout, int *error)
{
    int bytes_received ;
    int the_error_code = 0 ;
    struct recv_params R;

    //fill the struct recv_param R
    R.bufferPtr      =   bufptr;
    R.bufferLength   =   bufLen;
    R.flags          =   flags;
    R.fromPtr        =   NULL ;
    R.fromlengthPtr  =   NULL ;
    R.timeout        =   timeout; /*milliseconds*/

    //API
    _BX = sd;
    _DX = FP_SEG(&R);
    _SI = FP_OFF(&R);
    _AH = API_RECV ;
    asm { int TCPIPVECT  } ;
    bytes_received = _AX ;
    if (_DX != 0)
    {
        the_error_code = bytes_received ;       // AX value
        bytes_received = API_ERROR ;
    }
    *error = the_error_code ;
    return bytes_received ;
}
#endif

/*************************************************************************/
// InetToAscii
/*************************************************************************/
#ifdef _tcp11_
/*
 @doc DOC_tcp11_
 ---------------------------------------------------------------
 Convert an IP address to an IP string
 ---------------------------------------------------------------
 @clib  InetToAscii | Convert an IP address to an ASCII dotted decimal
        IP  string. |
        <cfont>int InetToAscii ( const unsigned long IPAddress,<nl>
        <tab><tab><tab><tab>&nbsp; char far *IPAddressStringPtr );<efont> |
 @parm  int  | IPAddress  | Pointer to the 32 bit IP address in
        network byte order ("big endian").
 @parm  int  | IPAddressStringPtr  | Output parameter:<s2>
        Pointer to a <bold>17 character buffer<bolde> (minimum)
        where a null terminated ASCII string
        will be written by this API function containing the
        IP address expressed in dotted decimal notation.
 @rdesc Always 0
 @comm  This buffer at <code>IPAddressStringPtr<codee> must have
        space for 17 Bytes!.
 @Also  tcp08.htm | inet_addr  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x11<linkmid>documentation<linktail>
        for more details.

*/
int InetToAscii (const unsigned long far * IPAddress,
                 char far * IPAddressStringPtr)
{
    _BX = FP_SEG(IPAddress) ;
    _SI = FP_OFF(IPAddress) ;
    _ES = FP_SEG(IPAddressStringPtr) ;
    _DI = FP_OFF(IPAddressStringPtr) ;

    _AH = API_INETTOASCII ;
    asm { int TCPIPVECT  } ;
    return 0;
}
#endif


/*************************************************************************/
//reset connection
/*************************************************************************/
#ifdef _tcp12_
/*
 @doc DOC_tcp12_
 ---------------------------------------------------------------
 Abort a connection on a socket
 ---------------------------------------------------------------
 @clib  ResetConnection | Abort a connection on a TCP socket. |
        <cfont>int ResetConnection ( int sd, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function only works on TCP sockets.<s2>
        <nl><nl>
        A reset socket cannot be used for a new connection.<s2> You must close the socket
        and open a new one.<nl>
        <nl>
        This API function does not close the socket,<s2> This must be done
        using the <linkto>tcp02.htm<linkmid>closesocket<linktail> API.<nl>
 @Also  tcp02.htm | closesocket |
        tcp01.htm | opensocket  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x12<linkmid>documentation<linktail>
        for more details.

*/
int ResetConnection (int sd, int * error)
{
    _BX = sd ;

    _AH = API_RESETCONNECTION ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;                    // AX = 0, unless error code
}
#endif



/*************************************************************************/
// set linger time on close
/*************************************************************************/
#ifdef _tcp13_
/*
 @doc DOC_tcp13_
 ---------------------------------------------------------------
 Set linger time on close
 ---------------------------------------------------------------
 @clib  setlinger | Set linger time on close (TCP only). |
        <cfont>int setlinger ( int sd, int seconds, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | seconds  | Linger time in seconds, default: 60 sec,
        0 means linger turned off.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function works only with TCP sockets.
 @Also  tcp01.htm | opensocket | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x13<linkmid>documentation<linktail>
        for more details.

*/
int setlinger (int sd, int seconds, int *error)
{
    _BX = sd ;
    _CX = seconds ;

    _AH = API_SETLINGER ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
// set resuse option on a listening socket
/*************************************************************************/
#ifdef _tcp14_
/*
 @doc DOC_tcp14_
 ---------------------------------------------------------------
 Set reuse option on a listening socket
 ---------------------------------------------------------------
 @clib  setreuse | Set reuse option on a listening socket. |
        <cfont>int setreuse ( int sd, int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function works only with TCP sockets.<s2>  It was necessary when
        a listening socket was closed and will be opened and bound to the same port as
        it was bound to before.<nl>
        <nl>
        Since @CHIP-RTOS 071, every TCP socket is set by default as reusable, so use of
        this API function is no longer necessary.
 @Also  tcp01.htm | opensocket |
        tcp03.htm | bind | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x14<linkmid>documentation<linktail>
        for more details.

*/
int setreuse (int sd, int *error)
{
    _BX = sd ;

    _AH = API_SETREUSE ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX;
}
#endif

/*************************************************************************/
// Set IP Type-Of-Service on a socket
/*************************************************************************/
#ifdef _tcp15_
/*
 @doc DOC_tcp15_
 ---------------------------------------------------------------
 Set socket's Type-Of-Service
 ---------------------------------------------------------------
 @clib  Set_IP_Type_Of_Service | Set socket's default Type-Of-Service
        put into the IP datagram header's TOS field. |
        <cfont>int Set_IP_Type_Of_Service ( int sd, unsigned char tos,<nl>
        <tab><tab><tab><tab><tab><tab><tab> int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | tos  | Type-Of-Service for IP datagrams
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  Bits in the IP Type-Of-Service field of a IP datagram:<nl>
        <ul>
        Bits 0-2: Datagram precedence, 7 is the highest.<nl>
        Bits 3-5: type of transport, see TCP/IP documentation e.g.<nl>
        <tab><tab>Internet networking with TCP/IP by Douglas E.Comer<nl>
        Bits 6-7: unused<nl>
        <ule><nl>
        Note:<s2> Many routers ignore this IP datagram header field.<nl>
 @Also  tcp01.htm | opensocket | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x15<linkmid>documentation<linktail>
        for more details.

*/
int Set_IP_Type_Of_Service (int sd, unsigned char tos, int *error)
{
    _BX = sd ;

    _AL = tos ;
    _AH = API_SETIPTOS ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;
}
#endif

/*************************************************************************/
// set socket option
/*************************************************************************/
#ifdef _tcp16_
/*
 @doc DOC_tcp16_
 ---------------------------------------------------------------
 Set options on a socket
 ---------------------------------------------------------------
 @clib  setsockopt | Set options on a socket. |
        <cfont>int setsockopt ( int sd, const SetSocketOption far *sockoptptr,<nl>
        <tab><tab><tab><tab> int *error);<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | sockoptptr  | Pointer to SetSocketOption
        <linkto>../../tcpstruc.htm#SetSocketOption<linkmid>type<linktail> that
        specifies the socket options (see tcpipapi.h).
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function makes it possible to manipulate options associated
        with a socket.<s2>Prior to calling this function the caller must fill in
        a SetSocketOption
        <linkto>../../tcpstruc.htm#SetSocketOption<linkmid>type<linktail>
        data structure.<s2>The socket options of an incoming connection (using
        <linkto>tcp0D.htm<linkmid>accept<linktail>) will be the same as the
        socket options of the listening socket.
 @Also  tcp01.htm | opensocket |
        tcp0D.htm | accept |
        tcp17.htm | getsockopt
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x16<linkmid>documentation<linktail>
        for more details.

*/
int setsockopt (int sd, const SetSocketOption far *sockoptptr, int *error)
{
    _BX = sd ;

    _ES = FP_SEG(sockoptptr);
    _DI = FP_OFF(sockoptptr);

    _AH = API_SETSOCKOPT ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;
}
#endif

/*************************************************************************/
// get socket option
/*************************************************************************/
#ifdef _tcp17_
/*
 @doc DOC_tcp17_
 ---------------------------------------------------------------
 Get options on a socket
 ---------------------------------------------------------------
 @clib  getsockopt | Get options on a socket. |
        <cfont>int getsockopt ( int sd, const GetSocketOption far *sockoptptr,<nl>
        <tab><tab><tab><tab> int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | sockoptptr  | Pointer to GetSocketOption
        <linkto>../../tcpstruc.htm#GetSocketOption<linkmid>type<linktail> that
        specifies the socket options (see tcpipapi.h) and contains pointers
        to where the specified socket option will be reported to.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  On success, this function reports the requested socket option value to
        the buffer pointed to by the <code>optionValue<codee> member of
        GetSocketOption type at <code>sockoptptr<codee> and the length
        of this reported value is written to the location pointed to by
        the <code>optionLen<codee> member.<nl>
        <nl>
        The caller must setup the GetSocketOption
        <linkto>../../tcpstruc.htm#GetSocketOption<linkmid>type<linktail> structure
        prior to calling.<s2>In particular, the <code>optionValue<codee>
        member must point to a sufficiently large buffer to receive the
        reported option value and the size of this buffer must be stated
        in the location referenced by the <code>optionLen<codee> member.<s2>
        Note that this length specifier is both an input and an output
        parameter for this function.<s2>If on entry, the location referenced
        by <code>optionLen<codee> contains a value less than the buffer
        space required to report the requested socket option value, then
        the function will fail and return error code 242.<nl>
        <nl>
        The socket options of an incoming connection (using
        <linkto>tcp0D.htm<linkmid>accept<linktail>) will be the same as the
        socket options of the listening socket.
 @Also  tcp01.htm | opensocket |
        tcp16.htm | setsockopt | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x17<linkmid>documentation<linktail>
        for more details.

*/
int getsockopt (int sd, const GetSocketOption far *sockoptptr, int *error)
{
    _BX = sd ;

    _ES = FP_SEG(sockoptptr);
    _DI = FP_OFF(sockoptptr);

    _AH = API_GETSOCKOPT ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX;
}
#endif

/*************************************************************************/
//Set blocking mode on a socket (default is blocking!!)
/*************************************************************************/
#ifdef _tcp18_
/*
 @doc DOC_tcp18_
 ---------------------------------------------------------------
 Set a socket in blocking or non-blocking mode
 ---------------------------------------------------------------
 @clib  Set_Blocking_Mode | Set a socket in blocking or non-blocking mode. |
        <cfont>int Set_Blocking_Mode ( int sd, unsigned char blocking_mode,<nl>
        <tab><tab><tab><tab><tab><tab>int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | blocking_mode  | 0: switch blocking off, 1:switch blocking on.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  By default all sockets are in blocking mode.<s2> If a socket is
        set to non blocking mode, socket calls such as
        <linkto>tcp04.htm<linkmid>connect<linktail> and
        <linkto>tcp0D.htm<linkmid>accept<linktail> do not wait until
        full completion.<s2>Instead they  return immediately.<nl>
        <nl>
        Example usage of non-blocking mode:<nl><ul>
        The <linkto>tcp04.htm<linkmid>connect<linktail> call returns
        at a non blocking socket with error code 236, when
        the connection was not completed.<s2> The user can repeat this
        API call in a periodic loop until a successful connection is
        established.<nl>
        <ule>
 @Also  tcp04.htm | connect  |
        tcp0D.htm | accept |
        tcp10.htm | recv
 @Also  tcp05.htm | recvfrom  |
        tcp0F.htm | send |
        tcp06.htm | sendto
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x18<linkmid>documentation<linktail>
        for more details.

*/
int Set_Blocking_Mode (int sd, unsigned char blocking_mode, int *error)
{
    _BX = sd ;

    _AL = blocking_mode ;
    _AH = API_SETBLOCKINGMODE ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;                    // AX = 0, unless error code
}
#endif


/*************************************************************************/
//Register callback function on socket events
/*************************************************************************/
#ifdef _tcp19_
/*
 @doc DOC_tcp19_
 ---------------------------------------------------------------
 Register a user callback function
 ---------------------------------------------------------------
 @clib  RegisterCallbackFunction | Register a user callback function. |
        <cfont>int RegisterCallbackFunction ( int sd, void far *funcptr,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int eventflagmask,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *error );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @parm  int  | funcptr  | Pointer to user's callback function.
 @parm  int  | eventflagmask  | Event flag mask bit field (see menu below)
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  The events which trigger the callback are controlled with the
        <code>eventflagmask<codee> input parameter.<s2>The set of possible
        event flags are (defined in TCPIPAPI.H):<nl>
        <nl>
        <ul>
        CB_CONNECT_COMPLT - connection complete<nl>
        CB_ACCEPT<s2>     - remote has established a connection to our listening server<nl>
        CB_RECV<s2>     - incoming data arrived<nl>
        CB_SEND_COMPLT<s2>     - sending of data has been acknowledged by the peer<nl>
        CB_REMOTE_CLOSE<s2>     - peer has shut down the connection<nl>
        CB_SOCKET_ERROR<s2>     - an error occured on the connection<nl>
        CB_RESET<s2>     - peer has sent a rest on the connection<nl>
        CB_CLOSE_COMPLT<s2>     - close has been completed<nl>
        <ule>
        <nl>
        These flags can be OR'ed together to form the mask for the set of events
        which you want to be called at.<nl>
        <nl>
        The callback function should satisfy the following prototype (Borland C):<nl>
        <nl>
        <code>void huge socketCallBackFunc(int socketdescriptor, int eventFlags)<codee>
        <nl><nl>
        The callback function's parameter <code>eventFlags<codee> contains the event(s)
        that have occured.<s2>This bit field is encoded the same as the
        <code>eventflagmask<codee> input parameter to this API function.
        <nl><nl>
        Before closing a socket, you must remove the callback function.<s2>
        This is done by calling this API function with a null <code>funcptr<codee>.
        and zero <code>eventflagmask<codee><nl><nl>
 @Also  tcp01.htm | opensocket  |
        tcp02.htm | closesocket |
        tcp7a.htm | Register IP Callback

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x19<linkmid>documentation<linktail>
        for more details.

*/
int RegisterCallbackFunction (int sd, void far *funcptr, int eventflagmask, int *error)
{
    _BX = sd ;
    _CX = eventflagmask ;
    _ES = FP_SEG(funcptr) ;
    _DI = FP_OFF(funcptr) ;

    _AH = API_REGISTERCALLBACK ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX;
}
#endif





/*************************************************************************/
//get last socket error
/*************************************************************************/
#ifdef _tcp21_
/*
 @doc DOC_tcp21_
 ---------------------------------------------------------------
 Get last socket error
 ---------------------------------------------------------------
 @clib  get_socketerror | Get last socket error. |
        <cfont>int get_socketerror ( int sd );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @rdesc <linkto>../../iperrlst.htm<linkmid>error<linktail> code
 @comm  Returns the last error which occured on specified socket.
 @Also  tcp01.htm | opensocket  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x21<linkmid>documentation<linktail>
        for more details.

*/
int get_socketerror (int sd)
{
    _BX = sd ;
    _AH = API_GET_SOCKETERROR ;
    asm { int TCPIPVECT  } ;

    return _AX ;
}
#endif


/*********************************************************************************/
// CHIP-RTOS with SNMP support only:
// search for tcp socket by given local and return state,remote ip and local port
/*********************************************************************************/
#ifdef _tcp22_
/*
 @doc DOC_tcp22_
 ---------------------------------------------------------------
 Find TCP socket and return state
 ---------------------------------------------------------------
 @clib  Get_TCP_Socket_State | Find TCP socket and return its state. |
        <cfont>unsigned char Get_TCP_Socket_State ( unsigned int localPort,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab><tab> unsigned long far *remoteIP,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab><tab> unsigned int *remotePort );<efont> |
 @parm  int  | localPort  | Local port (e.g. htons(23) for telnet).
 @parm  int  | remoteIP  | Output parameter:<s2>Pointer to storage for
        32 bit remote IP address.
 @parm  int  | remotePort  | Output parameter:<s2>Pointer to storage for
        16 bit remote peer port number.
 @rdesc Socket state (see list below)
 @comm  Search for a TCP socket by a given local port number, returning the
        socket state, IP address and port number of the remote peer (if any).<nl>
        <nl>
        This function is only available in @CHIP-RTOS versions which contain
        the SNMP MIB feature.<nl>
        <nl>
        If the return value is not INVALID (=20) and is greater or equal to SYN_SENT (=2)
        then a TCP connection is established.<s2>In this case the storage at
        <code>remoteIP<codee> holds the 32 bit IP address of the connected remote peer
        and <code>remotePort<codee> contains the remote peer port number.

        Possible TCP socket states:<nl>
        @raw
        CLOSED          0
        LISTEN          1
        SYN_SENT        2
        SYN_RECEIVED    3
        ESTABLISHED     4
        CLOSE_WAIT      5
        FIN_WAIT_1      6
        CLOSING         7
        LAST_ACK        8
        FIN_WAIT_2      9
        TIME_WAIT      10
        INVALID        20
 @Also  tcp01.htm | opensocket |
        tcphton.htm | htons    | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x22<linkmid>documentation<linktail>
        for more details.

*/
unsigned char Get_TCP_Socket_State (unsigned int localPort,
                                    unsigned long far * remoteIP,
                                    unsigned int * remotePort)
{
    unsigned char state ;

    _BX = localPort ;
    _ES = FP_SEG(remoteIP) ;
    _DI = FP_OFF(remoteIP) ;

    _AH = API_GET_TCP_STATE ;
    asm { int TCPIPVECT  } ;

    state = _AL ;
    *remotePort = _CX ;

    return state ;
}
#endif


#ifdef _tcp23_
/*
 @doc DOC_tcp23_
 ---------------------------------------------------------------
 Get information about all open sockets
 ---------------------------------------------------------------
 @clib  int FindAllOpenSockets | Find/return information about all open sockets at the internal socket table. |
        <cfont>int FindAllOpenSockets(SocketInfo far * SockInfoPtr, int max);<efont> |
 @parm  void *  | SockInfoPtr  | Output Parameter:<s2> Pointer to an array of max
        <cfont>SocketInfo<efont> <linkto>../../tcpstruc.htm#SocketInfo<linkmid>structures<linktail>
        in the user's memory space into which this API will report the socket information.
 @parm  int | max | Size of the array (maximum number of table entries reported).

 @rdesc Returns the number of all open sockets.<s2>
        The user provided array holds the socket information
        for up to <cfont>max<efont> sockets (the lesser of two values counts: Return, max).<nl>

 @comm  This function is only available in @CHIP-RTOS version which contain the SNMP MIB feature.

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x23<linkmid>documentation<linktail>
        for more details.
        This function is only available in @CHIP-RTOS version which contain the SNMP MIB feature.
*/
int FindAllOpenSockets(SocketInfo far * SockInfoPtr, int max)
{
   asm{
          mov ah,API_FINDALL_SOCKETS
          mov cx,max
          les di,SockInfoPtr
          int TCPIPVECT
   }
   return _AX;
}
#endif

/*************************************************************************/
//Register an IP callback function
/*************************************************************************/

#ifdef _tcp7A_

/*
 @doc DOC_tcp7a_
 ---------------------------------------------------------------
 Register an IP user callback handler
 ---------------------------------------------------------------
 @clib  RegisterIPCallbackHandler | Register an IP User callback handler. |
        <cfont>int RegisterIPCallbackHandler ( void far *funcptr );<efont> |
 @parm  int  | funcptr  | Pointer to user's IP callback Handler.
 @rdesc 0 = success <nl>
 @comm  This function allows the application programmer to install a ip packet
        filter callback function. The application programmer can implement a
        function from the type described below:<nl>
        <nl>
        typedef int (huge *IPSortOutFuncPtr)(
          <linkto>../../tcpstruc.htm#IpUserCallbackInfo<linkmid>IpUserCallbackInfo_t<linktail> *ipInfo );<nl>
        <nl>
        If a function of this type is installed by the user, the TCPIP stack
        will call this function at any incoming ip packet. Inside of this
        function the user is able to check the given IP parameters and decide,
        if the TCPIP stack should process this packet or ignore it.
        If the callback function returns -1 the incoming packet will be ignored
        by the TCPIP stack.<nl>
        <nl>
        To uninstall the callback function, this API call must be called with
        a null pointer.<nl>
        !Do not forget to uninstall the function, if your application exits!
 @Also  tcp19.htm | Register TCP Callback |  |  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x7A<linkmid>documentation<linktail>
        for more details.


*/
int RegisterIPCallbackHandler(void far *funcptr)
{
    _ES = FP_SEG(funcptr) ;
    _DI = FP_OFF(funcptr) ;

    _AH = IP_CALLBACK;
    asm { int TCPIPVECT  } ;
    return _DX;
}
#endif




/*************************************************************************/
//Register ARP  callback function
//(requires RTOS 1.10B or higher)
/*************************************************************************/
#ifdef _tcp7B_
/*
 @doc DOC_tcp7b_
 ---------------------------------------------------------------
 Register an ARP user callback handler
 ---------------------------------------------------------------
 @clib  Register_ARP_User_CB| Register an ARP User callback handler. |
        <cfont>void Register_ARP_User_CB(void far * funcPtr);<efont> |
 @parm  int  | funcptr  | Pointer to ARP callback function (or set to null to remove
                         a previously installed callback)
 @rdesc returns nothing<nl>
 @comm
        The application programmer can implement a function from the type described below:<nl>
        <nl>
        <tab><cfont>typedef int (huge *MyArpCallbackFuncPtr)(<nl><tab><tab><tab><tab>
        <linkto>../../tcpstruc.htm#ArpUserCallbackInfo<linkmid>ArpUserCallbackInfo_t<linktail>
        far *arpInfo );<efont><nl>
        <nl>
        If a function of this type is installed by the user, the TCP/IP stack will call this
        function for any incoming ARP packet.<s2> The input parameter to the callback function
        allows access to the ARP packet.<s2> Inside of this function the user is able to check the content
        of the incoming ARP packet and decide whether the TCP/IP stack should process this packet
        or ignore it.<s2> If the callback function returns -1, the incoming packet will be ignored
        by the TCP/IP stack.<nl>
        <nl>
        For structure of an ARP packet see <linkto>tcpstruc.htm#ArpHeader<linkmid>ArpHeader<linktail>.<nl>
        <nl>
        To remove the callback function, this function call must be called with a null pointer.<s2>
        <nl><bold><i>Do not forget to uninstall the callback, if your application exits!<bolde><ie><nl>
        <nl>
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x7B<linkmid>documentation<linktail>
        for more details.
 @Also  tcp19.htm | Register TCP Callback |
        tcp7A.htm | RegisterIPCallbackHandler |  |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x7B<linkmid>documentation<linktail>
        for more details.

*/
void Register_ARP_User_CB(void far * funcPtr)
{
  asm{
        les di,funcPtr
        mov ah,ARP_USER_CB
        int TCPIPVECT
  }
}
#endif
/*************************************************************************/
//PPP client installed
/*************************************************************************/
#ifdef _tcp40_
/*
 @doc DOC_tcp40_
 ---------------------------------------------------------------
 Check if PPP client installed
 ---------------------------------------------------------------
 @clib  PPP_Client_Installed | Check if PPP client is installed. |
        <cfont>int PPP_Client_Installed ( void );<efont> |
 @parm  int  |   | -- none --
 @rdesc 0:   PPP client is not installed<nl>
        non-zero: PPP client is installed<nl>
 @comm  Tests if PPP client services are available in @CHIP-RTOS
        version.
 @Also  tcp41.htm | PPP_Client_Open  |
        tcp70.htm | Get_Installed_Servers | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x40<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_Installed (void)
{
    _AH = PPPCLIENT_INSTALLED ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif
/*************************************************************************/
//PPP client open
/*************************************************************************/
#ifdef _tcp41_
/*
 @doc DOC_tcp41_
 ---------------------------------------------------------------
 Open a PPP connection
 ---------------------------------------------------------------
 @clib  PPP_Client_Open | Open a PPP connection. |
        <cfont>int PPP_Client_Open ( PPPClient_Init far *ptr, int *error );<efont> |
 @parm  int  | ptr  | Pointer to a <code>PPPClient_Init<codee>
        <linkto>../../tcpstruc.htm#PPPClient_Init<linkmid>type<linktail>
        data structure (declared in tcpipapi.h)
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  On success, the structure at <code>ptr<codee> contains the needed
        IP data for further TCP/IP socket communication.<nl>
        <nl>
        Refer to the PPPCLIE.C example for how to use this API.<s2>
        Also refer to the <code>PPPClient_Init<codee>
        <linkto>../../tcpstruc.htm#PPPClient_Init<linkmid>type<linktail>
        data structure documentation.<nl>
        <nl>
        Note: <bold>Only one PPP client connection can be opened
        at a time!!<bolde><nl>
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x41<linkmid>documentation<linktail>
        for more details.
 @Also  tcp40.htm | PPP_Client_Installed  |
        tcp42.htm | PPP_Client_Close  |
        tcp43.htm | PPP_Client_GetStatus
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x41<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_Open (PPPClient_Init far * ptr, int *error)
{
    _ES = FP_SEG(ptr) ;
    _DI = FP_OFF(ptr) ;
    _AH = PPPCLIENT_OPEN ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;
}
#endif


/*************************************************************************/
//PPP client close
/*************************************************************************/
#ifdef _tcp42_
/*
 @doc DOC_tcp42_
 ---------------------------------------------------------------
 Close a PPP client connection
 ---------------------------------------------------------------
 @clib  PPP_Client_Close | Close a PPP client connection. |
        <cfont>int PPP_Client_Close ( int * error );<efont> |
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  At the close call the PPP client also executes after closing
        the PPP session the modem hang-up commands from the struct
        <code>PPPClient_Init<codee>
        <linkto>../../tcpstruc.htm#PPPClient_Init<linkmid>type<linktail>
        defined in the <linkto>tcp41.htm<linkmid>PPP_Client_Open<linktail>
        call.<nl>
 @Also  tcp41.htm | PPP_Client_Open  |  |  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x42<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_Close (int *error)
{
    _AH = PPPCLIENT_CLOSE ;
    asm { int TCPIPVECT  } ;
    *error = _AX;
    return _DX ;
}
#endif

/*************************************************************************/
//PPP client get status
/*************************************************************************/
#ifdef _tcp43_
/*
 @doc DOC_tcp43_
 ---------------------------------------------------------------
 Get PPP client status
 ---------------------------------------------------------------
 @clib  PPP_Client_GetStatus | Get PPP client status. |
        <cfont>int PPP_Client_GetStatus ( int *error );<efont> |
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @rdesc non-negative:  PPP client
        <linkto>../../iperrlst.htm#status<linkmid>status<linktail> code<nl>
        negative:  Failure, PPP client
        <linkto>../../iperrlst.htm#client<linkmid>error<linktail> code
 @comm  This API function returns -2 if the version of RTOS does not
        support the PPP client services.
 @Also  tcp41.htm | PPP_Client_Open  |
        tcp40.htm | PPP_Client_Installed | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x43<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_GetStatus (int *error)
{
    int status ;

    _DX = 0;
    _AH = PPPCLIENT_GET_STATUS ;
    asm { int TCPIPVECT  } ;
    *error = _DX;
    return _AX;
}
#endif

/*************************************************************************/
//PPP client get DNS IP as negotiated by the remote PPP server
/*************************************************************************/
#ifdef _tcp44_
/*
 @doc DOC_tcp44_
 ---------------------------------------------------------------
 Get DNS IP
 ---------------------------------------------------------------
 @clib  PPP_Client_Get_DNSIP | Get DNS IP. |
        <cfont>int PPP_Client_Get_DNSIP ( unsigned long far *IPaddress,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int primary_sec, int *error );<efont> |
 @parm  int  | IPaddress  | Output parameter:<s2>DNS address is stored here.
 @parm  int  | primary_sec  | 1:<s2> Get primary DNS address<nl>
        2:<s2> get secondary address.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  This API function outputs the DNS addresses as negotiated by
        the remote PPP server.
 @Also  tcp41.htm | PPP_Client_Open  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x44<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_Get_DNSIP (unsigned long far * IPaddress,
                          int primary_sec,
                          int *error)
{
    _BX = primary_sec ;
    _ES = FP_SEG(IPaddress);
    _DI = FP_OFF(IPaddress);

    _AH = PPPCLIE_GET_DNSIP ;
    asm { int TCPIPVECT  } ;
    *error = _DX;
    return _AX ;
}
#endif

/*************************************************************************/
// PPP client set options
/*************************************************************************/
#ifdef _tcp45_
/*
 @doc DOC_tcp45_
 ---------------------------------------------------------------
 Set PPP options for the PPP client
 ---------------------------------------------------------------
 @clib  PPP_Client_SetOptions | Set PPP options for the PPP client. |
        <cfont>int PPP_Client_SetOptions ( const PPP_Option far *ptr );<efont> |
 @parm  int  | ptr  | Pointer to an array of
        <code>PPP_Option<codee>
        <linkto>../../tcpstruc.htm#PPP_Option<linkmid>type<linktail>
        data structure (declared in tcpipapi.h).
 @rdesc -3 : PPP Connection already established<nl>
        -2 : PPP client is not installed<nl>
        -1 : Invalid Option(s)<nl>
        0<s2> : Success
        @comm If you want to use this function, you have to call it before opening
        a connection.<s2> <bold>Setting options during an established connection
        does not work!<bolde><s2>
        The settings are only valid for the next established connection.<s2>
        While establishing a connection the <code>PPP_Option<codee> structure
        referenced by the <code>ptr<codee> parameter must remain valid.<s2>
        After the
        <linkto>tcp41.htm<linkmid>PPP_Client_Open<linktail> function is finished, the structure
        can then be released.
        <nl>
        <nl>
        The array of data structures at <code>ptr<codee> is terminated by
        the first structure encountered with a NULL <code>optionValuePtr<codee>
        member.<nl>
        <nl>

        @raw
        // Simplified C-Example:
        unsigned long DNS_Pri_IP  = 0L;   // allow remote to peer to set primary DNS IP
        unsigned long DNS_Sec_IP  = 0L;   // allow remote to peer to set secondary DNS IP
        unsigned int  ipcp_comp   = 1;    // allow remote to peer to use VJ TCPIP header compression

        PPP_Option My_Options[] = {
            { PPP_IPCP_PROTOCOL, PPP_OPTION_ALLOW, PPP_IPCP_COMP_PROTOCOL, (const char *)&ipcp_comp, sizeof(ipcp_comp)},
            { PPP_IPCP_PROTOCOL, PPP_OPTION_WANT, PPP_IPCP_DNS_PRI, (const char *)&DNS_Pri_IP, sizeof(DNS_Pri_IP)},
            { PPP_IPCP_PROTOCOL, PPP_OPTION_WANT, PPP_IPCP_DNS_SEC, (const char *)&DNS_Sec_IP, sizeof(DNS_Sec_IP)},
            { 0, 0, 0, NULL, 0}
        } ;

        //***** call the functions like this ****

        // Install option with CLIB function
        PPP_Client_SetOptions(&My_Options[0]);  // Point to first member of array

        PPP_Client_Open(&pppclient);           // Open the connection

 @Also  tcp41.htm | PPP_Client_Open   | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x45<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Client_SetOptions (const PPP_Option far *ptr)
{
    _ES = FP_SEG(ptr);
    _DI = FP_OFF(ptr);

    _AH = PPPCLIENT_SET_OPTIONS ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif



/*************************************************************************/
//PPP server installed
/*************************************************************************/
#ifdef _tcp50_
/*
 @doc DOC_tcp50_
 ---------------------------------------------------------------
 Check if PPP server is installed
 ---------------------------------------------------------------
 @clib  PPP_Server_Installed | Check if PPP server is installed. |
        <cfont>int PPP_Server_Installed ( void );<efont> |
 @parm  int  |   | -- none --
 @rdesc zero: PPP server is not installed<nl>
        non-zero: PPP server is installed
 @Also  tcp52.htm | PPP_Server_Activate  |
        tcp70.htm |Get_Installed_Servers | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x50<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_Installed (void)
{
    _AH = PPPSERVER_INSTALLED ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif


/*************************************************************************/
//PPP server suspend
/*************************************************************************/
#ifdef _tcp51_
/*
 @doc DOC_tcp51_
 ---------------------------------------------------------------
 Suspend the PPP server task
 ---------------------------------------------------------------
 @clib  PPP_Server_Suspend | Suspend the PPP server task. |
        <cfont>int PPP_Server_Suspend (int timeoutsecs, int *error );<efont> |
 @parm  int  | timeoutsecs  | Time-out in seconds.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  Note that the time-out value in <code>timeoutsecs<codee>
        depends on your time-out entries for the modem commands
        specified in chip.ini.<s2> If this call returns with -1, the most
        likely reason is that the modem commands are not finished after
        specified time-out.
 @related Chip.ini   | ../../config.htm#PPPSERVERHANGUPTIMEOUTx
        | entry: | PPPSERVER HANGUPTIMEOUTx
        time-out in seconds for wait on answer from modem
 @Also  tcp52.htm | PPP_Server_Activate  |
        tcp53.htm | PPP_Server_GetStatus | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x51<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_Suspend (int timeoutsecs, int *error)
{
    _BX = timeoutsecs ;
    _AH = PPPSERVER_SUSPEND ;
    asm { int TCPIPVECT  } ;
    *error = _DX;
    return _AX ;
}
#endif

/*************************************************************************/
//PPP server  activate
/*************************************************************************/
#ifdef _tcp52_
/*
 @doc DOC_tcp52_
 ---------------------------------------------------------------
 Activate the PPP server
 ---------------------------------------------------------------
 @clib  PPP_Server_Activate | Activate the PPP server. |
        <cfont>int PPP_Server_Activate ( int timeoutsecs, int *error );<efont> |
 @parm  int  | timeoutsecs  | Time-out in seconds.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0 = success <nl>
        Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @comm  If successful, the PPP server is now able to serve a connection.<nl>
        <nl>
        Note that the time-out value in <code>timeoutsecs<codee>
        depends on your time-out entries for the modem commands
        specified in chip.ini.<s2> If this call returns with -1, the most
        likely reason is that the modem commands are not finished after
        specified time-out.
 @related Chip.ini   | ../../config.htm#PPPSERVERHANGUPTIMEOUTx
        | entry: | PPPSERVER HANGUPTIMEOUTx
        time-out in seconds for wait on answer from modem
 @Also  tcp51.htm | PPP_Server_Suspend  |
        tcp50.htm | PPP_Server_Installed  |
        tcp53.htm | PPP_Server_GetStatus
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x52<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_Activate (int timeoutsecs, int *error)
{
    _BX = timeoutsecs ;
    _AH = PPPSERVER_ACTIVATE ;
    asm { int TCPIPVECT  } ;
    *error = _DX;
    return _AX ;
}
#endif

/*************************************************************************/
//PPP server, get status
/*************************************************************************/
#ifdef _tcp53_
/*
 @doc DOC_tcp53_
 ---------------------------------------------------------------
 Get the current state of the PPP server
 ---------------------------------------------------------------
 @clib  PPP_Server_GetStatus | Get the current state of the PPP server. |
        <cfont>int PPP_Server_GetStatus ( void );<efont> |
 @parm  int  |   | -- none --
 @rdesc PPP server state defined below
 @comm  PPP server states:<nl>
        <ul>
        -2<s2>  PPP server is not installed.<nl>
        -1<s2>  Error state, should not happen<nl>
        1<s3>  Server disabled<nl>
        2<s3>  Server enabled, waiting for connection<nl>
        3<s3>  PPP connection is established<nl>
        4<s3>  Server tries to hang up modem<nl>
        5<s3>  Server tries to initialize modem<nl>
        <ul>
 @Also  tcp51.htm | PPP_Server_Suspend  |
        tcp52.htm | PPP_Server_Activate  |
        tcp50.htm | PPP_Server_Installed
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x53<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_GetStatus(void)
{
    _AH = PPPSERVER_GET_STATUS ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif


/*************************************************************************/
//PPP server  get config
/*************************************************************************/
#ifdef _tcp54_
/*
 @doc DOC_tcp54_
 ---------------------------------------------------------------
 Get configuration data of the PPP server
 ---------------------------------------------------------------
 @clib  PPP_Server_Get_Cfg | Get the current main configuration
        data of the PPP server . |
        <cfont>int PPP_Server_Get_Cfg ( PPP_IPCfg_Data far *ptr, int *error );<efont> |
 @parm  int  | ptr  | Pointer to <code>PPP_IPCfg_Data<codee>
        <linkto>../../tcpstruc.htm#PPP_IPCfg_Data<linkmid>type<linktail>
        data structure which will be output to by this function.
 @parm  int  | error | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc Non-zero = Failure (see <code>error<codee> output parameter)<nl>
        zero: The user structure <code>PPP_IPCfg_Data<codee> is filled with
        the PPP server configuration data<nl>
 @Also  tcp51.htm | PPP_Server_Suspend  |
        tcp53.htm | PPP_Server_GetStatus  |
        tcp52.htm | PPP_Server_Activate
 @Also  tcp55.htm | PPP_Server_SetOptions | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x54<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_Get_Cfg (PPP_IPCfg_Data far *ptr, int *error)
{
    _ES = FP_SEG(ptr);
    _DI = FP_OFF(ptr);
    _AH = PPPSERVER_GET_CFG ;
    asm { int TCPIPVECT  } ;
    *error = _DX;
    return _AX ;
}
#endif

/*************************************************************************/
// PPP server set options
/*************************************************************************/
#ifdef _tcp55_
/*
 @doc DOC_tcp55_
 ---------------------------------------------------------------
 Set PPP options for the PPP server
 ---------------------------------------------------------------
 @clib  PPP_Server_SetOptions | Set PPP options for the PPP server. |
        <cfont>int PPP_Server_SetOptions ( const PPP_Option far *ptr );<efont> |
 @parm  byte | ptr  | Pointer to an array of <code>PPP_Option<codee>
        <linkto>../../tcpstruc.htm#PPP_Option<linkmid>type<linktail>
        data structures (defined in tcpipapi.h).
 @rdesc -3: PPP Connection already established<nl>
        -2: PPP server is not installed<nl>
        -1: Invalid Option(s)<nl>
        0:<s2> Success
 @comm  It is only possible to set PPP options when the PPP server is suspended.<s2>
        (PPP server status reported by
        <linkto>tcp53.htm<linkmid>PPP_Server_GetStatus<linktail> must be 1.)<s2>
        Settings options when the PPP server is active
        (state 2,3,4,5) does not have any effect.<s2>
        The installed options take affect when the PPP server is activated.<s2>
        If you want to reset the options call this function with a
        null <code>ptr<codee> parameter.<nl>
        <nl>
        The array of data structures at <code>ptr<codee> is terminated by
        the first structure encountered with a NULL <code>optionValuePtr<codee>
        member.<nl>

        @raw

        //  Simplified C-Example:
        unsigned long DNS_Pri_IP  = 0L;   // allow remote to peer to set primary DNS IP, but do no request for it
        unsigned long DNS_Sec_IP  = 0L;   // allow remote to peer to set secondary DNS IP , but do no request for it
        unsigned int  ipcp_comp   = 1;    // allow remote to peer to use VJ TCPIP header compression

        static PPP_Option My_Options[] = {
           { PPP_IPCP_PROTOCOL, PPP_OPTION_ALLOW, PPP_IPCP_COMP_PROTOCOL, (const char *)&ipcp_comp, sizeof(ipcp_comp)},
           { PPP_IPCP_PROTOCOL, PPP_OPTION_ALLOW, PPP_IPCP_DNS_PRI, (const char *)&DNS_Pri_IP, sizeof(DNS_Pri_IP)},
           { PPP_IPCP_PROTOCOL, PPP_OPTION_ALLOW, PPP_IPCP_DNS_SEC, (const char *)&DNS_Sec_IP, sizeof(DNS_Sec_IP)},
           { 0, 0, 0, NULL, 0}   // End of array marker
           } ;

        //*****call the functions like this****

        PPP_Server_Suspend(20);                 // Suspend the PPP server
        //install option with CLIB function
        PPP_Server_SetOptions(&My_Options[0]);  // point to first member of array

        PPP_Server_Activate(20);                // Activate

 @Also  tcp51.htm | PPP_Server_Suspend  |
        tcp53.htm | PPP_Server_GetStatus  |
        tcp52.htm | PPP_Server_Activate
 @Also  tcp54.htm | PPP_Server_Get_Cfg  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x55<linkmid>documentation<linktail>
        for more details.

*/
int PPP_Server_SetOptions (const PPP_Option far * ptr)
{
    _ES = FP_SEG(ptr);
    _DI = FP_OFF(ptr);
    _AH = PPPSERVER_SET_OPTIONS ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif


/*************************************************************************/
//Get pointer to internal TCIP SNMP mib structures
/*************************************************************************/
#ifdef _tcp60_
/*
 @doc DOC_tcp60_
 ---------------------------------------------------------------
 Get internal TCPIP SNMP variables
 ---------------------------------------------------------------
 @clib  Get_SNMP_Data | Get internal TCPIP SNMP variables. |
        <cfont>int Get_SNMP_Data ( unsigned char which, void far * *snmp_mib_ptr );<efont> |
 @parm  int  | which  |  1   Get pointer to
        <linkto>../../tcpstruc.htm#tagIfMib<linkmid>IfMib<linktail>
        data structure<nl>
         2   Get pointer to
         <linkto>../../tcpstruc.htm#tagIpMib<linkmid>IpMib<linktail>
         data structure<nl>
         3   Get pointer to
         <linkto>../../tcpstruc.htm#tagIcmpMib<linkmid>IcmpMib<linktail>
         data structure<nl>
         4   Get pointer to
         <linkto>../../tcpstruc.htm#tagTcpMib<linkmid>TcpMib<linktail>
         data structure<nl>
         5   Get pointer to
         <linkto>../../tcpstruc.htm#tagUdpMib<linkmid>UdpMib<linktail>
         data structure<nl>
         6   Get pointer to
         <linkto>../../tcpstruc.htm#tag_atEntry<linkmid>atEntry<linktail>
         data structure<nl>
 @rdesc non-zero : BIOS without internal SNMP mib variables<nl>
        0:  Location refereced by <code>snmp_mib_ptr<codee> contains a pointer
        to the structure<nl>
 @comm  Note:<s2> These structures are only available at @CHIP-RTOS versions,
        which contain the SNMP option.<s2> A SNMP agent is not part of the @CHIP-RTOS.
        <s2> But if a user is able to implement an agent based on
        the TCPIP API, they need access to the internal TCPIP SNMP
        variables.<s2>
        The SNMP variables are not a part of our current official 6 @CHIP-RTOS
        versions. You must order direct a @CHIP-RTOS version,
        which includes this feature.<nl>
 @Also  tcp78.htm | Get_TCPIP_Memory_Status  |
        tcp74.htm | Get_TCPIP_Statistics  | |

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x60<linkmid>documentation<linktail>
        for more details.

 @devnote The access to struct tag_atEntry is currently not supported.<nl>
*/
int Get_SNMP_Data(unsigned char which , void far * * snmp_mib_ptr)
{
    unsigned int es_reg ;
    _AL = which ;
    _AH = API_SNMP_GET ;
    asm { int TCPIPVECT  } ;
    if(_AX==0)
    {
       es_reg = _ES ;
       *snmp_mib_ptr = (void far *)MK_FP(es_reg, _DI) ;
       return 0;
    }
    return _AX;
}
#endif


#ifdef _tcp60a_
/*
 @doc DOC_tcp60a_
 ---------------------------------------------------------------
 Get internal TCPIP SNMP variables
 ---------------------------------------------------------------
 @clib  Get_SNMP_DataEx | Get internal TCPIP SNMP variables (Extended). |
        <cfont>int Get_SNMP_DataEx ( unsigned char which, void far * *snmp_mib_ptr,<nl>
        <tab><tab><tab><tab><tab>&nbsp; int *max );<efont> |
 @parm  int  | which  |  1   Get pointer to
        <linkto>../../tcpstruc.htm#tagIfMib<linkmid>IfMib<linktail>
        data structure<nl>
         2   Get pointer to
         <linkto>../../tcpstruc.htm#tagIpMib<linkmid>IpMib<linktail>
         data structure<nl>
         3   Get pointer to
         <linkto>../../tcpstruc.htm#tagIcmpMib<linkmid>IcmpMib<linktail>
         data structure<nl>
         4   Get pointer to
         <linkto>../../tcpstruc.htm#tagTcpMib<linkmid>TcpMib<linktail>
         data structure<nl>
         5   Get pointer to
         <linkto>../../tcpstruc.htm#tagUdpMib<linkmid>UdpMib<linktail>
         data structure<nl>
         6   Get pointer to
         <linkto>../../tcpstruc.htm#tag_atEntry<linkmid>atEntry<linktail>
         data structure<nl>
         7   Get array of pointers to
         <linkto>../../tcpstruc.htm#tagIfMib<linkmid>IfMib<linktail>
         entries of all current open TCPIP device interfaces (Local Loopback,
         Ethernet, PPP server, PPP client). <cfont>max<efont> must contain
         the max. number of pointers at this array.<nl>
         8 Get pointer to unsigned long sysuptime variable (10Hz counter).<nl>
 @rdesc non-zero : RTOS version  without internal SNMP mib variables<nl>
        0:  Location refereced by <code>snmp_mib_ptr<codee> contains a pointer
        to the structure<nl>
        If parameter <cfont>which<efont> == 7 the user provided array (addressed by <cfont>snmp_mib_ptr<efont>)
        contains the pointers to the <linkto>../../tcpstruc.htm#tagIfMib<linkmid>IfMib<linktail> entries of the
        current installed device interfaces. <cfont>max<efont> holds the number of all available entries.
 @comm  Note:<s2> These structures are only available at @CHIP-RTOS versions,
        which contain the SNMP option.<s2> A SNMP agent is not part of the @CHIP-RTOS.
        <s2> But if a user is able to implement an agent based on
        the TCPIP API, they need access to the internal TCPIP SNMP
        variables.<s2>
        The SNMP variables are not a part of our current official 6 @CHIP-RTOS
        versions. You must order direct a @CHIP-RTOS version,
        which includes this feature.<nl>
 @Also  tcp78.htm | Get_TCPIP_Memory_Status  |
        tcp74.htm | Get_TCPIP_Statistics  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x60<linkmid>documentation<linktail>
        for more details.

 @devnote The access to struct tag_atEntry is currently not supported.<nl>

*/
int Get_SNMP_DataEx(unsigned char which , void far * * snmp_mib_ptr, int * max)
{
   unsigned int es_reg;
   if(which==7)
   {
      _CX = *max;
      asm push ds
      asm lds  si,snmp_mib_ptr
   }
   asm{
         mov al,which
         mov ah,API_SNMP_GET
         int TCPIPVECT
   }
   if(_AX==0)
   {
      if(which!=7)
      {
        es_reg = _ES;
        *snmp_mib_ptr = (void *)MK_FP(es_reg,_DI);
      }
      else
      {
        *max          =  _CX;
        asm pop ds
      }
      return 0;
   }
   return _AX;
}
#endif

/*************************************************************************/
//FTP server: Return the addresss of the FTPserver login counters
/*************************************************************************/
#ifdef _tcp65_
/*
 @doc DOC_tcp65_
 ---------------------------------------------------------------
 Access the FTP server login counters
 ---------------------------------------------------------------
 @clib1 Get_FTP_Login_Counters | Access the FTP server login counters. |
        <cfont>int Get_FTP_Login_Counters ( unsigned long far * * FTP_Login_Count,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned long far * * FTP_Login_failCount );<efont> |
 @parm  int  | FTP_Login_Count  | Output parameter:<s2>  A far pointer to the
        32 bit login counter will be output to the location pointed to by this
        parameter.<s2>  On failure, a NULL pointer is output here.
 @parm  int  | FTP_Login_failCount  | Output parameter:<s2>  A far pointer to the
        32 bit login fail counter will be output to the location pointed to by
        this parameter.<s2>  On failure, a NULL pointer is output here.
 @rdesc 0:  success <nl>
        non-zero:  @CHIP-RTOS doesn't support FTP server
        pointers are set to NULL).
 @Also  tcp66.htm | Get_Telnet_Login_Counters  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x65<linkmid>documentation<linktail>
        for more details.

*/
int Get_FTP_Login_Counters(unsigned long far *  * FTP_Login_Count,
                           unsigned long far *  * FTP_Login_failCount)
{
    int error_code ;
    unsigned int es_reg, ds_reg ;

    _AH = API_FTP_GET_LOGIN ;
    asm { push DS } ;
    asm { int TCPIPVECT  } ;
    error_code = _AX ;
    ds_reg = _DS ;
    es_reg = _ES ;
    asm { pop DS } ;

    if (error_code != 0)
    {
        es_reg = 0 ;
        ds_reg = 0 ;
        _DI = 0 ;
        _SI = 0 ;
    }
    *FTP_Login_Count     = (unsigned long far *)MK_FP(es_reg, _DI);
    *FTP_Login_failCount = (unsigned long far *)MK_FP(ds_reg, _SI);

    return error_code ;
}
#endif

/*************************************************************************/
//Telnet server: Return the addresss of the Telnetserver login counters
/*************************************************************************/
#ifdef _tcp66_
/*
 @doc DOC_tcp66_
 ---------------------------------------------------------------
 Access the Telnet server login counters
 ---------------------------------------------------------------
 @clib1 Get_Telnet_Login_Counters | Access the Telnet server login counters. |
        <cfont>int Get_Telnet_Login_Counters(unsigned long far * * Telnet_Login_Count,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp; unsigned long far * * Telnet_Login_failCount);<efont> |
 @parm  int  | Telnet_Login_Count  | Output parameter:<s2>  A far pointer to the
        32 bit login counter will be output to the location pointed to by this
        parameter.<s2>  On failure, a NULL pointer is output here.
 @parm  int  | Telnet_Login_failCount  | Output parameter:<s2>  A far pointer to the
        32 bit login fail counter will be output to the location pointed to by
        this parameter.<s2>  On failure, a NULL pointer is output here.
 @rdesc 0:  success <nl>
        non-zero:  @CHIP-RTOS doesn't support Telnet server
        (pointers are set to NULL).
 @Also  tcp65.htm | Get_FTP_Login_Counters  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x66<linkmid>documentation<linktail>
        for more details.

*/
int Get_Telnet_Login_Counters(unsigned long far *  * Telnet_Login_Count,
                              unsigned long far *  * Telnet_Login_failCount)
{
    int error_code ;
    unsigned int es_reg, ds_reg ;

    _AH = API_TELNET_GET_LOGIN ;
    asm { push DS } ;
    asm { int TCPIPVECT  } ;
    error_code = _AX ;
    ds_reg = _DS ;
    es_reg = _ES ;
    asm { pop DS } ;

    if (error_code != 0)
    {
        es_reg = 0 ;
        ds_reg = 0 ;
        _DI = 0 ;
        _SI = 0 ;
    }
    *Telnet_Login_Count     = (unsigned long far *)MK_FP(es_reg, _DI);
    *Telnet_Login_failCount = (unsigned long far *)MK_FP(ds_reg, _SI);

    return error_code ;
}
#endif

/*************************************************************************/
//Get telnet state
/*************************************************************************/
#ifdef _tcp67_
/*
 @doc DOC_tcp67_
 ---------------------------------------------------------------
 Check if have an active telnet session
 ---------------------------------------------------------------
 @clib  Get_Telnet_State | Check if have an active telnet session. |
        <cfont>int Get_Telnet_State ( int *error );<efont> |
 @parm   int  | error    | Output parameter:<s2>
         0:  no Telnet session<nl>
         1:  Telnet session is active<nl>
        -1:  BIOS doesn't support Telnet server
 @rdesc  0 = success <nl>
         Non-zero = Failure (see <code>error<codee> output parameter)<nl>
 @Also  tcp66.htm | Get_Telnet_Login_Counters  |
        tcp70.htm | Get_Installed_Servers  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x67<linkmid>documentation<linktail>
        for more details.

*/
int Get_Telnet_State ( int *error)
{
    _AH = API_GET_TELNET_STATE ;
    asm { int TCPIPVECT  } ;

    *error = _DX;
    return _AX;
}
#endif



/*************************************************************************/
//Get installed servers and devices of the SC12
/*************************************************************************/
#ifdef _tcp70_
/*
 @doc DOC_tcp70_
 ---------------------------------------------------------------
 Get information about running servers
 ---------------------------------------------------------------
 @clib  Get_Installed_Servers | Get information about running servers
        and interfaces of the TCP/IP Stack. |
        <cfont>void Get_Installed_Servers ( unsigned int far* first,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned int far* second );<efont> |
 @parm  int  | first    | The first eight possible available servers:<s2>
         Bit 0: GIS_ETHERNET - Ethernet device<nl>
         Bit 1: GIS_PPP_SERVER  - PPP server<nl>
         Bit 2: GIS_PPP_CLIENT - PPP client<nl>
         Bit 3: GIS_WEB_SERVER - Web server<nl>
         Bit 4: GIS_TELNET_SERVER - Telnet server<nl>
         Bit 5: GIS_FTP_SERVER - FTP server<nl>
         Bit 6: GIS_TFTP_SERVER - TFTP server<nl>
         Bit 7: GIS_DHCP - DHCP client<nl>
         <nl>
 @parm  int  | second    | The next eight possible available servers:<s2>
         Bit 0: GIS_SNMP_MIB - SNMP MIB variables support<nl>
         Bit 1: GIS_CFG_SERVER - UDP Config server<nl>
         Bit 2: GIS_PING_CLIENT - Ping client<nl>
 @rdesc -- none --
 @comm  The masked listed here are defined in tcpip.h header file.<nl>
 @Also  tcp50.htm | PPP_Server_Installed  |
        tcp40.htm | PPP_Client_Installed  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x70<linkmid>documentation<linktail>
        for more details.

*/
void Get_Installed_Servers ( unsigned int far *first, unsigned int far *second )
{
    _AH = GET_INSTALLED_SERVERS ;
    asm { int TCPIPVECT  } ;
    *first = _AX;
    *second = _DX;
}
#endif

/*************************************************************************/
//Reconfigure ethernet interface
/*************************************************************************/
#ifdef _tcp71_
/*
 @doc DOC_tcp71_
 ---------------------------------------------------------------
 Reconfigure Ethernet interface
 ---------------------------------------------------------------
 @clib  Reconfigure_ethernet | Reconfigure Ethernet interface after
        changing the IP configuration. |
        <cfont>int Reconfigure_ethernet ( void );<efont> |
 @parm  int  |   | -- none --.
 @rdesc 0:  success <nl>
        non-zero:  error code,<nl>
        Error code 237 indicates that an Ethernet interface configuration was
        already in progress
 @comm  A new IP configuration set with the prompt
        <linkto>../../command.htm#IP_address<linkmid>commands<linktail>
        <cfont>ip<efont>, <cfont>netmask<efont> and
        <cfont>gateway<efont><nl>
        (or the corresponding library
        <linkto>tcpIPW.htm<linkmid>Set_IPConfig<linktail> call)
        becomes valid after a successful call to this function.<nl>
        <nl>
        If DHCP is changed from 1 to 0 then a new IP address, subnet mask
        and gateway should be set with the prompt
        <linkto>../../command.htm#IP_address<linkmid>commands<linktail>
        <cfont>ip<efont>, <cfont>netmask<efont> and <cfont>gateway<efont>
        or <linkto>tcpIPW.htm<linkmid>Set_IPConfig<linktail>
        library call.
 @Also  tcpIPW.htm | Set_IPConfig  |
        tcp72.htm | DHCP_use  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x71<linkmid>documentation<linktail>
        for more details.

*/
int Reconfigure_ethernet(void)
{
    _AH = REINIT_ETHERNET ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif

/*************************************************************************/
//DHCP use   dhcp_use 0/1
/*************************************************************************/
#ifdef _tcp72_
/*
 @doc DOC_tcp72_
 ---------------------------------------------------------------
 Set/Reset DHCP usage on the Ethernet interface
 ---------------------------------------------------------------
 @clib  DHCP_use | Set/Reset DHCP usage on the Ethernet interface. |
        <cfont>int DHCP_use ( unsigned char dhcp_use );<efont>  |
 @parm  int  | dhcp_use  | 0: do not use DHCP, 1: Use DHCP.
 @rdesc 0:  success <nl>
        non-zero:  DHCP not available in this RTOS version.
 @comm  This entry becomes valid only after rebooting the system  or after
        calling the
        <linkto>tcp71.htm<linkmid>Reconfigure_ethernet<linktail>
        library function.<nl>
        <nl>
        If DHCP is changed from 1 to 0 then a new IP address, subnet mask
        and gateway should be set with either the
        prompt <linkto>../../command.htm#IP_address<linkmid>commands<linktail>
        or the <linkto>tcpIPW.htm<linkmid>Set_IPConfig<linktail>
        library call.<nl>
 @Also  tcp71.htm  | Reconfigure_ethernet  |
        tcpIPW.htm | Set_IPConfig  |
        tcp73.htm  | Get_DHCP_Status
 @Also  tcp73a.htm | Get_DHCP_Status_Ext  | | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x72<linkmid>documentation<linktail>
        for more details.

*/
int DHCP_use(unsigned char dhcp_use)
{
    _AL = dhcp_use ;
    _AH = DHCP_USE ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif

/*************************************************************************/
//DHCP Stat
/*************************************************************************/
#ifdef _tcp73_
/*
 @doc DOC_tcp73_
 ---------------------------------------------------------------
 Get DHCP status of the Ethernet interface
 ---------------------------------------------------------------
 @clib  Get_DHCP_Status | Get DHCP status of the Ethernet interface. |
        <cfont>void Get_DHCP_Status ( unsigned int *dhcp_use, unsigned int *dhcp_stat );<efont> |
 @parm  int  | dhcp_use  | Output parameter:<s2>Pointer to a where
        a "DHCP in use" boolean value will be output to.<s2>The
        location will be set to '1' if the DHCP is currently being
        used and '0' if not in use.
 @parm  int  | dhcp_stat  | Output parameter:<s2>Pointer to a where
        a DHCP status word will be stored.<s2> The status is encoded
        as follows:<nl>
        <ul>
          0: System is not configured (is in progress)<nl>
          1: System is configured by a DHCP Server<nl>
          2: System configure retry failed (or no retry started before)<nl>
        <ule>
 @rdesc  -- none --
 @Also  tcp73a.htm | Get_DHCP_Status_Ext  |
        tcp72.htm  | DHCP_use | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x73<linkmid>documentation<linktail>
        for more details.

*/
void Get_DHCP_Status(unsigned int * dhcp_use, unsigned int *dhcp_stat)
{
    _AH = DHCP_STAT ;
    asm { int TCPIPVECT  } ;
    *dhcp_use  = _AX ;
    *dhcp_stat = _DX ;
}
#endif


/*************************************************************************/
//DHCP Stat extended function with getting dhcp config data
//(requires BIOS 1.03B or higher)
/*************************************************************************/
#ifdef _tcp73a_
/*
 @doc DOC_tcp73a_
 ---------------------------------------------------------------
 Get DHCP status of the Ethernet interface
 ---------------------------------------------------------------
 @clib  Get_DHCP_Status_Ext | Get DHCP status of the Ethernet interface. |
        <cfont>void Get_DHCP_Status_Ext ( unsigned int *dhcp_use,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned int *dhcp_stat,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; UserEthDhcp_Entry far * *dhcpptr );<efont> |
 @parm  int  | dhcp_use  | Output parameter:<s2>Pointer to a where
        a "DHCP in use" boolean value will be output to.<s2>The
        location will be set to '1' if the DHCP is currently being
        used and '0' if not in use.
 @parm  int  | dhcp_stat  | Output parameter:<s2>Pointer to a where
        a DHCP status word will be stored.<s2> The status is encoded
        as follows:<nl>
        <ul>
          0: System is not configured (is in progress)<nl>
          1: System is configured by a DHCP Server<nl>
          2: System configure retry failed (or no retry started before)<nl>
        <ule>
 @parm  int | dhcpptr | DHCP Config Data:<s2>Pointer to Pointer to a UserEthDhcp_
        Entry struct. The struct contains some DHCP Config data (read Only!).
        If no data available the pointer will be assigned to NULL.
 @rdesc -- none --
 @comm  The UserEthDhcp_Entry data is a read-only information.<s2>
        Do not write to this data structure!
 @Also  tcp73.htm | Get_DHCP_Status  |
        tcp72.htm | DHCP_use | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x73<linkmid>documentation<linktail>
        for more details.

*/
void Get_DHCP_Status_Ext(unsigned int * dhcp_use, unsigned int *dhcp_stat, UserEthDhcp_Entry far * * dhcpptr)
{
    unsigned int es_reg;
    unsigned int di_reg;

    _AH = DHCP_STAT ;
    asm { int TCPIPVECT  } ;
    es_reg = _ES;
    di_reg = _DI;
    *dhcp_use  = _AX ;
    *dhcp_stat = _DX ;
    if (_DX != 1)
    {
        es_reg =
        di_reg = 0 ;
    }

    *dhcpptr = (UserEthDhcp_Entry far *) MK_FP(es_reg, di_reg);
    return;
}
#endif
/*************************************************************************/
//return the address of TCPIP packet counter struct
/*************************************************************************/
#ifdef _tcp74_
/*
 @doc DOC_tcp74_
 ---------------------------------------------------------------
 Access the internal network packet counter
 ---------------------------------------------------------------
 @clib  Get_TCPIP_Statistics | Access the internal network packet counters. |
        <cfont>void Get_TCPIP_Statistics ( Packet_Count far * *pkt_cnt_ptr );<efont> |
 @parm  int | pkt_cnt_ptr | Packet COunter:<s2>Pointer to Pointer to a
        Packet_Count struct. The struct contains the counters for each
        protocol (read Only!).
 @rdesc -- none --
 @comm Note:<s2> The counters referenced by pointer members
       <code>count_all_packets<codee> and <code>count_all_sended packets<codee>
       count only Ethernet packets.<nl>
       <nl>
       Other counters referenced also count the packets from and
       to other devices: e.g. local loop back packets and PPP packets.<nl>
       <nl>
       The user is free to read and/or reset any of these referenced
       counters.<nl>
 @Also tcp78.htm | Get_TCPIP_Memory_Status  |
       tcp60.htm | Get_SNMP_Data  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x74<linkmid>documentation<linktail>
        for more details.

*/
void Get_TCPIP_Statistics ( Packet_Count far * * pkt_cnt_ptr )
{
    unsigned es_reg;
    _ES = 0 ;
    _DI = 0 ;       // [ES:DI] remains null if no TCP/IP in RTOS.

    _AH = TCPIP_STATISTICS ;
    asm { int TCPIPVECT  } ;
    es_reg = _ES;
    *pkt_cnt_ptr = (Packet_Count far *)MK_FP(es_reg, _DI);
}
#endif

/*************************************************************************/
//ping open
/*************************************************************************/
#ifdef _tcp75_
/*
 @doc DOC_tcp75_
 ---------------------------------------------------------------
 Open and start ICMP echo requests
 ---------------------------------------------------------------
 @clib  Ping_Open | Open and start ICMP echo requests. |
        <cfont>int Ping_Open ( Ping far *pingptr, int *error );<efont> |
 @parm  int  | pingptr  | Pointer to user's  <code>Ping<codee>
        <linkto>../../tcpstruc.htm#Ping<linkmid>structure<linktail>.<nl>
        The contents of this structure will be initialized by this
        function call.
        <nl>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc -2: Ping API not supported by RTOS version<nl>
        -1:  Failure, see reported <code>error<codee> code.<nl>
        >= 0:  Socket descriptor<nl>
 @comm  Note:<s2> This API will send ICMP packets forever, until you
        call <linkto>tcp76.htm<linkmid>Ping_Close<linktail>.
 @Also  tcp76.htm | Ping_Close  |
        tcp77.htm | Ping_Statistics | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x75<linkmid>documentation<linktail>
        for more details.

*/
int Ping_Open (Ping far * pingptr, int * error)
{
    _ES = FP_SEG(pingptr);
    _DI = FP_OFF(pingptr);
    _AH = PING_OPEN ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
//ping close
/*************************************************************************/
#ifdef _tcp76_
/*
 @doc DOC_tcp76_
 ---------------------------------------------------------------
 Finish ICMP echo requests
 ---------------------------------------------------------------
 @clib  Ping_Close | Stop ICMP echo requests. |
        <cfont>int Ping_Close ( int sd );<efont> |
 @parm  int  | sd  | Socket descriptor.
 @rdesc -2: Ping API not supported by RTOS<nl>
        -1: Ping close failed, should not happen, only if socket
        descriptor is invalid<nl>
        0: Success<nl>
 @Also  tcp75.htm | Ping_Open |
        tcp77.htm | Ping_Statistics  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x76<linkmid>documentation<linktail>
        for more details.

*/
int Ping_Close (int sd)
{
    _BX = sd ;
    _AH = PING_CLOSE ;
    asm { int TCPIPVECT  } ;
    return _DX ;
}
#endif

/*************************************************************************/
//ping statistics
/*************************************************************************/
#ifdef _tcp77_
/*
 @doc DOC_tcp77_
 ---------------------------------------------------------------
 Retrieve ping information
 ---------------------------------------------------------------
 @clib  Ping_Statistics | Retrieve ping information. |
        <cfont>int Ping_Statistics ( Ping far *pingptr );<efont> |
 @parm  int  | pingptr  | Output parameter:<s2>
        Pointer to user's  <code>Ping<codee>
        <linkto>../../tcpstruc.htm#Ping<linkmid>structure<linktail>
        from the
        <linkto>tcp75.htm<linkmid>Ping_Open<linktail> call.<nl>
 @rdesc -2: Ping API not supported by RTOS version<nl>
        -1: Failure<nl>
        0: Success<nl>
 @comm  The structure at <code>pingptr<codee> is filled with
        the ping statistics.<nl>
 @Also  tcp75.htm | Ping_Open  |
        tcp76.htm | Ping_Close | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x77<linkmid>documentation<linktail>
        for more details.

*/
int Ping_Statistics(Ping far * pingptr)
{
    _ES = FP_SEG(pingptr);
    _DI = FP_OFF(pingptr);
    _AH = PING_STATISTICS ;
    asm { int TCPIPVECT  } ;
    return _DX ;
}
#endif

/*************************************************************************/
//memory
/*************************************************************************/
#ifdef _tcp78_
/*
 @doc DOC_tcp78_
 ---------------------------------------------------------------
 Get information about TCP/IP stack memory usage
 ---------------------------------------------------------------
 @clib  Get_TCPIP_Memory_Status | Get information about TCP/IP stack
        memory usage. |
        <cfont>void Get_TCPIP_Memory_Status ( unsigned long far *total,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned long far *used );<efont> |
 @parm  int  | total  | Output parameter:<s2>Pointer to 32 bit location
        what will receive the maximum number of bytes of available memory.
 @parm  int  | used  | Output parameter:<s2>Pointer to 32 bit location
        what will receive the number of bytes of memory currently used.
 @rdesc -- none --
 @comm  This library function uses the @CHIP-RTOS TCP/IP software
        interrupt 0xAC service
        <linkto>../../tcpipapi.htm#0xAC0x78<linkmid>0x78<linktail>.<s2>
        Refer to the documentation for this API function for more details.
 @Also  tcp60.htm | Get_SNMP_Data  |
        tcp74.htm | Get_TCPIP_Statistics  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x78<linkmid>documentation<linktail>
        for more details.

*/
void Get_TCPIP_Memory_Status(unsigned long far * total,
                             unsigned long far * used)
{
    asm { push DS } ;
    _ES = FP_SEG(total);
    _DI = FP_OFF(total);
    _DS = FP_SEG(used);
    _SI = FP_OFF(used);

    _AH = GET_MEMORY_INFO ;
    asm { int TCPIPVECT  } ;
    asm { pop DS } ;
}
#endif


/*************************************************************************/
//Set FTP Server Idle timeout (using: Set/Get Server Idle Timeout)
/*************************************************************************/
#ifdef _tcp79_
/*
 @doc DOC_tcp79_
 ---------------------------------------------------------------
 Set FTP server time-out
 ---------------------------------------------------------------
 @clib  Set_FTPServer_Idle_Timeout | Set FTP server time-out. |
        <cfont>int Set_FTPServer_Idle_Timeout ( unsigned int timeout );<efont> |
 @parm  int  | timeout  | Time out value in seconds.
 @rdesc 0:  success <nl>
        non-zero:  FTP server not available in this RTOS version
 @comm  This function inserts the specified FTP time-out into the
        CHIP.INI file.<s2> The new time-out value takes affect
        immediately, without requiring a reboot.<nl>
        <nl>
        Refer to
        <linkto>../../config.htm#FTP_TIMEOUT<linkmid>FTP_TIMEOUT<linktail>
        in CHIP.INI documentation.
 @Also  tcp79a.htm | Set_TelnetServer_Idle_Timeout  |
        tcp79b.htm | Get_FTPServer_Idle_Timeout |
        tcp79c.htm | Get_TelnetServer_Idle_Timeout
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x79<linkmid>documentation<linktail>
        for more details.

*/
int Set_FTPServer_Idle_Timeout ( unsigned int timeout )
{
    _BX = 0x00;                 // Ftp server
    _DX = timeout;

    _AL = 0x00;                 // set timeout
    _AH = SET_SERVER_IDLE_TIMEOUT ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif

/*************************************************************************/
//Set Telnet Server Idle timeout (using: Set/Get Server Idle Timeout)
/*************************************************************************/
#ifdef _tcp79a_
/*
 @doc DOC_tcp79a_
 ---------------------------------------------------------------
 Set Telnet server time-out
 ---------------------------------------------------------------
 @clib  Set_TelnetServer_Idle_Timeout | Set Telnet server time-out. |
        <cfont>int Set_TelnetServer_Idle_Timeout ( unsigned int timeout );<efont> |
 @parm  int  | timeout  | Time out in seconds.
 @rdesc 0:  success <nl>
        non-zero:  Telnet server not available in this RTOS version
 @comm  This function inserts the specified Telnet time-out into the
        CHIP.INI file.<s2> The new time-out value takes affect
        immediately, without requiring a reboot.<nl>
        <nl>
        Refer to
        <linkto>../../config.htm#TELNET_TIMEOUT<linkmid>TELNET_TIMEOUT<linktail>
        in CHIP.INI documentation.
 @Also  tcp79.htm  | Set_FTPServer_Idle_Timeout   |
        tcp79c.htm | Get_TelnetServer_Idle_Timeout |
        tcp79b.htm | Get_FTPServer_Idle_Timeout
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x79<linkmid>documentation<linktail>
        for more details.

*/
int Set_TelnetServer_Idle_Timeout( unsigned int timeout )
{
    _BX = 0x01;                 // Telnet server
    _DX = timeout;

    _AL = 0x00;                 // set timeout
    _AH = SET_SERVER_IDLE_TIMEOUT ;
    asm { int TCPIPVECT  } ;
    return _AX ;
}
#endif

/*************************************************************************/
//Get FTP Server Idle timeout (using: Set/Get Server Idle Timeout)
/*************************************************************************/
#ifdef _tcp79b_
/*
 @doc DOC_tcp79b_
 ---------------------------------------------------------------
 Get FTP server time-out value
 ---------------------------------------------------------------
 @clib  Get_FTPServer_Idle_Timeout | Get FTP server time-out value. |
        <cfont>unsigned int Get_FTPServer_Idle_Timeout ( void );<efont> |
 @parm  int  |   | -- none --
 @rdesc 0xFFFF or 0xFFFE:<s2>FTP server not available in this RTOS version<nl>
        Otherwise:<s2> Existing FTP server time-out value, in seconds.
 @comm  This function accesses the  FTP server's existing time-out value.<nl>
        <nl>
        Refer to
        <linkto>../../config.htm#FTP_TIMEOUT<linkmid>FTP_TIMEOUT<linktail>
        in CHIP.INI documentation.
 @Also  tcp79.htm  | Set_FTPServer_Idle_Timeout   |
        tcp79a.htm | Set_TelnetServer_Idle_Timeout |
        tcp79c.htm | Get_TelnetServer_Idle_Timeout
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x79<linkmid>documentation<linktail>
        for more details.

*/
unsigned int Get_FTPServer_Idle_Timeout (void)
{
    _BX = 0x00;                 // FTP server

    _AL = 0x01;                 // get timeout
    _AH = SET_SERVER_IDLE_TIMEOUT ;
    asm { int TCPIPVECT  } ;
    return _DX ;
}
#endif

/*************************************************************************/
// Get Telnet Server Idle timeout (using: Set/Get Server Idle Timeout)
/*************************************************************************/
#ifdef _tcp79c_
/*
 @doc DOC_tcp79c_
 ---------------------------------------------------------------
 Get Telnet server time-out value
 ---------------------------------------------------------------
 @clib  Get_TelnetServer_Idle_Timeout | Get Telnet server time-out value. |
        <cfont>unsigned int Get_TelnetServer_Idle_Timeout ( void );<efont> |
 @parm  int  |   | -- none --.
 @rdesc 0xFFFF or 0xFFFE:<s2>Telnet server not available in this RTOS version<nl>
        Otherwise:<s2> Existing Telnet server time-out value, in seconds.
 @comm  This function accesses the  Telnet server's existing time-out value.
        <nl>
        <nl>
        Refer to
        <linkto>../../config.htm#TELNET_TIMEOUT<linkmid>TELNET_TIMEOUT<linktail>
        in CHIP.INI documentation.
 @Also  tcp79a.htm | Set_TelnetServer_Idle_Timeout |
        tcp79.htm  | Set_FTPServer_Idle_Timeout   |
        tcp79b.htm | Get_FTPServer_Idle_Timeout
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x79<linkmid>documentation<linktail>
        for more details.

*/
unsigned int Get_TelnetServer_Idle_Timeout(void)
{
    _BX = 0x01;                 // Telnet server
    _AL = 0x01;                 // get timeout
    _AH = SET_SERVER_IDLE_TIMEOUT ;
    asm { int TCPIPVECT  } ;
    return _DX ;
}
#endif
/*************************************************************************/
// Add gateway
/*************************************************************************/
#ifdef _tcp80_
/*
 @doc DOC_tcp80_
 ---------------------------------------------------------------
 Add the default gateway
 ---------------------------------------------------------------
 @clib  AddDefaultGateway | Add the default gateway. |
        <cfont>int AddDefaultGateway ( IP_INTERFACE which_interface,<nl>
        <tab><tab><tab><tab><tab><tab>const unsigned long far *Gateway,<nl>
        <tab><tab><tab><tab><tab><tab>int *error );<efont> |
 @parm  int  | which_interface  | Enumeration type:<nl>
        <ul>
        <li>II_ETHERNET = 0:<s2>  Ethernet
        <li>II_PPP_SERVER = 1:<s2>  PPP server
        <li>II_PPP_CLIENT = 2:<s2>  PPP client
        <ule>
 @parm  int  | Gateway  | Pointer to gateway IP address.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  If this function is used, the
        <linkto>../../config.htm#IP_GATEWAY<linkmid>gateway<linktail>
        entry in the <cfont>CHIP.INI<efont> becomes invalidated, but
        unchanged.<s2> If the PPP server or PPP client is specified by the
        <code>which_interface<codee> parameter, then the gateway is set
        to the remote peer IP address.
 @Also  tcp81.htm | DelDefaultGateway  |
        tcp82.htm |  GetDefaultGateway | |
 @Also  tcp83.htm | AddStaticRoute  |
        tcp84.htm | DelStaticRoute  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x80<linkmid>documentation<linktail>
        for more details.

 @related IP | ../../config.htm#IP_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
 @related PPP server | ../../config.htm#PPPSERVER_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
*/
int  AddDefaultGateway (IP_INTERFACE which_interface,
                        const unsigned long far * Gateway,
                        int * error)
{
    _BX = which_interface ;
    _ES = FP_SEG(Gateway);
    _DI = FP_OFF(Gateway);
    _AH = ADD_DEFAULT_GATEWAY ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif


#ifdef _tcp80a_
/*
 @doc DOC_tcp80a_
 ---------------------------------------------------------------
 Add the default gateway
 ---------------------------------------------------------------
 @clib  AddDefaultGatewayEx | Add the default gateway (Extended). |
        <cfont>int AddDefaultGatewayEx ( IP_INTERFACE which_interface,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; unsigned long far *Gateway,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; void far *userIfaceHandle,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; int *error );<efont> |
 @parm  int  | which_interface  | Enumeration type:<nl>
        <ul>
        <li>II_ETHERNET = 0:<s2>  Ethernet
        <li>II_PPP_SERVER = 1:<s2>  PPP server
        <li>II_PPP_CLIENT = 2:<s2>  PPP client
        <li>II_USER_DEVICE = 3:<s2>  user device driver
        <ule>
 @parm  int  | Gateway  | Pointer to gateway IP address.
 @parm  int  | userIfaceHandle | If <cfont>which_interface<efont>==3,
        Pointer to user device handle.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  If this function is used, the
        <linkto>../../config.htm#IP_GATEWAY<linkmid>gateway<linktail>
        entry in the <cfont>CHIP.INI<efont> becomes invalidated, but
        unchanged.<s2> If the PPP server or PPP client is specified by the
        <code>which_interface<codee> parameter, then the gateway is set
        to the remote peer IP address.
 @Also  tcp81.htm | DelDefaultGateway  |
        tcp82.htm |  GetDefaultGateway | |
 @Also  tcp83.htm | AddStaticRoute  |
        tcp84.htm | DelStaticRoute  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x80<linkmid>documentation<linktail>
        for more details.

 @related IP | ../../config.htm#IP_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
 @related PPP server | ../../config.htm#PPPSERVER_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
*/
int  AddDefaultGatewayEx (IP_INTERFACE which_interface,
                          const unsigned long far * Gateway,
                          const void  far * userIfaceHandle,
                          int * error)
{
    _BX = which_interface ;
    _ES = FP_SEG(Gateway);
    _DI = FP_OFF(Gateway);
    _DX = FP_SEG(userIfaceHandle);
    _SI = FP_OFF(userIfaceHandle);
    _AH = ADD_DEFAULT_GATEWAY ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
// Delete gateway
/*************************************************************************/
#ifdef _tcp81_
/*
 @doc DOC_tcp81_
 ---------------------------------------------------------------
 Delete the default gateway
 ---------------------------------------------------------------
 @clib  DelDefaultGateway | Delete the default gateway. |
        <cfont>int DelDefaultGateway ( const unsigned long far *Gateway,<nl>
        <tab><tab><tab><tab><tab><tab>int *error );<efont> |
 @parm  int  | Gateway  | Pointer to gateway IP address to be
        removed.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  If this function is used, the
        <linkto>../../config.htm#IP_GATEWAY<linkmid>gateway<linktail>
        entry in the <cfont>CHIP.INI<efont> becomes invalidated, but
        unchanged.
 @Also  tcp80.htm | AddDefaultGateway |
        tcp82.htm | GetDefaultGateway | |
 @Also  tcp83.htm | AddStaticRoute  |
        tcp84.htm | DelStaticRoute  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x81<linkmid>documentation<linktail>
        for more details.

 @related IP | ../../config.htm#IP_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
 @related PPP server | ../../config.htm#PPPSERVER_GATEWAY | Gateway |
    <cfont>CHIP.INI<efont> Configuration
*/
int  DelDefaultGateway (const unsigned long far * Gateway, int * error)
{
    _ES = FP_SEG(Gateway);
    _DI = FP_OFF(Gateway);
    _AH = DEL_DEFAULT_GATEWAY ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
// Get gateway
/*************************************************************************/
#ifdef _tcp82_
/*
 @doc DOC_tcp82_
 ---------------------------------------------------------------
 Get the current default gateway
 ---------------------------------------------------------------
 @clib  GetDefaultGateway | Get the current default gateway. |
        <cfont>int GetDefaultGateway ( unsigned long far *Gateway,<nl>
        <tab><tab><tab><tab><tab><tab>int *error );<efont> |
 @parm  int  | Gateway  | Output parameter:<s2>Pointer to where
        the gateway IP will be written.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @Also  tcp80.htm | AddDefaultGateway  |
        tcp81.htm | DelDefaultGateway  | |
 @Also  tcp83.htm | AddStaticRoute  |
        tcp84.htm | DelStaticRoute  | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x82<linkmid>documentation<linktail>
        for more details.

*/
int  GetDefaultGateway (unsigned long far * Gateway, int * error)
{
    _ES = FP_SEG(Gateway);
    _DI = FP_OFF(Gateway);
    _AH = GET_DEFAULT_GATEWAY ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
// Add a static route
/*************************************************************************/
#ifdef _tcp83_
/*
 @doc DOC_tcp83_
 ---------------------------------------------------------------
 Add a route for a interface
 ---------------------------------------------------------------
 @clib  AddStaticRoute | Add a route for a interface. |
        <cfont>int AddStaticRoute ( IP_INTERFACE which_interface,<nl>
        <tab><tab><tab><tab><tab> const Route_Entry far *route,<nl>
        <tab><tab><tab><tab><tab> int *error );<efont> |
 @parm  int  | which_interface  | Enumeration type:<nl>
        <ul>
        <li>II_ETHERNET = 0:<s2>  Ethernet
        <li>II_PPP_SERVER = 1:<s2>  PPP server
        <li>II_PPP_CLIENT = 2:<s2>  PPP client
        <ule>
 @parm  int  | route  | Pointer to user  <code>Route_Entry<codee>
        <linkto>../../tcpstruc.htm#Route_Entry<linkmid>structure<linktail>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  The <code>Route_Entry<codee> structure is defined in tcpipapi.h:<nl>
        <nl>Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x83<linkmid>documentation<linktail>
        for more details.
 @Also  tcp84.htm | DelStaticRoute  | | | |
 @Also  tcp80.htm | AddDefaultGateway |
        tcp81.htm | DelDefaultGateway |
        tcp82.htm | GetDefaultGateway
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x83<linkmid>documentation<linktail>
        for more details.

*/
int  AddStaticRoute (IP_INTERFACE which_interface,
                     const Route_Entry far * route,
                     int * error)
{
    _BX = which_interface;
    _ES = FP_SEG(route);
    _DI = FP_OFF(route);
    _AH = ADD_STATIC_ROUTE ;
    asm { int TCPIPVECT  } ;
    *error = _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
// Delete a static route
/*************************************************************************/
#ifdef _tcp84_
/*
 @doc DOC_tcp84_
 ---------------------------------------------------------------
 Delete a route from an interface
 ---------------------------------------------------------------
 @clib  DelStaticRoute | Delete a route from an interface. |
        <cfont>int DelStaticRoute ( IP_INTERFACE which_interface,<nl>
        <tab><tab><tab><tab><tab> const Route_Entry far *route,<nl>
        <tab><tab><tab><tab><tab> int *error );<efont> |
 @parm  int  | which_interface  | Enumeration type:<nl>
        <ul>
        <li>II_ETHERNET = 0:<s2>  Ethernet
        <li>II_PPP_SERVER = 1:<s2>  PPP server
        <li>II_PPP_CLIENT = 2:<s2>  PPP client
        <ule>
 @parm  int  | route  | Pointer to user  <code>Route_Entry<codee>
        <linkto>../../tcpstruc.htm#Route_Entry<linkmid>structure<linktail>
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  Only <code>route<codee>structure members <code>destIPAddress<codee>
        and <code>destNetmask<codee> must be set valid before calling.<nl>
 @Also  tcp83.htm | AddStaticRoute  | | | |
 @Also  tcp80.htm | AddDefaultGateway  |
        tcp81.htm | DelDefaultGateway |
        tcp82.htm | GetDefaultGateway
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x84<linkmid>documentation<linktail>
        for more details.

*/
int  DelStaticRoute (IP_INTERFACE which_interface,
                     const Route_Entry far * route,
                     int * error)
{
    _BX = which_interface;
    _ES = FP_SEG(route);
    _DI = FP_OFF(route);
    _AH = DEL_STATIC_ROUTE ;
    asm { int TCPIPVECT  } ;
    *error= _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
// Delete ARP entry by physical address
/*************************************************************************/
#ifdef _tcp88_
/*
 @doc DOC_tcp88_
 ---------------------------------------------------------------
 Delete ARP entry by physical address
 ---------------------------------------------------------------
 @clib  DelArpEntryByPhys | Delete ARP entry by physical address. |
        <cfont>int DelArpEntryByPhys ( const unsigned char far *phy, int *error );<efont> |
 @parm  unsigned char far *  | phy | Pointer to the physical address (6 Byte)
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.

 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA8<linkmid>documentation<linktail>
        for more details.
 @Also  tcp89.htm | AddArpEntry  |
        tcp8A.htm | GetARPRouteCacheEntries | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x88<linkmid>documentation<linktail>
        for more details.

*/
int DelArpEntryByPhys(const unsigned char far *phy, int *error)
{
    _ES = FP_SEG(phy);
    _DI = FP_OFF(phy);
    _AH = DEL_ARP_ENTRY_BY_PHYS;
    asm { int TCPIPVECT  } ;
    *error= _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
// Add ARP entry
/*************************************************************************/
#ifdef _tcp89_
/*
 @doc DOC_tcp89_
 ---------------------------------------------------------------
 Add ARP entry
 ---------------------------------------------------------------
 @clib  AddArpEntry | Add an entry in the ARP table. |
        <cfont>int AddArpEntry ( const unsigned long far *ip,<nl>
        <tab><tab><tab><tab>&nbsp; const unsigned char far *phy,<nl>
        <tab><tab><tab><tab>&nbsp; int *error );<efont> |
 @parm  unsigned long far *  | ip  | Pointer to the IP address
 @parm  unsigned char far *  | phy | Pointer to the physical address (6 Byte)
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @Also  tcp88.htm | DelArpEntryByPhys  |
        tcp8A.htm | GetARPRouteCacheEntries  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x89<linkmid>documentation<linktail>
        for more details.

*/
int AddArpEntry(const unsigned long far *ip, const unsigned char far *phy, int *error)
{
    _BX = FP_SEG(ip);
    _SI = FP_OFF(ip);
    _ES = FP_SEG(phy);
    _DI = FP_OFF(phy);
    _AH = ADD_ARP_ENTRY;
    asm { int TCPIPVECT  } ;
    *error= _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
// Get ARP cache copy
/*************************************************************************/
#ifdef _tcp8A_
/*
 @doc DOC_tcp8A_
 ---------------------------------------------------------------
 Read ARP/Route cache table
 ---------------------------------------------------------------
 @clib  GetARPRouteCacheEntries |This function returns all valid entries in the internal ARP/Route cache table.|
        <cfont>int GetARPRouteCacheEntries(ArpRouteCacheEntry far * arpuserptr);<efont> |
 @parm  void *  | ArpCacheEntry  | <s2> Pointer to a user provided array of 64 of
            data <linkto>../../tcpstruc.htm#ArpRouteCacheEntry<linkmid>structures<linktail>
            for storing ARP/Route cache table.
 @rdesc  Number of valid entries at the table<nl>
         The user provided array contains the current ARP/Route cache table.<nl>
 @Also  tcp88.htm | DelArpEntryByPhys  |
        tcp89.htm | AddArpEntry  |   |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x8A<linkmid>documentation<linktail>
        for more details.

*/

int GetARPRouteCacheEntries(ArpRouteCacheEntry far * arpuserptr)
{
    _BX = FP_SEG(arpuserptr);
    _SI = FP_OFF(arpuserptr);
    _AH = GET_ARPROUTE_CACHE;
    asm { int TCPIPVECT  } ;
    return _CX ;   //number of valid entries at arpuser array
}
#endif


/*************************************************************************/
//interface information
/*************************************************************************/
#ifdef _tcp8D_
/*
 @doc DOC_tcp8D_
 ---------------------------------------------------------------
 Read table of TCP/IP device interfaces
 ---------------------------------------------------------------
 @clib  Get_Iface_Entries |This function returns  information about all installed TCP/IP device interfaces.|
        <cfont>int Get_Iface_Entries(Iface_Entry far * iface_entry, int max);<efont> |
 @parm  void *  | IfaceEntry  | <s2> Pointer to a user provided array of
             <cfont>Iface_Entry<efont> data <linkto>../../tcpstruc.htm#Iface_Entry<linkmid>structures<linktail>
             for storing the information.
 @parm  int   | max | Number of data structures in array.<s2> Up to this number of
               device interfaces will be reported.
 @rdesc  Number of valid entries at the table<nl>
         The user provided array contains the current device interface information<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x8D<linkmid>documentation<linktail>
        for more details.

*/
int Get_Iface_Entries(Iface_Entry far * iface_entry, int max)
{
    _BX = FP_SEG(iface_entry);
    _SI = FP_OFF(iface_entry);
    _CX = max;
    _AH = GET_IFACE_ENTRIES;
    asm { int TCPIPVECT  } ;
    return (int)_AX ;   //number of valid entries at iface_entry array
}

#endif

/*************************************************************************/
// Install an IP multicast address, join a multicast group
/*************************************************************************/
#ifdef _tcp90_
/*
 @doc DOC_tcp90_
 ---------------------------------------------------------------
 Install an IP multicast address entry
 ---------------------------------------------------------------
 @clib  Add_IGMP_Membership | Install an IP multicast address entry. |
        <cfont>int Add_IGMP_Membership ( const unsigned long far *MulticastIP,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; const unsigned char far *MacAddress,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; int *error );<efont> |
 @parm  int  | MulticastIP  | Pointer to multicast IP address.
 @parm  int  | MacAddress  | Pointer to a 6 byte array that
        contains the corresponding Ethernet address.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @Also  tcp91.htm | Drop_IGMP_Membership  |
        tcp92.htm |  IPMulticast_to_MacAddr | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x90<linkmid>documentation<linktail>
        for more details.

*/
int  Add_IGMP_Membership(const unsigned long far * MulticastIP,
                         const unsigned char far * MacAddress,
                         int * error)
{
    asm {
           push DS
           les di,MulticastIP
           lds si,MacAddress
           mov ah, ADD_IGMP_MEMBERSHIP ;
           int TCPIPVECT
           pop ds
    }
    *error = _AX ;
    return _DX ;
}
#endif

/*************************************************************************/
//Delete an IP multicast address, leave a multicast group
/*************************************************************************/
#ifdef _tcp91_
/*
 @doc DOC_tcp91_
 ---------------------------------------------------------------
 Delete an IP multicast address entry
 ---------------------------------------------------------------
 @clib  Drop_IGMP_Membership | Delete an IP multicast address entry. |
        <cfont>int Drop_IGMP_Membership ( const unsigned long far *MulticastIP,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; int *error );<efont> |
 @parm  int  | MulticastIP  | Pointer to multicast IP address
        to be deleted.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @Also  tcp90.htm | Add_IGMP_Membership  |
        tcp92.htm |  IPMulticast_to_MacAddr | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x91<linkmid>documentation<linktail>
        for more details.

*/
int  Drop_IGMP_Membership (const unsigned long far * MulticastIP, int * error)
{
    asm{
          les di,MulticastIP
          mov ah,DROP_IGMP_MEMBERSHIP
          int TCPIPVECT
    }
    *error = _AX ;
    return _DX ;
}
#endif


/*************************************************************************/
//Map IP multicast to Ethernet multicast
/*************************************************************************/
#ifdef _tcp92_
/*
 @doc DOC_tcp92_
 ---------------------------------------------------------------
 Map IP multicast address to Ethernet address
 ---------------------------------------------------------------
 @clib  IPMulticast_to_MacAddr | Map IP multicast address to Ethernet address. |
        <cfont>int IPMulticast_to_MacAddr ( const unsigned long far *MulticastIP,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned char far *MacAddress,<nl>
        <tab><tab><tab><tab><tab><tab><tab> int *error );<efont> |
 @parm  int  | MulticastIP  | Pointer to multicast IP address.
 @parm  int  | MacAddress  | Output Parameter:<s2>Pointer to a 6 byte array
        where the corresponding MAC address will be stored by this function.
 @parm  int  | error    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>error<codee> code.
 @comm  This API function computes the MAC address in the following way:<nl>
        To map an IP multicast address to a corresponding Ethernet multicast
        address, place the low-order 23 bits of the IP multicast address
        into the low order 23 bits of the special Ethernet multicast
        address<nl>
        <tab>01 00 5E 00 00  00<nl>
        <nl>
        e.g. IP multicast address 224.0.0.1 becomes Ethernet
        address<nl>
        <tab>01 00 5E 00 00 01
 @Also  tcp90.htm | Add_IGMP_Membership  |
        tcp91.htm | Drop_IGMP_Membership |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0x92<linkmid>documentation<linktail>
        for more details.

*/
int  IPMulticast_to_MacAddr (const unsigned long far * MulticastIP,
                             unsigned char far * MacAddress,
                             int * error)
{
    asm {
         push DS
         les di, MulticastIP
         lds si, MacAddress
         mov ah,MCASTIP_TO_MACADDR
         int TCPIPVECT
         pop DS
    }
    *error = _AX ;
    return _DX ;
}
#endif

#ifdef _tcpA0_
/*
 @doc DOC_tcpA0_
 ---------------------------------------------------------------
 Install/Add TCPIP device driver interface
 ---------------------------------------------------------------
 @clib  Dev_Open_Interface |  Install user device driver |

        <cfont>Dev_Open_Interface(DevUserDriver far * DriverInfo, int * errorcode);<efont> |

 @parm  int  | DriverInfo  | Pointer to
         <linkto>../../tcpstruc.htm#DevUserDriver<linkmid>user driver information<linktail>.
 @parm  int  | errorcode    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  tcpA1.htm | Close_Open_Interface |
        ../../ip_dev.htm#Install_Dev | Example | - Installing device driver |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA0<linkmid>documentation<linktail>
        for more details.


*/
int  Dev_Open_Interface(DevUserDriver far * DriverInfo,int * errorcode)
{
   asm{
          les di, DriverInfo
          mov ah, DEV_OPEN_IFACE
          int TCPIPVECT
   }
   *errorcode = _AX;
   return (int)_DX;
}
#endif


#ifdef _tcpA1_
/*
@doc DOC_tcpA1_
 ---------------------------------------------------------------
 Close TCPIP device driver interface
 ---------------------------------------------------------------
 @clib  Dev_Close_Interface |  Close user device driver/interface |

        <cfont>int  Dev_Close_Interface(DevUserIfaceHandle DevHandlePtr, int * errorcode);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @parm  int  | errorcode    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  tcpA0.htm | Dev_Open_Interface  |
        ../../ip_dev.htm#Close_Dev | Example | - Closing device driver |
  @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA1<linkmid>documentation<linktail>
        for more details.

*/
int  Dev_Close_Interface(DevUserIfaceHandle DevHandlePtr, int * errorcode)
{
   asm{
        les di,DevHandlePtr
        mov ah,DEV_CLOSE_IFACE
        int TCPIPVECT
   }
   *errorcode=_AX;
   return _DX;
}
#endif

#ifdef _tcpA2_
/*************************************************************************/
/*
@doc DOC_tcpA2_
 ---------------------------------------------------------------
 Receive and process incoming data at the TCP/IP stack
 ---------------------------------------------------------------
 @clib  Dev_Recv_Interface |  Receive and process incoming data at the TCP/IP stack |

        <cfont>int Dev_Recv_Interface(DevUserIfaceHandle DevHandlePtr,int * errorcode);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @parm  int  | errorcode    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  tcpA2.htm | Dev_Recv_Interface  | | | |
 @Also  ../../ip_dev.htm#DeviceReceive_Dev | Example | - function which would be called by this API
        ../../ip_dev.htm#ISR_and_RECVTask | Example | - Receiver Task which calls this API |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA2<linkmid>documentation<linktail>
        for more details.

*/
/*************************************************************************/
int  Dev_Recv_Interface(DevUserIfaceHandle DevHandlePtr, int * errorcode)
{
   asm{
         les di,DevHandlePtr
         mov ah,DEV_RECV_IFACE
         int TCPIPVECT
    }
    *errorcode=_AX;
    return _DX;
}
#endif

#ifdef _tcpA3_
/*
@doc DOC_tcpA3_
 ---------------------------------------------------------------
 Wait for received data.
 ---------------------------------------------------------------
 @clib  Dev_Recv_Wait | Waits for data received signal from Interrupt Service Routine |

        <cfont>int Dev_Recv_Wait(DevUserIfaceHandle DevHandlePtr, int * errorcode);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @parm  int  | errorcode    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.
 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  tcpA2.htm | Dev_Recv_Interface  |
        ../../ip_dev.htm#ISR_and_RECVTask | Example | - Receiver Task |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA3<linkmid>documentation<linktail>
        for more details.

*/
/*************************************************************************/
int Dev_Recv_Wait(DevUserIfaceHandle DevHandlePtr, int * errorcode)
{
   asm{
         les di,DevHandlePtr
         mov ah,DEV_RECV_WAIT
         int TCPIPVECT
    }
    *errorcode=_AX;
    return _DX;
}
#endif

/*************************************************************************/
#ifdef _tcpA4_
/*
@doc DOC_tcpA4_
 ---------------------------------------------------------------
 Signal from ISR
 ---------------------------------------------------------------
 @clib  Dev_Notify_ISR | Notify the TCP/IP stack from inside of an Interrupt Service Routine
          that there is incoming data available and/or that the device has sent a
          frame successfully. |

        <cfont>int Dev_Notify_ISR(DevUserIfaceHandle DevHandlePtr,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned int receivedPackets,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned int sentPackets);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @parm  int  | receivedPackets | Number of available received packets
 @parm  int  | sentPackets     | Number of sent packets completed (provisional)
 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  ../../ip_dev.htm#ISR_and_RECVTask | Example | - User device ISR | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA4<linkmid>documentation<linktail>
        for more details.

*/
int Dev_Notify_ISR(DevUserIfaceHandle DevHandlePtr,
                   unsigned int receivedPackets,
                   unsigned int sentPackets)
{
   asm{
         les di,DevHandlePtr
         mov ah,DEV_NOTIFY_ISR
         mov cx,sentPackets
         mov bx,receivedPackets
         int TCPIPVECT
   }
   return _AX;
}
#endif
/*************************************************************************/
#ifdef _tcpA5_
/*
@doc DOC_tcpA5_
 ---------------------------------------------------------------
 Get a buffer from TCP/IP stack
 ---------------------------------------------------------------
 @clib  Dev_Get_Buffer | Get a buffer from TCP/IP stack's pre-allocated buffer for storing
          incoming data from this device |
        <cfont>int Dev_Get_Buffer(DevUserBufferHandle BufHandlePtr,<nl>
        <tab><tab><tab><tab><tab><tab><tab> char far * far * buffer,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned int length);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @parm  void *  | buffer | Output Parameter:<s2> Pointer to a variable of type
        <linkto>../../tcpstruc.htm#DevUserBufferHandle<linkmid>DevUserBufferHandle<linktail>
        where the user buffer handle is stored.<s2> (This value is for internal use.)
 @parm  int  | length     | Number of bytes buffer required
 @rdesc 0:  success , *buffer points to the beginning of the allocated buffer<nl>
        -1:  Allocation failed, *buffer is a null pointer.
 @Also  ../../ip_dev.htm#DeviceReceive | Receiver | function |  |  |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA5<linkmid>documentation<linktail>
        for more details.

*/
int Dev_Get_Buffer(DevUserBufferHandle BufHandlePtr,
                   char far * far * buffer,
                   unsigned int length)
{
   *buffer=NULL;
   asm{
         push ds
         les di,BufHandlePtr
         mov ah,DEV_GET_BUF
         mov cx,length
         int TCPIPVECT
         push dx
   }
   if(_AX==0)
   {
      *buffer = MK_FP(_DS,_SI);
   }
   asm pop dx
   asm pop ds
   return _DX;
}
#endif

#ifdef _tcpA6_
/*
@doc DOC_tcpA6_
 ---------------------------------------------------------------
 Signal message send complete
 ---------------------------------------------------------------
 @clib  Dev_Send_Complete | This API signals the TCP/IP stack that the final frame in a message
         has been sent.<s2> (Note that a message can be fragmented into
        multiple link layer frames.) |

        <cfont>int Dev_Send_Complete(DevUserIfaceHandle DevHandlePtr);<efont> |

 @parm  void *  | DevHandlePtr  | <linkto>../../tcpstruc.htm#DevUserIfaceHandle<linkmid>Device handle<linktail>
 @rdesc 0:  success
 @Also  ../../ip_dev.htm#DeviceSend | Example | - User's Device Send Function | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA6<linkmid>documentation<linktail>
        for more details.

*/

int Dev_Send_Complete(DevUserIfaceHandle DevHandlePtr)
{
   asm{
         les di,DevHandlePtr
         mov ah,DEV_SND_COMPLETE
         int TCPIPVECT
   }
   return _AX;
}
#endif


#ifdef _tcpA7_
/*
@doc DOC_tcpA7_
 ---------------------------------------------------------------
 Wait for DHCP completion
 ---------------------------------------------------------------
 @clib  Dev_Wait_DHCP_Complete | Waits for completion of IP configuration process by DHCP ("<i>Dynamic Host
        Configuration Protocol<ie>").<s2> |
        <cfont>int  Dev_Wait_DHCP_Complete(DevUserDriver far * DriverInfo,<nl>
        <tab><tab><tab><tab><tab><tab><tab> unsigned int time_s,<nl>
        <tab><tab><tab><tab><tab><tab><tab> int * errorcode);<efont> |

 @parm  void *  | DriverInfo  | Pointer to
         <linkto>../../tcpstruc.htm#DevUserDriver<linkmid>user driver information<linktail>.
 @parm  int  | time_s | Timeout seconds.<s2> This value will depend on the DHCP server used.<s2>
           A minimum value of 15 seconds is recommended.
 @parm  int  | errorcode    | Output parameter:<s2>Failure
        <linkto>../../iperrlst.htm<linkmid>code<linktail>, 0 on success.

 @rdesc 0:  success <nl>
        -1:  See reported <code>errorcode<codee> code.
 @Also  ../../ip_dev.htm#ISR_and_RECVTask | Example | - User's receiver task | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../tcpipapi.htm#0xAC0xA7<linkmid>documentation<linktail>
        for more details.

*/
int  Dev_Wait_DHCP_Complete(DevUserDriver far * DriverInfo, unsigned int time_s, int * errorcode)
{
   asm{
         mov cx,time_s
         les di,DriverInfo
         mov ah, DEV_WAIT_DHCP_COMPLETE
         int TCPIPVECT
   }
   *errorcode=_AX;
    return _DX;
}
#endif
/*************************************************************************/
//end tcpip.c
/*************************************************************************/
