/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     countcgi.c
* Function:   This program demonstrates the usage of the CGI interface of IPC@CHIP.
*             It has a counter variable that is incremented every second. A CGI
*             function is installed which, when called through a webbrowser, shows
*             the actual value of the counter.
*             After about two minutes the program uninstalls the CGI function and
*             exits. A browser request will now return with a messae saying
*             "Object not found".
* Note:       The URL of the CGI function is http://<ip-address>/countcgi
*             A directory named "web" containing a file named "index.htm" should
*             exist on the chip's flash. These will be set as root directory resp.
*             main page.
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\CountCGI\CountCGI.c, 3, 31.03.2004 11:15:35, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Definition of constants
******************************************************************************/
#define PAGENAME  "countcgi"   // Name of CGI function (for browser request)
#define RUNTIME   120          // Time in seconds for which the program will run
#define MAINPAGE  "index.htm"  // Name of the main page of the webserver
#define ROOTDIR   "web"        // Root directory of the webserver



/******************************************************************************
* Global variables
******************************************************************************/
unsigned long i;              // Counter variable that will be displayed by CGI
                              // function



/******************************************************************************
* cgiFunction()
*
* This function will be executed when a browser requests the page named "countcgi".
* It constructs a small HTML page containing the actual value of the counter
* variable and returns it to the webserver.
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr CgiRequest )
{
   static char buffer[512];  // Buffer to contain web page


   // Construct HTML page
   sprintf( buffer, "<html>\r\n"
                    "<head><title>CountCGI</title></head>\r\n"
                    "<body>\r\n"
                    "<h1>CountCGI - IPC@Chip CGI example</h1>\r\n"
                    "<p>Actual counter value: %lu</p>\r\n"
                    "<hr size=\"0\" />\r\n"
                    "<p><small><em>&copy; 2004 by BECK IPC GmbH</em></small></p>\r\n"
                    "</body>\r\n"
                    "</html>\r\n", i );

   // Give it to the webserver
   CgiRequest->fHttpResponse = CgiHttpOk;
   CgiRequest->fResponseBufferPtr = buffer;
   CgiRequest->fResponseBufferLength = strlen( buffer );
}



/******************************************************************************
* main()
*
* Sets and gets main page and root directory, installs CGI function, increments
* the counter variable for about 2 minutes and deinstalls CGI function again.
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  CGI_Entry cgiEntry;         // Holds information on the CGI function
  char *pageName = PAGENAME;  // Name of CGI function (for browser request)
  char *cptr;                 // Pointer to character string return by several
                              // functions


   // Say hello
   printf( "\r\nCountCGI - IPC@Chip CGI example"
           "\r\n" );


   // Set main page
   if( CGI_SetMainPage( MAINPAGE ) != 0 )
   {
      printf( "\r\nSetting main page %s failed", MAINPAGE );
   }

   // Set root directory
   if( CGI_SetRootDir( ROOTDIR ) != 0 )
   {
     printf( "\r\nSetting root directory %s failed", ROOTDIR );
   }

   // Get main page
   CGI_GetMainPage( &cptr );
   printf( "\r\nMain page: %s", cptr );

   // Get root directory
   CGI_GetRootDir( &cptr );
   printf("\r\nRoot directory: %s",cptr);


   // Install CGI function
   cgiEntry.PathPtr = pageName;        // Name of the page
   cgiEntry.method = CgiHttpGet;       // HTTP method
   cgiEntry.CgiFuncPtr = cgiFunction;  // Function called on browser request

   if( CGI_Install( &cgiEntry ) != 0 )
   {
     printf( "\r\nInstalling CGI function %s failed"
             "\r\n", cgiEntry.PathPtr );
     return 1;
   }


   // Main loop: Increment a counter
   for( i = 0; i < RUNTIME; ++i )
   {
      // Sleep a second
      RTX_Sleep_Time( 1000 );
   }


   // Remove CGI function
   if( CGI_Delete( cgiEntry.PathPtr ) != 0 )
   {

     printf( "\r\nRemoving %s failed"
             "\r\n", cgiEntry.PathPtr );
     return 1;
   }


   // Say bye
   printf( "\r\nGood bye!" );

   return 0;
}



// End of file

