/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     submit.c
* Function:   This program demonstrates the usage of the CGI interface of IPC@CHIP.
*             A CGI function is installed which, when called through a webbrowser
*             shows a small form with two field into which the user can enter some
*             text. When the user then clicks on submit, the same CGI function is
*             called again to process the form. It displays the values entered by
*             the user and the form as well.
*             After about two minutes the program uninstalls the CGI function and
*             exits. A browser request will now return with a messae saying
*             "Object not found".
* Notes:      The URL of the CGI function is http://<ip-address>/submit
*
*******************************************************************************
*
* $Header: c:\proj2000\IPC@CHIP\Examples\cgi\SUBMIT\SUBMIT.C, 4, 31.03.2004 11:16:02, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"


/******************************************************************************
* Constants
******************************************************************************/
#define PAGENAME  "submit"  // Name of CGI function (for browser request)
#define RUNTIME   120       // Time in seconds for which the program will run

//max. length of a nameitem, incl. '\0'
#define MAX_FORMNAME_LENGTH   64
/*
  max. length of a full path (pagename + argumentbuffer) incl '\0'
  e.g. pagename?Name1=Value1&Name2=Value2
*/
#define MAX_FULLPATH_LENGTH      256

//the max. size for our CGI page at this example
#define MAX_SUBMIT_PAGELEN     1024

/******************************************************************************
* cgiFunction()
*
* This function will be executed when a browser requests the page named "submit".
* It constructs a small HTML page showing the parameters given to it and a form
* into which one can enter text that will be passed to the same function with
* another call when the user clicks on submit.
******************************************************************************/
void huge _pascal cgiFunction( rpCgiPtr CgiRequest )
{
  FormItem formItem;

  //user provided buffers for CGI_GetFormItem
  static char namePtr[MAX_FORMNAME_LENGTH];
  /*
     If API function API_GETFORMITEM is used for checking the arguments:
     For a safe protection against an internal  buffer overflow ,the
     storage for a incoming value argument must have the max. size of a
     full path name
  */
  static char valuePtr[MAX_FULLPATH_LENGTH];
  //Buffers for building the page
  static char buffer   [MAX_SUBMIT_PAGELEN];
  static char tmpBuffer[MAX_SUBMIT_PAGELEN];

  // Set pointers
  formItem.NamePtr = namePtr;
  formItem.ValuePtr = valuePtr;

  // Construct HTML page
  sprintf( buffer, "<html>\r\n"
                   "<head><title>Submit</title></head>\r\n"
                   "<body>\r\n"
                   "<h1>Submit - IPC@Chip CGI example</h1>\r\n"
                   "<h2>Arguments</h2>\r\n"
                   "<p>\r\n" );

  if( CgiRequest->fArgumentBufferLength == 0 )
    strcat( buffer, "None<br />\r\n" );

  while( strlen( CgiRequest->fArgumentBufferPtr ) > 0 )
  {
    CGI_GetFormItem( CgiRequest->fArgumentBufferPtr, &formItem );

    if(strlen(formItem.NamePtr)>0)
    {
      sprintf( tmpBuffer, "Item: %s<br />", formItem.NamePtr);
    }
    else
    {
      sprintf( tmpBuffer, "Item: No item name found<br />");
    }
    strcat( buffer, tmpBuffer );
    if(strlen(formItem.ValuePtr)>0)
    {
      sprintf( tmpBuffer, "Value: %s<br />", formItem.ValuePtr);
    }
    else
    {
      sprintf( tmpBuffer, "Value: No item value found<br />");
    }
    strcat( buffer, tmpBuffer );
    //Try to get pointer of next item (if any exist)
    if( CGI_GetNextItem( CgiRequest->fArgumentBufferPtr,
                         &CgiRequest->fArgumentBufferPtr ) != 0 )
    {
      break;
    }
  }

  strcat( buffer, "</p>\r\n"
                  "<h2>Form</h2>\r\n"
                  "<p>\r\n"
                  "Please enter some text here and click submit.<br />"
                  "<form action=\"/submit\">\r\n"
                  "<input type=\"text\" name=\"FormItem1\" size=\"49\" maxsize=\"49\" /><br />\r\n"
                  "<input type=\"text\" name=\"FormItem2\" size=\"49\" maxsize=\"49\" /><br />\r\n"
                  "<input type=\"submit\" value=\"Submit\" /><br />\r\n"
                  "</form>\r\n"
                  "</p>\r\n"
                  "<hr size=\"0\" />\r\n"
                  "<p><small><em>&copy; 2004 by BECK IPC GmbH</em></small></p>\r\n"
                  "</body>\r\n"
                  "</html>\r\n" );

  // Give page to the webserver
  CgiRequest->fHttpResponse = CgiHttpOk;
  CgiRequest->fDataType = CGIDataTypeHtml;
  CgiRequest->fResponseBufferPtr = buffer;
  CgiRequest->fResponseBufferLength = strlen( buffer );
}
/******************************************************************************
* main()
*
* Installs CGI function, runs for about 2 minutes and deinstalls CGI function
* again.
******************************************************************************/
#pragma argsused
int main( int argc, char *argv[] )
{
  CGI_Entry cgiEntry;         // Holds information on the CGI function
  char *pageName = PAGENAME;  // Name of CGI function (for browser request)
  unsigned int i;             // Counter variable

  // Say hello
  printf( "\r\nSubmit - IPC@Chip CGI example"
           "\r\n" );

  // Install CGI function
  cgiEntry.PathPtr = pageName;        // Name of the page
  cgiEntry.method = CgiHttpGet;       // HTTP method
  cgiEntry.CgiFuncPtr = cgiFunction;  // Function called on browser request

  if( CGI_Install( &cgiEntry ) != 0 )
  {
    printf( "\r\nInstalling CGI function %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }

  // Main loop: Increment a counter
  for( i = 0; i < RUNTIME; ++i )
  {
    // Sleep a second
    RTX_Sleep_Time( 1000 );
  }

  // Remove CGI function
  if( CGI_Delete( cgiEntry.PathPtr ) != 0 )
  {

    printf( "\r\nRemoving %s failed"
            "\r\n", cgiEntry.PathPtr );
    return 1;
  }

  printf( "\r\nGood bye!" );

  return 0;
}
// End of file

