/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Module:   smtp.h
* Function: This program is a simple SMTP client that connects to a given SMTP
*           server and sends an E-Mail. The IP address of the SMTP server,
*           the sender address and at least one recipient address are expected
*           as command line parameters. To get a usage message call the program
*           without any parameters.
* Notes:    To get a better grasp of what is done in the function sendMail()
*           you should gallop through RFC 2821 (espaccially chapter 4).
*
*******************************************************************************
*
* $Header: smtp.h, 1, 23.02.2004 13:17:57, Jan Schatz$
*
******************************************************************************/



#ifndef SMTP_H
#define SMTP_H



/******************************************************************************
* Constants
******************************************************************************/
#define SMTP_PORT                 25      // Well known SMTP port

// Maximum sizes as of RFC 2821 4.5.3.1
// Larger sizes that the server might announce to be able with his EHLO reply
// will not be considered
                                          // Maximum (size of) ...
#define MAXSIZE_LOCALPART         64      // ... local parts (e.g. a user name)
#define MAXSIZE_DOMAIN            255     // ... domain name
#define MAXSIZE_PATH              256     // ... reverse- or forward-path
#define MAXSIZE_COMMANDLINE       512     // ... command line incl. <CRLF>
#define MAXSIZE_REPLYLINE         512     // ... reply line incl. <CRLF>
#define MAXSIZE_TEXTLINE          1000    // ... text line incl. <CRLF> + evetually
                                          // duplicated leading period
#define MAXSIZE_MESSAGECONTENT    32768L  // ... entire message content
                                          // This is smaller than defined in
                                          // RFC 2821. It's easier to handle that
                                          // way.
#define MAXSIZE_RECIPIENTSBUFFER  100     // ... number of recipients

// Timeouts as of RFC 2821 4.5.3.2
                                          // Timeout for ...
#define TIMEOUT_CONNECT           300     // ... initial 220 Message
#define TIMEOUT_MAIL              300     // ... MAIL command
#define TIMEOUT_RCPT              300     // ... RCPT command
#define TIMEOUT_DATAINITIATION    120     // ... DATA initiation
#define TIMEOUT_DATABLOCK         180     // ... data block - Assuming that a
                                          // chunk of data is equal to the size
                                          // of the TCP send queue
#define TIMEOUT_DATATERMINATION   600     // ... DATA termination
#define TIMEOUT                   120     // This is not specified in RFC 2821



#endif  // #ifndef SMTP_H

