/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     snmpv1.c
*             Requires SC13 @CHIP-RTOS V0.91 or SC12 @CHIP-RTOS V1.10 or higher
*
* Function:   Modules/functions for IPC@CHIP SNMPV1 Proxy agent
*             Includes SNMP agent task (UDP server , Port 161)
*             SNMP agent start/initialize
*             Send SNMP traps (Port 162)
*             Process/reply incoming GET,GET NEXT and SET requests
*             DK40/DK50 digital out variables support , see define DKXX_SNMP_DEMO
*
*             The SNMP proxy agent example requires SC12 @CHIP-RTOS version V1.10B
*             or SC13 @CHIP-RTOS 0.92 or higher
*             with SNMP MIB support. These versions are part of this ZIP archive.
*
*             The current version of the agent has some important changes.
*             The agent provides now access to the following dynamic mib-2 tables as described at RFC1123:
*             a) ifTable
*             b) atTable
*             c) ipAddrTable
*             d) ipRouteTable
*             e) ipNetToMediaTable
*             f) TCPConnTable
*             g) UDPConntable
*
*             For more information see readme.txt at the Doc directory
*
*******************************************************************************
*
* $Header: snmpv1.c, 2, 31.03.2004 11:16:00, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include <STDIO.H>
#include <DOS.H>
#include <CONIO.H>
#include <STRING.H>
#include <stdlib.h>

#include <DKxx.h>
#include <TCPIPAPI.H>
#include <tcpip.h>
#include <rtos.h>
#include <bios_api.h>
#include <hwapi.h>
#include <helper.h>

#include <err.h>
#include <typedef.h>
#include <snmpv1.h>
#include <snmphlp.h>
#include <mib.h>

/***************************************************************************************/
/*Defines*/
/***************************************************************************************/
//declare static functions as near
#define SNMP_NEAR  near

//    Stacksize of the snmp agent task
//!! If you modify the code, check the snmp agent tasks stack usage by using the tasks command
#define    TASK_STACKSIZE              600   //Words

/***************************************************************************************/
/*Prototypes*/
/***************************************************************************************/
static void huge snmp_agent(void);

static int  SNMP_NEAR send_BER_message(unsigned char *abBerMessage,struct sockaddr_in * to );

static int  SNMP_NEAR analyze_SNMP_message(tSNMPMessage * ptReceivesMessage,
                                           tSNMPMessage *ptSendMessage);

static int  SNMP_NEAR get_request(unsigned char *pbSourceOID, unsigned char *pbDestOID,
                                  unsigned char *pbValue);

static int  SNMP_NEAR get_next_request(unsigned char *pbSourceOID, unsigned char *pbDestOID,
                                       unsigned char *pbValue);

static int  SNMP_NEAR set_request(unsigned char *pbSourceOID, unsigned char *pbSourceValue,
                                  unsigned char *pbDestOID, unsigned char *pbDestValue);

static int  SNMP_NEAR get_SNMP_value(unsigned int wChannel, unsigned char * pbValue,
                                     unsigned char tbl_idx);

static int  SNMP_NEAR set_SNMP_value(unsigned int bChan, unsigned char * pbValue,
                                     unsigned char tbl_idx);

static int SNMP_NEAR send_BER_trap(unsigned char * abTrapMessage, unsigned long IPAddr,
                                   unsigned int wPort);

static int  SNMP_NEAR insert_trap_oid(unsigned char bTrapNumber, unsigned char bSecTrapNumber,
                                      unsigned int * wNumberBindings, tPDUBinding * ptBindings);

static void SNMP_NEAR SNMP_message_input(struct sockaddr_in * tSendr,
                                         unsigned char * rcvbuf,unsigned char * sndbuf);
static void SNMP_NEAR check_SNMP_cfgdata(void);

/***************************************************************************************/
/*External vars
/***************************************************************************************/
extern char * BaseOID[];     //mib.c
extern tMib   tMIB[];        //mib.c
unsigned char target=SC12;
/***************************************************************************************/
/* Variables
/***************************************************************************************/
static unsigned char snmp_rcvbuf[SNMP_MAX_MSG_LENGTH+1];
static unsigned char snmp_sndbuf[SNMP_MAX_MSG_LENGTH+1];
static tSNMPMessage  tReceivesMessage;
static tSNMPMessage  tSendMessage;
static tSNMPTrap     tTrapMessage;

static unsigned char  bTrapSend[SNMP_MAX_MSG_LENGTH+1];


static char sSystemSysDescr   [MAX_LENGTH_SNMP_STRING];
static char sSystemMibContact [MAX_LENGTH_SNMP_STRING];
static char sSystemMibName    [MAX_LENGTH_SNMP_STRING];
static char sSystemMibLocation[MAX_LENGTH_SNMP_STRING];

static volatile boolean ConfigDataChanged = FALSE;
static int iInterface_Status = INTERFACE_DOWN;

//chip.ini item names
static char const * SNMP_Section           = "SNMP";
static char const * SNMP_SysDescr          = "SYSDESCR";
static char const * SNMP_Contact           = "CONTACT";
static char const * SNMP_Name              = "NAME";
static char const * SNMP_Location          = "LOCATION";
static char const * SNMP_Traps             = "TRAPS";
static char const * SNMP_WriteCommunity    = "WRITECOMMUNITY";
static char const * SNMP_ReadCommunity     = "READCOMMUNITY";
static char const * SNMP_TrapCommunity     = "TRAPCOMMUNITY";
static char const * SNMP_TrapReceiver      = "TRAPRECEIVER";
static char const * SNMP_AuthTraps         = "AUTHTRAPS";

unsigned long agent_ip;
unsigned long trap_ip;


static char sReadCommunity[51];
static char sTrapCommunity[51];
static char sWriteCommunity[51];
static char ipAddressTrap[18];          /* Trap receiver ip*/

/*
   Trap ASN1 encoded OIDs
   To do: Improve and simplify sending of traps
*/
static unsigned char OIDSTART[]   =  {0x06,0x06,0x2b,0x06,0x01,0x02,0x01,0x21};
static unsigned char abIPAddr[]   =  {0x40,0x4,0x00,0x00,0x00,0x00};
static unsigned char abOIDNull2[] =  {0x06,0x1,0x00};

#ifdef DKXX_SNMP_DEMO
//DKxx specific Trap OID  asn1 encoded: 1.3.6.1.2.1.80.2.1.0
static unsigned char abDKXX_O_7_high[]   =  {0x06,0x09,0x2b,0x06,0x01,0x02,0x01,0x50,0x02,0x01,0x00};
#endif

static unsigned long dwChangeInterfaceStatusTime;

/****************************************************************************/
//SNMP counters
static struct mib_snmp     mib_snmp;
/****************************************************************************/
//Pointers to @CHIP-RTOS internal network and system counters
/****************************************************************************/
static IpMib         * mib_ip;
static IcmpMib       * mib_icmp;
static TcpMib        * mib_tcp;
static UdpMib        * mib_udp;
static IfMib         * mib_if;
static IfMib         * devIfMib[MAX_DEV];
static unsigned long * sysuptime;

/****************************************************************************/
//TCP/IP system caches
/****************************************************************************/
static ArpRouteCacheEntry arproute_cacheEntry[MAX_ARP];
static Iface_Entry        iface_entry        [MAX_DEV];
static SocketInfo         socks              [MAX_SOCKETS];

static int max_dev = MAX_DEV;

//++++++++++++ sub mib trees for dynmamic mib-2 tables +++++++++++++++++++++++++++
//sub mib tree for address translation table
tMib  at_tble[MAX_ARP*(atNetAddress-atIfIndex)];
//sub mib tree for ipnettomedia table
tMib  ipntm_tble[MAX_ARP*(ipNetToMediaType-ipNetToMediaIfIndex)];
//sub mib tree for ipnettomedia table
tMib  iprt_tble[MAX_ROUTE*(ipRouteInfo-ipRouteDest)];
//sub mib tree for interface table
tMib  iface_tble[MAX_DEV*(ifSpecific-ifIndex)];
//sub mib tree for udp listener table
tMib  udp_tble[MAX_SOCKETS*(udpLocalPort-udpLocalAddress)];
//sub mib tree for tcp connection table
tMib  tcp_tble[MAX_SOCKETS*(tcpConnRemPort-tcpConnState)];
//sub mib tree ip address table
tMib  ipaddr_tble[MAX_IP_ADDR*(ipAdEntReasmMaxSize - ipAdEntAddr)];


//++++++++++++ OID strings for dynmamic mib-2 tables +++++++++++++++++++++++++++
//Memory is allocated at runtime
//address translation table
static char * at_oid_str    [MAX_ARP*(atNetAddress-atIfIndex)];
//ipnettomedia  table
static char * ipntm_oid_str [MAX_ARP*(ipNetToMediaType-ipNetToMediaIfIndex)];
//iproute table
static char * iprt_oid_str  [MAX_ROUTE*(ipRouteInfo-ipRouteDest)];
//interface entry table
static char * iface_oid_str [MAX_DEV*(ifSpecific-ifIndex)];
//udp socket table
static char * udp_oid_str   [MAX_SOCKETS*(udpLocalPort-udpLocalAddress)];
//tcp socket table
static char * tcp_oid_str   [MAX_SOCKETS*(tcpConnRemPort-tcpConnState)];
//ip address table
static char * ipaddr_oid_str[MAX_IP_ADDR*(ipAdEntReasmMaxSize - ipAdEntAddr)];

/****************************************************************************/
//RTOS specific variables
/****************************************************************************/
static int   trap_semaphoreID; /*semaphoreID*/
static char  Trap_Sem[5]="TRAP";

int   mib_semaphoreID; /*semaphoreID*/
static char  Mib_Sem[5]="MIB ";

static int iSNMPTaskHandle;
static unsigned int SNMP_stack[TASK_STACKSIZE];

static TaskDefBlock  tSNMPTaskDefinition =
{
  snmp_agent,
  "SNMP",                         // a name: 4 chars
  &SNMP_stack[TASK_STACKSIZE],    // top of stack
  (TASK_STACKSIZE*sizeof(int)),   // size of stack
  0,                              // attributes, not supported now
  20,                             // priority 20(high) ... 127(low)
  0,                              // time slice (if any), not supported now
  0,0,0,0                         // mailbox depth, not supported now
};


//socket descriptor for SNMP UDP server, Port 161
static unsigned int  snmp_socket;

/***************************************************************************************/
//initialize dynamic  table member oid strings as null pointers, called at init_snmp
/***************************************************************************************/
static void SNMP_NEAR init_dynamic_table_oid(void)
{
   int i;
   for(i=0;i<MAX_ARP*(atNetAddress-atIfIndex);i++)
   {
      at_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_ARP*(ipNetToMediaType-ipNetToMediaIfIndex);i++)
   {
      ipntm_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_ROUTE*(ipRouteInfo-ipRouteDest);i++)
   {
      iprt_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_DEV*(ifSpecific-ifIndex);i++)
   {
      iface_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_SOCKETS*(udpLocalPort-udpLocalAddress);i++)
   {
      udp_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_SOCKETS*(tcpConnRemPort-tcpConnState);i++)
   {
      tcp_oid_str[i]=NULL;
   }
   for(i=0;i<MAX_IP_ADDR*(ipAdEntReasmMaxSize - ipAdEntAddr);i++)
   {
      ipaddr_oid_str[i]=NULL;
   }
}
/***************************************************************************************/
//Initialize and start SNMP agent task, called at main.c
/***************************************************************************************/
int init_snmp(void)
{
   int iReturn;
   int  err,i;
   char sTemp[40];
   struct sockaddr_in isa;
   int iTemp;



   #ifdef DKXX_SNMP_DEMO
   /****** init DKXX IOs ***********/
   //init chip select 6
   pfe_enable_pcs(6);
   #endif

   // target detection
   BIOS_Version_String (sTemp,39);
   if(0==strncmp(sTemp,"SC12",4))
   {
       target=SC12;
       if(BIOS_Get_RTOS_Version((unsigned int *)&iTemp)<0x110)
       {
          helper_printf("\r\nSC12 @CHIP-RTOS V1.10 or higher required");
          return -1;
       }

       #ifdef DKXX_SNMP_DEMO
       //Target == SC12, we assume a DK40 evaluation board = > enable PIO13 (Bit 7 at DK40 IO port)
       pfe_enable_pio ( 13, 5 );
       #endif

   }
   else
   if(0==strncmp(sTemp,"SC13",4))
   {
       target=SC13;
       if(BIOS_Get_RTOS_Version((unsigned int *)&iTemp)<0x091)
       {
          helper_printf("\r\nSC13 @CHIP-RTOS V0.91 or higher required");
          return -1;
       }
   }
   else
   {
       //unknown target, return
       helper_printf("\r\nUable to detect target IPC@CHIP");
       return -1;
   }

   if(0 == (BIOS_Get_Features() & BGF_SNMP_MIB))
   {
      helper_printf("\r\n@CHIP-RTOS doesn't support SNMP MIB");
      return -1;
   }

   //Create the semaphore protecting sending of traps
   if(0!=RTX_Create_Sem(&trap_semaphoreID,Trap_Sem,-1))
   {
     return -1;
   }
   //Create the semaphore ,protecting access/building of sub mib tables
   if(0!=RTX_Create_Sem(&mib_semaphoreID,Mib_Sem,1))
   {
     return -1;
   }

   //Allocate and init/build dynamic tables
   init_dynamic_table_oid();

   // Get MIB structure pointers from the @Chip-RTOS TCPIP stack
   Get_SNMP_Data(1,(void *)&mib_if);
   Get_SNMP_Data(2,(void *)&mib_ip);
   Get_SNMP_Data(3,(void *)&mib_icmp);
   Get_SNMP_Data(4,(void *)&mib_tcp);
   Get_SNMP_Data(5,(void *)&mib_udp);
   Get_SNMP_Data(8,(void *)&sysuptime);

   //reset sysuptime
   *sysuptime=0L;

   //clear snmp packet counter
   memset(&mib_snmp,0,sizeof(struct mib_snmp));

   //enable auth traps
   mib_snmp.snmpEnableAuthTraps = TRAP_ENABLED;

   //Read agents system data from chip.ini or set default system data
   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_SysDescr,  sSystemSysDescr, MAX_LENGTH_SNMP_STRING))
   {
        strcpy(sSystemSysDescr,"IPC@Chip SNMP Proxy Agent");
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_Contact,  sSystemMibContact, MAX_LENGTH_SNMP_STRING))
   {
        strcpy(sSystemMibContact,"First Contact");
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_Name,  sSystemMibName, MAX_LENGTH_SNMP_STRING))
   {
        strcpy(sSystemMibName,"Mibname");
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_Location,  sSystemMibLocation, MAX_LENGTH_SNMP_STRING))
   {
        strcpy(sSystemMibLocation,"Home");
   }

   if(BIOS_Get_Ini_String(SNMP_Section, SNMP_Traps,  sTemp, 15))
   {
       if(0==strcmp("Enabled",sTemp))
       {
         mib_snmp.snmpEnableAuthTraps=TRAP_ENABLED;
       }
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_ReadCommunity,  sReadCommunity, 50))
   {
       strcpy(sReadCommunity,"public");
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_WriteCommunity,  sWriteCommunity, 50))
   {
       strcpy(sWriteCommunity,"private");
   }

   if(0==BIOS_Get_Ini_String(SNMP_Section, SNMP_TrapCommunity,  sTrapCommunity, 50))
   {
       strcpy(sTrapCommunity,"trap");
   }

   //Checking the trap receiver
   strcpy(ipAddressTrap,"0.0.0.0");
   if(BIOS_Get_Ini_String(SNMP_Section, SNMP_TrapReceiver,  ipAddressTrap, 17))
   {
     if(-1==inet_addr(ipAddressTrap,&trap_ip))
     {
       strcpy(ipAddressTrap,"0.0.0.0");
       trap_ip=0L;
     }
     else
     {
      iTemp = Get_Iface_Entries(&iface_entry[0],MAX_DEV);
      for(i=0;i<iTemp;i++)
      {
         if( (iface_entry[i].devIPAddr & iface_entry[i].devNetmask)
              ==
             (trap_ip & iface_entry[i].devNetmask)
           )
         {
            agent_ip=iface_entry[i].devIPAddr;
            break;
         }
      }
      //no match found, must select one ip
      if(i==iTemp)
      {
         for(i=0;i<iTemp;i++)
         {
           if(iface_entry[i].devType!=5)
           {
             agent_ip=iface_entry[i].devIPAddr;
             break;
           }
         }
      }
     }
   }

   //create/init snmp socket at SNMP PORT
   snmp_socket = opensocket(SOCK_DGRAM,&err);
   if ((int)snmp_socket != -1)
   {
        isa.sin_family      = AF_INET;
        isa.sin_port        = htons(SNMP_PORT);
        isa.sin_addr.s_addr =  0L;

        iReturn = bind(snmp_socket,
                      (const struct sockaddr_in far*)&isa
                      ,&err);

         if (iReturn == NO_ERR)
         {
           if(RTX_Create_Task(&iSNMPTaskHandle , &tSNMPTaskDefinition)!=0)
           {
               iReturn = -8;
           }
           else
           {
               iInterface_Status = INTERFACE_UP;
               send_SNMP_trap(SNMP_TRAP_COLDSTART,0 );
           }
        }
   }
   else
   {
      iReturn = -7;
   }
   return iReturn;
}

/***************************************************************************************/
// SNMP agent task
//   Receive/process and answer incoming SNMP requests
/***************************************************************************************/
void huge snmp_agent(void)
{
   struct sockaddr_in from;
   int result,err;
   long sema_timeout=0L;

   while (1)
   {
      result = recvfrom(snmp_socket,
                       (char far *)snmp_rcvbuf,
                        sizeof(snmp_rcvbuf),
                        MSG_BLOCKING,
                        0L,
                        (struct sockaddr_in far *)&from,
                        &err);

      if(result>0)
      {
         mib_snmp.snmpInPkts++;
         RTX_Wait_Sem(mib_semaphoreID, &sema_timeout);
         SNMP_message_input(&from,snmp_rcvbuf,snmp_sndbuf);
         RTX_Signal_Sem(mib_semaphoreID);
         check_SNMP_cfgdata();
      }

      RTX_Sleep_Time(3);
   }//while(1)
}

/***************************************************************************************/
//Send BER encoded message
/***************************************************************************************/
static int  SNMP_NEAR send_BER_message(unsigned char *abBerMessage,struct sockaddr_in * to )
{
   int result = NO_ERR;
   int err;
   unsigned int wNumber;
   unsigned int wDummy = 0;

   mib_snmp.snmpOutPkts++;
   wNumber = get_BER_send_length(&abBerMessage[1],&wDummy); // Length of message starts at Byte 1
   result = sendto(snmp_socket,
                  (const char far *)abBerMessage,
                  wNumber,MSG_BLOCKING,
                  (const struct sockaddr_in far * )to
                  ,&err);
   return result;
}
/***************************************************************************************/
//Parse/Process  incomming SNMP request
/***************************************************************************************/
static int  SNMP_NEAR analyze_SNMP_message(tSNMPMessage * ptReceivesMessage,tSNMPMessage *ptSendMessage)
{
   int iReturn = NO_ERR;
   unsigned int wCounter;

   if ((ptReceivesMessage -> tPUDDataHead.bType == SNMP_GET_REQ_PDU)||
       (ptReceivesMessage -> tPUDDataHead.bType == SNMP_GETNEXT_REQ_PDU)||
       (ptReceivesMessage -> tPUDDataHead.bType == SNMP_SET_REQ_PDU))
   {
       if (strlen((char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString)<  COMMUNITY_STRING_LENGTH)
       {
           strcpy((char *) ptSendMessage -> tSNMPHeadData.bCommunityString ,(char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString );
           ptSendMessage -> tPUDDataHead.bType = SNMP_GET_RSP_PDU;
           memcpy(ptSendMessage -> tPUDDataHead.bRequestNumber,ptReceivesMessage -> tPUDDataHead.bRequestNumber,MAX_LEN_REQUEST);
           ptSendMessage -> tPUDDataHead.wErrorStatus        = ptReceivesMessage -> tPUDDataHead.wErrorStatus ;
           ptSendMessage -> tPUDDataHead.wErrorNumber        = ptReceivesMessage -> tPUDDataHead.wErrorNumber ;
           ptSendMessage -> tSNMPHeadData.bVersion            = ptReceivesMessage -> tSNMPHeadData.bVersion ;
           ptSendMessage -> tPUDDataHead.wNumberOfBindings = ptReceivesMessage -> tPUDDataHead.wNumberOfBindings  ;
           if ((strcmp( (char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString,sReadCommunity ) == 0 ) ||
               (strcmp( (char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString,sWriteCommunity ) == 0 )  )
               {
               if (ptSendMessage -> tPUDDataHead.wNumberOfBindings <= MAX_NUMBER_BINDINGS)
               {
                   for (wCounter = 0; wCounter <  ptSendMessage -> tPUDDataHead.wNumberOfBindings; wCounter++)
                   {
                       //Get request
                       if (ptReceivesMessage -> tPUDDataHead.bType == SNMP_GET_REQ_PDU)
                       {
                           if (strcmp( (char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString,sReadCommunity ) == 0 )
                           {
                               if ((iReturn = get_request(ptReceivesMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                          ptSendMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                          ptSendMessage -> tPUDBindings[wCounter].abValueBinding)) !=NO_ERR)
                               {
                                   break;
                               }
                               else
                               {
                                   mib_snmp.snmpInGetRequests++;
                                   mib_snmp.snmpInTotalReqVars++;
                                   mib_snmp.snmpOutGetResponses++;
                               }
                           }
                           else
                           {
                               iReturn = BAD_COMMUNITY_USES_ERR;
                               mib_snmp.snmpInBadCommunityUses++;
                               send_SNMP_trap(SNMP_TRAP_AUTHFAIL,0);
                               break;

                           }
                       }
                       //Get next request
                       if (ptReceivesMessage -> tPUDDataHead.bType == SNMP_GETNEXT_REQ_PDU)
                       {
                           if (strcmp( (char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString, sReadCommunity ) == 0 )
                           {
                               if ((iReturn = get_next_request(ptReceivesMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                               ptSendMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                               ptSendMessage -> tPUDBindings[wCounter].abValueBinding))!=NO_ERR)
                               {
                                   if (iReturn == END_OF_MIB_ERROR)
                                   {
                                       iReturn = NO_ERR;
                                       ptSendMessage -> tPUDDataHead.wErrorStatus = SNMP_ERROR_NOSUCHNAME;
                                       ptSendMessage -> tPUDDataHead.wErrorNumber = wCounter+1;
                                       mib_snmp.snmpInTotalReqVars++;
                                       mib_snmp.snmpInGetNexts++;
                                       mib_snmp.snmpOutGetResponses++;
                                   }
                                   break;
                               }
                               else
                               {
                                   mib_snmp.snmpInTotalReqVars++;
                                   mib_snmp.snmpInGetNexts++;
                                   mib_snmp.snmpOutGetResponses++;
                               }

                           }
                           else
                           {
                               iReturn = BAD_COMMUNITY_USES_ERR;
                               mib_snmp.snmpInBadCommunityUses++;
                               send_SNMP_trap(SNMP_TRAP_AUTHFAIL,0);
                               break;
                           }
                       }

                       if (ptReceivesMessage -> tPUDDataHead.bType == SNMP_SET_REQ_PDU)
                       {
                           if (strcmp( (char *) ptReceivesMessage -> tSNMPHeadData.bCommunityString, sWriteCommunity ) == 0 )
                           {
                               if ((iReturn = set_request(ptReceivesMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                          ptReceivesMessage -> tPUDBindings[wCounter].abValueBinding,
                                                          ptSendMessage -> tPUDBindings[wCounter].abOIDBinding,
                                                          ptSendMessage -> tPUDBindings[wCounter].abValueBinding)) !=NO_ERR)
                               {
                                   break;
                               }
                               else
                               {
                                   mib_snmp.snmpInTotalSetVars++;
                               }
                           }
                           else
                           {
                               iReturn = BAD_COMMUNITY_USES_ERR;
                               mib_snmp.snmpInBadCommunityUses++;
                               break;

                           }
                       }
                   }
               }
               else
               {
                   iReturn = RANGE_ERR;
               }
           }
           else
           { // invalid community string
               send_SNMP_trap(SNMP_TRAP_AUTHFAIL,0);
               iReturn = BAD_COMMUNITY_NAME_ERR;
               mib_snmp.snmpInBadCommunityNames++;
           }
       }
       else
       {   //invalid length of  community string
           send_SNMP_trap(SNMP_TRAP_AUTHFAIL,0);
           iReturn = BAD_COMMUNITY_NAME_ERR;
           mib_snmp.snmpInBadCommunityNames++;
       }
   }
   else
   {
       //unknown request type
       memcpy(ptSendMessage,ptReceivesMessage,sizeof(tSNMPMessage));
   }
   if (iReturn == SNMP_NO_SUCH_NAME_ERR)
   {
       ptSendMessage -> tPUDDataHead.wErrorStatus = SNMP_ERROR_NOSUCHNAME;
       ptSendMessage -> tPUDDataHead.wErrorNumber = wCounter+1;
       mib_snmp.snmpInNoSuchNames++;
       iReturn = NO_ERR;
   }

   if (iReturn == SNMP_ERROR_BADVALUE_ERR)
   {
       ptSendMessage -> tPUDDataHead.wErrorStatus = SNMP_ERROR_BADVALUE;
       ptSendMessage -> tPUDDataHead.wErrorNumber = wCounter+1;
       mib_snmp.snmpInBadValues++;
       iReturn = NO_ERR;
   }
   if (iReturn == SNMP_READ_ONLY_ERR)
   {
       ptSendMessage -> tPUDDataHead.wErrorStatus = SNMP_ERROR_READONLY;
       ptSendMessage -> tPUDDataHead.wErrorNumber = wCounter+1;
       mib_snmp.snmpInReadOnlys++;
       iReturn = NO_ERR;
   }
   return iReturn;
}
/***************************************************************************************/
//Handle a GET request
/***************************************************************************************/
static int  SNMP_NEAR get_request(unsigned char *pbSourceOID, unsigned char *pbDestOID, unsigned char *pbValue)
{
   int   iReturn = SNMP_NO_SUCH_NAME_ERR;   //assume error
   char  abOIDString[OID_ADRESS_STRING_LENGTH];
   char  OIDTemp[OID_ADRESS_STRING_LENGTH];
   tMib * ptMibPointer=&tMIB[0];

   convert_OID_to_string(abOIDString, pbSourceOID);

   get_SNMP_value(0,pbValue,0);
   while (ptMibPointer->abAddress!= (char *)0L)
   {
         //Build full mib OID string by adding baseOID and abAddress
         strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
         strcat(OIDTemp,(char *)ptMibPointer->abAddress);

         if (strcmp(OIDTemp, abOIDString)==0)
         {
           iReturn = NO_ERR;
           get_SNMP_value(ptMibPointer->wChannel,pbValue,ptMibPointer->tbl_idx);
           break;
         }
         if(ptMibPointer->mibPointer==(tMib *)0L)
         {
             //no link to other mib, select next entry of current mib tree
             ptMibPointer++;
         }
         else
         {
             //select existing link to other mib table
             ptMibPointer=ptMibPointer->mibPointer;
         }
   }
   memcpy(pbDestOID,pbSourceOID,MAX_OID_LENGTH);
   return iReturn;
}
/***************************************************************************************/
//Handle a GETNEXT request
/***************************************************************************************/
static int  SNMP_NEAR get_next_request(unsigned char *pbSourceOID, unsigned char *pbDestOID, unsigned char *pbValue)
{
   int iReturn = NO_ERR;
   unsigned int wLength;
   boolean bitFound = FALSE;
   char  abOIDString[OID_ADRESS_STRING_LENGTH];
   char  OIDTemp[OID_ADRESS_STRING_LENGTH];
   tMib * ptMibPointerNext;
   tMib * ptMibPointer=&tMIB[0];

   convert_OID_to_string(abOIDString, pbSourceOID);
   wLength = strlen(abOIDString);
   get_SNMP_value(0,pbValue,0);
   //First walk through the tree and try to find exact match and get next
   while (ptMibPointer->abAddress!= (char *)0L)
   {
       //Build full mib OID string by adding baseOID and abAddress
       strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
       strcat(OIDTemp,(char *)ptMibPointer->abAddress);
       if ((strcmp(OIDTemp,abOIDString)==0) )
       {
          if(ptMibPointer->mibPointer==(tMib *)0L)
          {
             //no link to other mib, select next entry of current mib tree
             ptMibPointerNext = ptMibPointer+1;
          }
          else
          {
             //select existing link to other mib table
             ptMibPointerNext=ptMibPointer->mibPointer;
          }
          ptMibPointer=ptMibPointerNext;
          if (ptMibPointer->abAddress==(char *)0L)
          {
               iReturn = END_OF_MIB_ERROR;
          }
          else
          {
             get_SNMP_value(ptMibPointer->wChannel,pbValue,ptMibPointer->tbl_idx);
             strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
             strcat(OIDTemp,(char *)ptMibPointer->abAddress);
             string_to_oid(pbDestOID,&OIDTemp[4]);
          }
          bitFound = TRUE;
          break;
       }
       if(ptMibPointer->mibPointer==(tMib *)0L)
       {
         //no link to other mib, select next entry of current mib tree
         ptMibPointer++;
       }
       else
       {
         //select existing link to other mib table
         ptMibPointer=ptMibPointer->mibPointer;
       }
   }

   // Exact match not found, try to find non_exact match and get next
   if (bitFound == FALSE)
   {
       ptMibPointer=&tMIB[0];
       while (ptMibPointer->abAddress!= (char *)0L)
       {
           //Build full mib OID string by adding baseOID and abAddress
           strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
           strcat(OIDTemp,(char *)ptMibPointer->abAddress);

           if ((strncmp(OIDTemp, abOIDString,wLength)==0) || (abOIDString[0]=='0'))
           {
               get_SNMP_value(ptMibPointer->wChannel,pbValue,ptMibPointer->tbl_idx);
               strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
               strcat(OIDTemp,(char *)ptMibPointer->abAddress);
               string_to_oid(pbDestOID,&OIDTemp[4]);
               bitFound = TRUE;
               break;
          }
          if(ptMibPointer->mibPointer==(tMib *)0L)
          {
             //no link to other mib, select next entry of current mib tree
             ptMibPointer++;
          }
          else
          {
             //select existing link to other mib table
             ptMibPointer=ptMibPointer->mibPointer;
          }
       }
   }
   if (bitFound == FALSE)
   {
      memcpy(pbDestOID,pbSourceOID,MAX_OID_LENGTH); // copy address
      iReturn = END_OF_MIB_ERROR;
   }
   return iReturn;
}
/***************************************************************************************/
//Handle a SET  request
/***************************************************************************************/
static int  SNMP_NEAR set_request(unsigned char *pbSourceOID, unsigned char *pbSourceValue, unsigned char *pbDestOID, unsigned char *pbDestValue)
{
   int iReturn = SNMP_NO_SUCH_NAME_ERR;  //assume error
   char  abOIDString[OID_ADRESS_STRING_LENGTH];
   char  OIDTemp[MAX_OID_LENGTH];
   tMib * ptMibPointer = &tMIB[0];

   convert_OID_to_string(abOIDString, pbSourceOID);
   while (ptMibPointer->abAddress!= (char *)0L)
   {
       //Build full mib OID string by adding baseOID and abAddress
       strcpy(OIDTemp,(char *)BaseOID[ptMibPointer->baseOIDindex]);
       strcat(OIDTemp,(char *)ptMibPointer->abAddress);

       if (strcmp(OIDTemp , abOIDString)==0)
       {
           if (ptMibPointer->bAccess == READ_WRITE)
           {
               iReturn = set_SNMP_value(ptMibPointer->wChannel,pbSourceValue,ptMibPointer->tbl_idx);
           }
           else
           {
               iReturn = SNMP_READ_ONLY_ERR;
               get_SNMP_value(ptMibPointer-> wChannel,pbSourceValue,ptMibPointer->tbl_idx);
           }
           break;
       }
       if(ptMibPointer->mibPointer==(tMib *)0L)
       {
             //no link to other mib, select next entry of current mib tree
             ptMibPointer++;
       }
       else
       {
             //select existing link to other mib table
             ptMibPointer=ptMibPointer->mibPointer;
       }
   }
   memcpy(pbDestOID,pbSourceOID,MAX_OID_LENGTH);
   memcpy(pbDestValue,pbSourceValue,MAX_OID_VALUE_LENGTH);
   return iReturn;
}
/**********************************************************************************************/
//Get snmp values
/**********************************************************************************************/
static int  SNMP_NEAR get_SNMP_value(unsigned int wChannel, unsigned char * pbValue,unsigned char tbl_idx)
{
   int iReturn = NO_ERR;
   unsigned int wIndex = 0;
   unsigned char abTemp[80];
   unsigned char abIPAddress[4];

   abTemp[80-1] ='J';
   abTemp[80-2] ='R';

   pbValue[0]= SNMP_NULLOBJ;
   pbValue[1]= 0;
   switch (wChannel)
   {
       /**************************************************************************************/
       /*Mib II*/
       /**************************************************************************************/
       case  sysDescr:
                          insert_BER_string(pbValue,sSystemSysDescr,&wIndex);
                          break;
       case  sysObjectID:
                          memcpy(pbValue,OIDSTART,8);
                          break;
       case  sysUpTime:
                          insert_BER_timeticks(pbValue,*sysuptime,&wIndex);
                          break;
       case  sysContact:
                          insert_BER_string(pbValue,sSystemMibContact,&wIndex);
                          break;
       case  sysName:
                          insert_BER_string(pbValue,sSystemMibName,&wIndex);
                          break;
       case  sysLocation:
                          insert_BER_string(pbValue,sSystemMibLocation,&wIndex);
                          break;
       case  sysService:
                          insert_BER_word(pbValue,70,&wIndex);
                          break;
       /**************************************************************************************/
       //Interface
       /**************************************************************************************/
       case ifNumber:
                            insert_BER_dword(pbValue,max_dev,&wIndex);
                            break;

       case ifIndex:
                            insert_BER_dword(pbValue,devIfMib[tbl_idx]->ifIndex,&wIndex);
                            break;

       case ifDescr:
                            insert_BER_string(pbValue,devIfMib[tbl_idx]->ifDescr,&wIndex);
                            break;

       case ifType:         insert_BER_dword(pbValue,devIfMib[tbl_idx]->ifType,&wIndex);
                            break;

       case ifMtu:
                            insert_BER_dword(pbValue,devIfMib[tbl_idx]->ifMtu,&wIndex);
                            break;

       case ifSpeed:        insert_BER_gauge(pbValue,devIfMib[tbl_idx]->ifSpeed,&wIndex);
                            break;

       case ifPhysAddress:  insert_BER_macaddress(pbValue,devIfMib[tbl_idx]->ifPhysAddress,&wIndex);
                            break;

       case ifAdminStatus:  insert_BER_dword(pbValue,devIfMib[tbl_idx]->ifAdminStatus,&wIndex);
                            break;

       case ifOperStatus:   insert_BER_dword(pbValue,devIfMib[tbl_idx]->ifOperStatus,&wIndex);
                            break;

       case ifLastChange:   insert_BER_timeticks(pbValue,devIfMib[tbl_idx]->devLastChange,&wIndex);
                            break;


       case ifInOctets:     insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInOctets,&wIndex);
                            break;

       case ifInUcastPkts:  insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInUcastPkts,&wIndex);
                            break;

       case ifInNucastPkts: insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInBroadcastPkts,&wIndex);
                            break;

       case ifInDiscards:   insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInDiscards,&wIndex);
                            break;

       case ifInErrors:     insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInErrors,&wIndex);
                            break;

       case ifInUnknownProtos:
                            insert_BER_counter(pbValue,devIfMib[tbl_idx]->devInUnknownProtos,&wIndex);
                            break;

       case ifOutOctets:    insert_BER_counter(pbValue,devIfMib[tbl_idx]->devOutOctets,&wIndex);
                            break;

       case ifOutUcastPkts: insert_BER_counter(pbValue,devIfMib[tbl_idx]->devOutUcastPkts,&wIndex);
                            break;

       case ifOutNUcastPkts: insert_BER_counter(pbValue,devIfMib[tbl_idx]->devOutBroadcastPkts,&wIndex);
                             break;

       case ifOutDicards:   insert_BER_counter(pbValue,devIfMib[tbl_idx]->devOutDiscards,&wIndex);
                            break;

       case ifOutErrors:    insert_BER_counter(pbValue,devIfMib[tbl_idx]->devOutErrors,&wIndex);
                            break;

       case ifOutLen:       insert_BER_gauge(pbValue,devIfMib[tbl_idx]->devOutQLen,&wIndex);
                            break;

       case ifSpecific:     memcpy(pbValue,abOIDNull2,3);
                            break;
                            //A reference to MIB definitions specific to the
                           //particular media being used to realize the
                           //interface.  For example, if the interface is
                           //realized by an ethernet, then the value of this
                           //object refers to a document defining objects
                           //specific to ethernet.  If this information is not
                           //present, its value should be set to the OBJECT
                           //IDENTIFIER  0 0 , which is a syntatically valid
                           //object identifier, and any conformant
                           //implementation of ASN.1 and BER must be able to
                           //generate and recognize this value.
       /**************************************************************************************/
       // At netToMedia Table
       /**************************************************************************************/
       case atIfIndex:     insert_BER_dword(pbValue,arproute_cacheEntry[tbl_idx].ifIndex,&wIndex);
                           break;
                           // The interface on which this entry's equivalence
                           // is effective.  The interface identified by a
                           // particular value of this index is the same
                           // interface as identified by the same value of
                           // ifIndex.

       case atPhysAddress: insert_BER_macaddress(pbValue
                                                 ,arproute_cacheEntry[tbl_idx].ArpRouteGwayUnion.ArpEntry.MacAddress
                                                 ,&wIndex);
                           break;
                           //The media-dependent `physical' address.
                           // Setting this object to a null string (one of zero
                           // length) has the effect of invaliding the
                           // corresponding entry in the atTable object.  That
                           // is, it effectively dissasociates the interface
                           // identified with said entry from the mapping
                           // identified with said entry.  It is an
                           // implementation-specific matter as to whether the
                           // agent removes an invalidated entry from the table.
                           // Accordingly, management stations must be prepared
                           // to receive tabular information from agents that
                           // corresponds to entries not currently in use.
                           // Proper interpretation of such entries requires
                           // examination of the relevant atPhysAddress object."
       case atNetAddress:  ip_to_array(arproute_cacheEntry[tbl_idx].arpIpAddress,abIPAddress);
                           insert_BER_ipaddress(pbValue
                                               ,abIPAddress
                                               ,&wIndex);
                           break;
                           // The NetworkAddress (e.g., the IP address)
                           // corresponding to the media-dependent `physical'
                           // address."

       /**************************************************************************************/
       //IP specific data
       /**************************************************************************************/
       case ipForwarding:  insert_BER_dword(pbValue,mib_ip->ipForwarding,&wIndex);
                           break;
                           // The indication of whether this entity is acting
                           // as an IP gateway in respect to the forwarding of
                           // datagrams received by, but not addressed to, this
                           // entity.  IP gateways forward datagrams.  IP hosts
                           // do not (except those source-routed via the host).
                           //
                           // Note that for some managed nodes, this object may
                           // take on only a subset of the values possible.
                           // Accordingly, it is appropriate for an agent to
                           // return a `badValue' response if a management
                           // station attempts to change this object to an
                           // inappropriate value.
       case ipDefaultTTL: insert_BER_dword(pbValue,mib_ip->ipDefaultTTL,&wIndex);
                          break;
                           // The default value inserted into the Time-To-Live
                           // field of the IP header of datagrams originated at
                           // this entity, whenever a TTL value is not supplied
                           // by the transport layer protocol
       case ipInReceives: insert_BER_counter(pbValue,mib_ip->ipInReceives,&wIndex);
                          break;
                          //  The total number of input datagrams received from
                          //  interfaces, including those received in error.


       case ipInHdrErrors: insert_BER_counter(pbValue,mib_ip->ipInHdrErrors,&wIndex);
                           break;
                           // The number of input datagrams discarded due to
                           // errors in their IP headers, including bad
                           // checksums, version number mismatch, other format
                           // errors, time-to-live exceeded, errors discovered
                           // in processing their IP options, etc.
       case ipInAddrErrors: insert_BER_counter(pbValue,mib_ip->ipInAddrErrors,&wIndex);
                           break;
                           //The number of input datagrams discarded because
                           //the IP address in their IP header's destination
                           //field was not a valid address to be received at
                           //this entity.  This count includes invalid
                           //addresses (e.g., 0.0.0.0) and addresses of
                           //unsupported Classes (e.g., Class E).  For entities
                           //which are not IP Gateways and therefore do not
                           //forward datagrams, this counter includes datagrams
                           //discarded because the destination address was not
                           //a local address.

       case ipForwDatagrams: insert_BER_counter(pbValue,mib_ip->ipForwDatagrams,&wIndex);
                             break;
                           //The number of input datagrams for which this
                           //entity was not their final IP destination, as a
                           //result of which an attempt was made to find a
                           //route to forward them to that final destination.
                           //In entities which do not act as IP Gateways, this
                           //counter will include only those packets which were
                           //Source-Routed via this entity, and the Source-
                           //Route option processing was successful.
       case ipInUnknownProtos: insert_BER_counter(pbValue,mib_ip->ipInUnknownProtos,&wIndex);
                            break;
                           //The number of locally-addressed datagrams
                           //received successfully but discarded because of an
                           //unknown or unsupported protocol.


       case ipInDiscards:  insert_BER_counter(pbValue,mib_ip->ipInDiscards,&wIndex);
                           break;
                           //The number of input IP datagrams for which no
                           //problems were encountered to prevent their
                           //continued processing, but which were discarded
                           //(e.g., for lack of buffer space).  Note that this
                           //counter does not include any datagrams discarded
                           //while awaiting re-assembly.
       case ipInDelivers:  insert_BER_counter(pbValue,mib_ip->ipInDelivers,&wIndex);
                           break;
                           // The total number of input datagrams successfully
                           // delivered to IP user-protocols (including ICMP)."The total number of input datagrams successfully
                           // delivered to IP user-protocols (including ICMP)."

       case ipOutrequests: insert_BER_counter(pbValue,mib_ip->ipOutRequests,&wIndex);
                           break;
                           // The total number of IP datagrams which local IP
                           // user-protocols (including ICMP) supplied to IP in
                           // requests for transmission.  Note that this counter
                           // does not include any datagrams counted in
                           // ipForwDatagrams
       case ipOutDiscards: insert_BER_counter(pbValue,mib_ip->ipOutDiscards,&wIndex);
                           break;
                           // The number of output IP datagrams for which no
                           // problem was encountered to prevent their
                           // transmission to their destination, but which were
                           // discarded (e.g., for lack of buffer space).  Note
                           // that this counter would include datagrams counted
                           // in ipForwDatagrams if any such packets met this
                           // (discretionary) discard criterion.
       case ipOutNoRoutes: insert_BER_counter(pbValue,mib_ip->ipOutNoRoutes,&wIndex);
                           break;
                           // The number of IP datagrams discarded because no
                           // route could be found to transmit them to their
                           // destination.  Note that this counter includes any
                           // packets counted in ipForwDatagrams which meet this
                           // `no-route' criterion.  Note that this includes any
                           // datagarms which a host cannot route because all of
                           // its default gateways are down.
       case ipreasmTimeout: insert_BER_dword(pbValue,mib_ip->ipReasmTimeout,&wIndex);
                            break;
                           // The maximum number of seconds which received
                           // fragments are held while they are awaiting
                           // reassembly at this entity."
       case ipReasmRedqs:  insert_BER_counter(pbValue,mib_ip->ipReasmReqds,&wIndex);
                           break;
                           // The number of IP fragments received which needed
                           // to be reassembled at this entity.
       case ipReasmOKs:   insert_BER_counter(pbValue,mib_ip->ipReasmOKs,&wIndex);
                           break;
                           // The number of IP datagrams successfully re-
                           // assembled.
       case ipReasmFails:  insert_BER_counter(pbValue,mib_ip->ipReasmFails,&wIndex);
                           break;
                           // The number of failures detected by the IP re-
                           // assembly algorithm (for whatever reason: timed
                           // out, errors, etc).  Note that this is not
                           // necessarily a count of discarded IP fragments
                           // since some algorithms (notably the algorithm in
                           // RFC 815) can lose track of the number of fragments
                           // by combining them as they are received.

       case ipFragOKs:    insert_BER_counter(pbValue,mib_ip->ipFragOKs,&wIndex);
                          break;
                           // The number of IP datagrams that have been
                           // successfully fragmented at this entity.
       case ipFragFails:  insert_BER_counter(pbValue,mib_ip->ipFragFails,&wIndex);
                          break;
                           // The number of IP datagrams that have been
                           // discarded because they needed to be fragmented at
                           // this entity but could not be, e.g., because their
                           // Don't Fragment flag was set
       case ipFragCreates: insert_BER_counter(pbValue,mib_ip->ipFragCreates,&wIndex);
                           break;
                           // The number of IP datagram fragments that have
                           // been generated as a result of fragmentation at
                           // this entity.

       case ipAdEntAddr:
                           ip_to_array(iface_entry[tbl_idx].devIPAddr,abTemp);
                           insert_BER_ipaddress(pbValue,abTemp,&wIndex);
                           break;
                           //The IP address to which this entry's addressing
                           //information pertains.
       case ipAdEntIfIndex:
                           insert_BER_dword(pbValue,iface_entry[tbl_idx].devIndex,&wIndex);
                           break;
                           // The index value which uniquely identifies the
                           // interface to which this entry is applicable.  The
                           // interface identified by a particular value of this
                           // index is the same interface as identified by the
                           // same value of ifIndex
       case ipAdEntNetMask:
                           ip_to_array(iface_entry[tbl_idx].devNetmask,abTemp);
                           insert_BER_ipaddress(pbValue,abTemp,&wIndex);
                            break;
                           // The subnet mask associated with the IP address of
                           // this entry.  The value of the mask is an IP
                           // address with all the network bits set to 1 and all
                           // the hosts bits set to 0.
       case ipAdEntBcastAddr:
                           insert_BER_dword(pbValue,1,&wIndex);
                           break;
                           // The value of the least-significant bit in the IP
                           // broadcast address used for sending datagrams on
                           // the (logical) interface associated with the IP
                           // address of this entry.  For example, when the
                           // Internet standard all-ones broadcast address is
                           // used, the value will be 1.  This value applies to
                           // both the subnet and network broadcasts addresses
                           // used by the entity on this (logical) interface."
       case ipAdEntReasmMaxSize:
                           insert_BER_dword(pbValue,0xFFFFL,&wIndex);
                            break;
                           // The size of the largest IP datagram which this
                           // entity can re-assemble from incoming IP fragmented
                           // datagrams received on this interface."

       case ipRouteDest:   if(arproute_cacheEntry[tbl_idx].flags & LOCALROUTE)
                           {
                             ip_to_array(arproute_cacheEntry[tbl_idx].ArpRouteGwayUnion.LocalNetRouteEntry.DevIpAddress
                                         ,abTemp);
                           }
                           else
                           {
                             ip_to_array(arproute_cacheEntry[tbl_idx].ArpRouteGwayUnion.GatewayEntry.GatewayIpAddress
                                        ,abTemp);
                           }
                           insert_BER_ipaddress(pbValue,abTemp,&wIndex);
                           break;
                           // The destination IP address of this route.  An
                           // entry with a value of 0.0.0.0 is considered a
                           // default route.  Multiple routes to a single
                           // destination can appear in the table, but access to
                           // such multiple entries is dependent on the table-
                           // access mechanisms defined by the network
                           // management protocol in use.
       case ipRouteIfIndex:
                            insert_BER_dword(pbValue,arproute_cacheEntry[tbl_idx].ifIndex,&wIndex);
                            break;
                           // The index value which uniquely identifies the
                           // local interface through which the next hop of this
                           // route should be reached.  The interface identified
                           // by a particular value of this index is the same
                           // interface as identified by the same value of
                           // ifIndex.
       case ipRouteMetric1:
                           insert_BER_dword(pbValue,arproute_cacheEntry[tbl_idx].hops,&wIndex);
                           break;
                           // The primary routing metric for this route.  The
                           // semantics of this metric are determined by the
                           // routing-protocol specified in the route's
                           // ipRouteProto value.  If this metric is not used,
                           // its value should be set to -1.
       case ipRouteMetric2:
       case ipRouteMetric3:
       case ipRouteMetric4:
                           insert_BER_dword(pbValue,-1,&wIndex);
                           break;
                           // The primary routing metric for this route.  The
                           // semantics of this metric are determined by the
                           // routing-protocol specified in the route's
                           // ipRouteProto value.  If this metric is not used,
                           // its value should be set to -1.

       case ipRouteNextHop:
                            //inet_addr(ip,&dwTemp);
                            ip_to_array(agent_ip,abTemp);
                            insert_BER_ipaddress(pbValue,abTemp,&wIndex);
                            break;
                           // The IP address of the next hop of this route.
                           // (In the case of a route bound to an interface
                           // which is realized via a broadcast media, the value
                           // of this field is the agent's IP address on that
                           // interface.)
       case ipRouteType:
                           if(arproute_cacheEntry[tbl_idx].flags & LOCALROUTE)
                           {
                             insert_BER_dword(pbValue,3,&wIndex);
                           }
                           else
                           {
                             insert_BER_dword(pbValue,4,&wIndex);
                           }
                           break;
                           // The type of route.  Note that the values
                           // direct(3) and indirect(4) refer to the notion of
                           // direct and indirect routing in the IP
                           // architecture.
                           // Setting this object to the value invalid(2) has
                           // the effect of invalidating the corresponding entry
                           // in the ipRouteTable object.  That is, it
                           // effectively dissasociates the destination
                           // identified with said entry from the route
                           // identified with said entry.  It is an
                           // implementation-specific matter as to whether the
                           // agent removes an invalidated entry from the table.
                           // Accordingly, management stations must be prepared
                           // to receive tabular information from agents that
                           // corresponds to entries not currently in use.
                           // Proper interpretation of such entries requires
                           // examination of the relevant ipRouteType object.
       case ipRouteProto:
                           insert_BER_dword(pbValue,2,&wIndex);// entries set via a network
                           break;
                           // The routing mechanism via which this route was
                           // learned.  Inclusion of values for gateway routing
                           // protocols is not intended to imply that hosts
                           // should support those protocols.
       case ipRouteAge:
                           insert_BER_dword(pbValue,arproute_cacheEntry[tbl_idx].arpTtl/1000L,&wIndex);
                           break;
                           // The number of seconds since this route was last
                           // updated or otherwise determined to be correct.
                           // Note that no semantics of `too old' can be implied
                           // except through knowledge of the routing protocol
                           // by which the route was learned.


       case ipRouteMask:
                          ip_to_array(arproute_cacheEntry[tbl_idx].mask,abTemp);
                          insert_BER_ipaddress(pbValue,abTemp,&wIndex);
                          break;
                          // Indicate the mask to be logical-ANDed with the
                          // destination address before being compared to the
                          // value in the ipRouteDest field.  For those systems
                          // that do not support arbitrary subnet masks, an
                          // agent constructs the value of the ipRouteMask by
                          // determining whether the value of the correspondent
                          // ipRouteDest field belong to a class-A, B, or C
                          // network, and then using one of:
                          //
                          //         mask           network
                          //         255.0.0.0      class-A
                          //         255.255.0.0    class-B
                          //         255.255.255.0  class-C
                          //
                          // If the value of the ipRouteDest is 0.0.0.0 (a
                          // default route), then the mask value is also
                          // 0.0.0.0.  It should be noted that all IP routing
                          // subsystems implicitly use this mechanism.
       case ipRouteMetric:
                          insert_BER_dword(pbValue,-1,&wIndex);
                          break;
                          // An alternate routing metric for this route.  The
                          // semantics of this metric are determined by the
                          // routing-protocol specified in the route's
                          // ipRouteProto value.  If this metric is not used,
                          // its value should be set to -1."
       case ipRouteInfo:  memcpy(pbValue,abOIDNull2,3);
                          break;
                          // A reference to MIB definitions specific to the
                          // particular routing protocol which is responsible
                          // for this route, as determined by the value
                          // specified in the route's ipRouteProto value.  If
                          // this information is not present, its value should
                          // be set to the OBJECT IDENTIFIER  0 0 , which is
                          // a syntatically valid object identifier, and any
                          // conformant implementation of ASN.1 and BER must be
                          // able to generate and recognize this value.
       case ipNetToMediaIfIndex:
                          insert_BER_dword(pbValue,arproute_cacheEntry[tbl_idx].ifIndex,&wIndex);
                          break;
                          // The interface on which this entry's equivalence
                          // is effective.  The interface identified by a
                          // particular value of this index is the same
                          // interface as identified by the same value of
                          // ifIndex.
       case ipNetToMediaPhyAddress:
                         insert_BER_macaddress(pbValue,
                                               arproute_cacheEntry[tbl_idx].ArpRouteGwayUnion.ArpEntry.MacAddress
                                              ,&wIndex);
                         break;
                         // The media-dependent `physical' address.
       case ipNetToMediaNetAddress:
                         ip_to_array(arproute_cacheEntry[tbl_idx].arpIpAddress,abIPAddress);
                         insert_BER_ipaddress(pbValue
                                             ,abIPAddress
                                             ,&wIndex);
                         break;
                        // The IpAddress corresponding to the media-
                        // dependent `physical' address.
       case ipNetToMediaType:
                        insert_BER_dword(pbValue,3/*Dynamic??*/,&wIndex);
                        break;
                       // The type of mapping.
                       // Setting this object to the value invalid(2) has
                       // the effect of invalidating the corresponding entry
                       // in the ipNetToMediaTable.  That is, it effectively
                       // dissasociates the interface identified with said
                       // entry from the mapping identified with said entry.
                       // It is an implementation-specific matter as to
                       // whether the agent removes an invalidated entry
                       // from the table.  Accordingly, management stations
                       // must be prepared to receive tabular information
                       // from agents that corresponds to entries not
                       // currently in use.  Proper interpretation of such
                       // entries requires examination of the relevant
                       // ipNetToMediaType object
       case ipRoutingDiscards:
                      insert_BER_counter(pbValue,(int)mib_ip->ipRoutingDiscards,&wIndex);
                       break;
                       // The number of routing entries which were chosen
                       // to be discarded even though they are valid.  One
                       // possible reason for discarding such an entry could
                       // be to free-up buffer space for other routing
                       // entries.
       /**************************************************************************************/
       // ICMP Block:
       /**************************************************************************************/
       case icmpInMsgs:
                 insert_BER_counter(pbValue,mib_icmp->icmpInMsgs,&wIndex);
                 break;
       case icmpInErrors:
                 insert_BER_counter(pbValue,mib_icmp->icmpInErrors,&wIndex);
                 break;
       case icmpInDestUnreachs:
                 insert_BER_counter(pbValue,mib_icmp->icmpInDestUnreachs,&wIndex);
                 break;
       case icmpInTimeExcds:
                 insert_BER_counter(pbValue,mib_icmp->icmpInTimeExcds,&wIndex);
                 break;
       case icmpInParmProbs:
                 insert_BER_counter(pbValue,mib_icmp->icmpInParmProbs,&wIndex);
                 break;
       case icmpInSrcQuenchs:
                 insert_BER_counter(pbValue,mib_icmp->icmpInSrcQuenchs,&wIndex);
                 break;
       case icmpRedirects:
                 insert_BER_counter(pbValue,mib_icmp->icmpInRedirects,&wIndex);
                 break;
       case icmpInEchos:
                 insert_BER_counter(pbValue,mib_icmp->icmpInEchos,&wIndex);
                 break;
       case icmpInEchoReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpInEchoReps,&wIndex);
                 break;
       case icmpInTimestamps:
                 insert_BER_counter(pbValue,mib_icmp->icmpInTimestamps,&wIndex);
                 break;
       case icmpInTimestampReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpInTimestampReps,&wIndex);
                 break;
       case icmpInAddrMasks:
                 insert_BER_counter(pbValue,mib_icmp->icmpInAddrMasks,&wIndex);
                 break;
       case icmpInAddrMaskReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpInAddrMaskReps,&wIndex);
                 break;
       case icmpOutMsgs:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutMsgs,&wIndex);
                 break;
       case icmpOutErrors:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutErrors,&wIndex);
                 break;
       case icmpOutDestUnreachs:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutDestUnreachs,&wIndex);
                 break;
       case icmpOutTimeExcds:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutTimeExcds,&wIndex);
                 break;
       case icmpOutParmProbs:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutParmProbs,&wIndex);
                 break;
       case icmpOutSrcQuenchs:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutSrcQuenchs,&wIndex);
                 break;
       case icmpOutRedirects:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutRedirects,&wIndex);
                 break;
       case icmpOutEchos:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutEchos,&wIndex);
                 break;
       case icmpOutEchoReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutEchoReps,&wIndex);
                 break;
       case icmpOutTimestamps:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutTimestamps,&wIndex);
                 break;
       case icmpOutTimestampReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutTimestampReps,&wIndex);
                 break;
       case icmpOutAddrMasks:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutAddrMasks,&wIndex);
                 break;
       case icmpOutAddrMaskReps:
                 insert_BER_counter(pbValue,mib_icmp->icmpOutAddrMaskReps,&wIndex);
                 break;
       /**************************************************************************************/
       // TCP
       /**************************************************************************************/
       case tcpRtoAlgorithm:  // other
                       insert_BER_dword(pbValue,mib_tcp->tcpRtoAlgorithm,&wIndex);
                       break;
                       // The algorithm used to determine the timeout value
                       // used for retransmitting unacknowledged octets.
       case tcpRtoMin: insert_BER_dword(pbValue,mib_tcp->tcpRtoMin,&wIndex);
                       break;
                       // The minimum value permitted by a TCP
                       // implementation for the retransmission timeout,
                       // measured in milliseconds.  More refined semantics
                       // for objects of this type depend upon the algorithm
                       // used to determine the retransmission timeout.  In
                       // particular, when the timeout algorithm is rsre(3),
                       // an object of this type has the semantics of the
                       // LBOUND quantity described in RFC 793.

       case tcpRtoMax: insert_BER_dword(pbValue,mib_tcp->tcpRtoMax,&wIndex);
                       break;
                       // The maximum value permitted by a TCP
                       // implementation for the retransmission timeout,
                       // measured in milliseconds.  More refined semantics
                       // for objects of this type depend upon the algorithm
                       // used to determine the retransmission timeout.  In
                       // particular, when the timeout algorithm is rsre(3),
                       // an object of this type has the semantics of the
                       // UBOUND quantity described in RFC 793.
       case tcpMaxConn: insert_BER_dword(pbValue,mib_tcp->tcpMaxConn,&wIndex);
                        break;
                       // The limit on the total number of TCP connections
                       // the entity can support.  In entities where the
                       // maximum number of connections is dynamic, this
                       // object should contain the value -1.
       case tcpActiveOpens:
                       insert_BER_counter(pbValue,mib_tcp->tcpActiveOpens,&wIndex);
                       break;
                       // The number of times TCP connections have made a
                       // direct transition to the SYN-SENT state from the
                       // CLOSED state.

       case tcpPassivOpens:
                       insert_BER_counter(pbValue,mib_tcp->tcpPassiveOpens,&wIndex);
                       break;
                       // The number of times TCP connections have made a
                       // direct transition to the SYN-RCVD state from the
                       // LISTEN state.
       case tcpAttemptFails:
                       insert_BER_counter(pbValue,mib_tcp->tcpAttemptFails,&wIndex);
                       break;
                       // The number of times TCP connections have made a
                       // direct transition to the CLOSED state from either
                       // the SYN-SENT state or the SYN-RCVD state, plus the
                       // number of times TCP connections have made a direct
                       // transition to the LISTEN state from the SYN-RCVD
                       // state.

       case tcpEstabResets:
                       insert_BER_counter(pbValue,mib_tcp->tcpEstabResets,&wIndex);
                        break;
                       // The number of times TCP connections have made a
                       // direct transition to the CLOSED state from either
                       // the ESTABLISHED state or the CLOSE-WAIT state.
       case tcpCurrEstab:
                        insert_BER_gauge(pbValue,mib_tcp->tcpCurrEstab,&wIndex);
                        break;
                       // The number of TCP connections for which the
                       // current state is either ESTABLISHED or CLOSE-
                       // WAIT.

       case tcpInSegs:
                       insert_BER_counter(pbValue,mib_tcp->tcpInSegs,&wIndex);
                       break;
                       // The total number of segments received, including
                       // those received in error.  This count includes
                       // segments received on currently established
                       // connections.
      case tcpOutSegs:
                       insert_BER_counter(pbValue,mib_tcp->tcpOutSegs,&wIndex);
                       break;
                       // The total number of segments sent, including
                       // those on current connections but excluding those
                       // containing only retransmitted octets.
       case tcpRetransSegs:
                       insert_BER_counter(pbValue,mib_tcp->tcpRetransSegs,&wIndex);
                       break;
                       // The total number of segments retransmitted - that
                       // is, the number of TCP segments transmitted
                       // containing one or more previously transmitted
                       // octets.
       case tcpConnState:
                       insert_BER_dword(pbValue,socks[tbl_idx].tcpState,&wIndex);
                       break;
                       // The state of this TCP connection.
                       //
                       // The only value which may be set by a management
                       // station is deleteTCB(12).  Accordingly, it is
                       // appropriate for an agent to return a `badValue'
                       // response if a management station attempts to set
                       // this object to any other value.
                       //
                       // If a management station sets this object to the
                       // value deleteTCB(12), then this has the effect of
                       // deleting the TCB (as defined in RFC 793) of the
                       // corresponding connection on the managed node,
                       // resulting in immediate termination of the
                       // connection.
                       //
                       // As an implementation-specific option, a RST
                       // segment may be sent from the managed node to the
                       // other TCP endpoint (note however that RST segments
                       // are not sent reliably).
       case tcpConnLocalAddress:
                       // The local IP address for this TCP connection.  In
                       // the case of a connection in the listen state which
                       // is willing to accept connections for any IP
                       // interface associated with the node, the value
                       // 0.0.0.0 is used.

                       if(socks[tbl_idx].tcpState==1)  //Listen
                          ip_to_array(0L,&abIPAddr[2]);
                       else
                          ip_to_array(socks[tbl_idx].IfIpAddress,&abIPAddr[2]);
                       memcpy(pbValue,abIPAddr,6);
                       break;

       case tcpConnLocalPort:
                       // The local port number for this TCP connection.
                       insert_BER_dword(pbValue,socks[tbl_idx].localPort,&wIndex);
                       break;

       case tcpConnRemAddress:
                       // The remote IP address for this TCP connection.
                       ip_to_array(socks[tbl_idx].remoteIP,&abIPAddr[2]);
                       memcpy(pbValue,abIPAddr,6);
                       break;

       case tcpConnRemPort:
                       // The remote port number for this TCP connection.
                       insert_BER_dword(pbValue,socks[tbl_idx].remotePort,&wIndex);
                       break;

       case tcpInErrs:
                    insert_BER_counter(pbValue,mib_tcp->tcpInErrs,&wIndex);
                    break;
       case tcpOutRsts:
                    insert_BER_counter(pbValue,mib_tcp->tcpOutRsts,&wIndex);
                    break;

       /**************************************************************************************/
       /*UDP */
       /**************************************************************************************/
       case udpInDatagrams: insert_BER_counter(pbValue,mib_udp->udpInDatagrams,&wIndex);
                            break;
                       // The total number of UDP datagrams delivered to
                       // UDP users."
       case udpNoPorts:   insert_BER_counter(pbValue,mib_udp->udpNoPorts,&wIndex);
                          break;
                       // The total number of received UDP datagrams for
                       // which there was no application at the destination
                       // port.
       case udpInErrors:  insert_BER_counter(pbValue,mib_udp->udpInErrors,&wIndex);
                          break;
                       // The number of received UDP datagrams that could
                       // not be delivered for reasons other than the lack
                       // of an application at the destination port.
       case udpOutDatagrams:
                          insert_BER_counter(pbValue,mib_udp->udpInDatagrams,&wIndex);
                          break;
                       // The total number of UDP datagrams sent from this
                       // entity.
       case udpLocalAddress:
                          ip_to_array(0L,&abIPAddr[2]);
                          memcpy(pbValue,abIPAddr,6);
                          break;
                       // The local IP address for this UDP listener.  In
                       // the case of a UDP listener which is willing to
                       // accept datagrams for any IP interface associated
                       // with the node, the value 0.0.0.0 is used.
       case udpLocalPort: insert_BER_dword(pbValue,socks[tbl_idx].localPort,&wIndex); break;
                       // The local port number for this UDP listener.
       /**************************************************************************************/
       // SNMP
       /**************************************************************************************/
       case snmpInPkts: insert_BER_counter(pbValue,mib_snmp.snmpInPkts,&wIndex);
                        break;
                       // The total number of Messages delivered to the
                       // SNMP entity from the transport service.
       case snmpOutPkts: insert_BER_counter(pbValue,mib_snmp.snmpOutPkts,&wIndex);
                        break;
                       // The total number of SNMP Messages which were
                       // passed from the SNMP protocol entity to the
                       // transport service.
       case snmpInBadVersions:
                       insert_BER_counter(pbValue,mib_snmp.snmpInBadVersions,&wIndex);
                        break;
                       // The total number of SNMP Messages which were
                       // delivered to the SNMP protocol entity and were for
                       // an unsupported SNMP version.
       case snmpInBadCommunityNames:
                       insert_BER_counter(pbValue,mib_snmp.snmpInBadCommunityNames,&wIndex);
                       break;
                       // The total number of SNMP Messages delivered to
                       // the SNMP protocol entity which used a SNMP
                       // community name not known to said entity.
       case snmpInBadCommunityUses:
                       insert_BER_counter(pbValue,mib_snmp.snmpInBadCommunityUses,&wIndex);
                       break;
                       // The total number of SNMP Messages delivered to
                       // the SNMP protocol entity which represented an SNMP
                       // operation which was not allowed by the SNMP
                       // community named in the Message.
       case snmpInASNParseErrs:
                       insert_BER_counter(pbValue,mib_snmp.snmpInASNParseErrs,&wIndex);
                       break;
                       // The total number of ASN.1 or BER errors
                       // encountered by the SNMP protocol entity when
                       // decoding received SNMP Messages.
       case snmpInTooBigs:
                       insert_BER_counter(pbValue,mib_snmp.snmpInTooBigs,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // delivered to the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `tooBig'.
       case snmpInNoSuchNames:
                       insert_BER_counter(pbValue,mib_snmp.snmpInNoSuchNames,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // delivered to the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `noSuchName'.
       case snmpInBadValues:
                       insert_BER_counter(pbValue,mib_snmp.snmpInBadValues,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // delivered to the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `badValue'.
       case snmpInReadOnlys:
                       insert_BER_counter(pbValue,mib_snmp.snmpInReadOnlys,&wIndex);
                       break;
                       // The total number valid SNMP PDUs which were
                       // delivered to the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `readOnly'.  It should be noted that it is a
                       // protocol error to generate an SNMP PDU which
                       // contains the value `readOnly' in the error-status
                       // field, as such this object is provided as a means
                       // of detecting incorrect implementations of the
                       // SNMP.
       case snmpIngGenErrs:
                       insert_BER_counter(pbValue,mib_snmp.snmpInGenErrs,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // delivered to the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `genErr'.
       case snmpInTotalReqVars:
                       insert_BER_counter(pbValue,mib_snmp.snmpInTotalReqVars,&wIndex);
                       break;
                       // The total number of MIB objects which have been
                       // retrieved successfully by the SNMP protocol entity
                       // as the result of receiving valid SNMP Get-Request
                       // and Get-Next PDUs.
       case snmpInTotalSetVars:
                       insert_BER_counter(pbValue,mib_snmp.snmpInTotalSetVars,&wIndex);
                       break;
                       // The total number of MIB objects which have been
                       // altered successfully by the SNMP protocol entity
                       // as the result of receiving valid SNMP Set-Request
                       // PDUs.

       case snmpInGetRequests:
                       insert_BER_counter(pbValue,mib_snmp.snmpInGetRequests,&wIndex);
                       break;
                       // The total number of SNMP Get-Request PDUs which
                       // have been accepted and processed by the SNMP
                       // protocol entity.
       case snmpInGetNexts:
                       insert_BER_counter(pbValue,mib_snmp.snmpInGetNexts,&wIndex);
                       break;
                       // The total number of SNMP Get-Next PDUs which have
                       // been accepted and processed by the SNMP protocol
                       // entity.
       case snmpInSetReqeusts:
                       insert_BER_counter(pbValue,mib_snmp.snmpInSetRequests,&wIndex);
                       break;
                       // The total number of SNMP Set-Request PDUs which
                       // have been accepted and processed by the SNMP
                       // protocol entity.
       case snmpInGetResponses:
                       insert_BER_counter(pbValue,mib_snmp.snmpInGetResponses,&wIndex);
                       break;
                       // The total number of SNMP Get-Response PDUs which
                       // have been accepted and processed by the SNMP
                       // protocol entity.
       case snmpInTraps:
                        insert_BER_counter(pbValue,mib_snmp.snmpInTraps,&wIndex);
                        break;
                       // The total number of SNMP Trap PDUs which have
                       // been accepted and processed by the SNMP protocol
                       // entity.
       case snmpOutTooBigs:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutTooBigs,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // generated by the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `tooBig.
       case snmpOutNoSuchNames:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutNoSuchNames,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // generated by the SNMP protocol entity and for
                       // which the value of the error-status is
                       // `noSuchName'.
       case snmpOutBadValues:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutBadValues,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // generated by the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `badValue'
       case snmpOutGenErrs:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutGenErrs,&wIndex);
                       break;
                       // The total number of SNMP PDUs which were
                       // generated by the SNMP protocol entity and for
                       // which the value of the error-status field is
                       // `genErr'.
       case snmpOutGetRequests:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutGetRequests,&wIndex);
                       break;
                       // The total number of SNMP Get-Request PDUs which
                       // have been generated by the SNMP protocol entity.
       case snmpOutGetNexts:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutGetNexts,&wIndex);
                       break;
                       // The total number of SNMP Get-Next PDUs which have
                       // been generated by the SNMP protocol entity.
       case snmpOutSetRequests:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutSetRequests,&wIndex);
                       break;
                       // The total number of SNMP Set-Request PDUs which
                       // have been generated by the SNMP protocol entity.
       case snmpOutGetResponses:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutGetResponses,&wIndex);
                       break;
                       // The total number of SNMP Get-Response PDUs which
                       // have been generated by the SNMP protocol entity.
       case snmpOutTraps:
                       insert_BER_counter(pbValue,mib_snmp.snmpOutTraps,&wIndex);
                       break;
                       // The total number of SNMP Trap PDUs which have
                       // been generated by the SNMP protocol entity.
       case snmpEnableAuthTraps:
                       insert_BER_dword(pbValue,mib_snmp.snmpEnableAuthTraps,&wIndex);
                       break;
                       // Indicates whether the SNMP agent process is
                       // permitted to generate authentication-failure
                       // traps.  The value of this object overrides any
                       // configuration information; as such, it provides a
                       // means whereby all authentication-failure traps may
                       // be disabled.
                       //
                       // Note that it is strongly recommended that this
                       // object be stored in non-volatile memory so that it
                       // remains constant between re-initializations of the
                       // network management system.



      #ifdef DKXX_SNMP_DEMO
      /*Example for appending own SNMP variables: Handle DK40/DK50 IO port GET requests*/
      case dkxxOut0:    insert_BER_dword(pbValue,get_dkxx_bit(0),&wIndex);
                        break;
      case dkxxOut1:    insert_BER_dword(pbValue,get_dkxx_bit(1),&wIndex);
                        break;
      case dkxxOut2:    insert_BER_dword(pbValue,get_dkxx_bit(2),&wIndex);
                        break;

      case dkxxOut3:    insert_BER_dword(pbValue,get_dkxx_bit(3),&wIndex);
                        break;

      case dkxxOut4:    insert_BER_dword(pbValue,get_dkxx_bit(4),&wIndex);
                        break;

      case dkxxOut5:    insert_BER_dword(pbValue,get_dkxx_bit(5),&wIndex);
                        break;

      case dkxxOut6:    insert_BER_dword(pbValue,get_dkxx_bit(6),&wIndex);
                        break;
      case dkxxOut7:    insert_BER_dword(pbValue,get_dkxx_bit(7),&wIndex);
                        break;
      #endif

      default :        iReturn = SNMP_ERROR_BADVALUE_ERR;
                       break;
   }//switch (wChannel)
   return iReturn;
}
/***************************************************************************************/
//Set value
/***************************************************************************************/
static int  SNMP_NEAR set_SNMP_value(unsigned int bChan, unsigned char * pbValue, unsigned char tbl_idx)
{
   int iReturn = NO_ERR;
   unsigned long dwTemp;
   unsigned int wDummy;
   unsigned char ucDummy;

   switch (bChan)
   {
       case  sysContact :
                   convert_BER_to_string(pbValue,sSystemMibContact);
                   ConfigDataChanged = TRUE;
                   break;
       case  sysName :
                   convert_BER_to_string(pbValue,sSystemMibName);
                   ConfigDataChanged = TRUE;
                   break;
       case  sysLocation :
                   convert_BER_to_string(pbValue,sSystemMibLocation);
                   ConfigDataChanged = TRUE;
                   break;

       case  snmpEnableAuthTraps :
                   mib_snmp.snmpEnableAuthTraps = get_SNMP_dword(pbValue,&wDummy);
                   if (mib_snmp.snmpEnableAuthTraps == TRAP_ENABLED)
                   {
                       ConfigDataChanged = TRUE;
                       get_SNMP_value(snmpEnableAuthTraps,pbValue,0);
                   }
                   else
                   {
                       if (mib_snmp.snmpEnableAuthTraps== TRAP_DISABLED)
                       {
                           ConfigDataChanged = TRUE;
                           get_SNMP_value(snmpEnableAuthTraps,pbValue,0);
                       }
                       else
                       {
                           iReturn =  SNMP_ERROR_BADVALUE_ERR;
                       }
                   }
                   break;
       case  ifAdminStatus :
                   dwTemp = get_SNMP_dword(pbValue,&wDummy);
                   if (dwTemp == INTERFACE_DOWN)
                   {
                       set_interface_status(INTERFACE_DOWN);
                       get_SNMP_value(ifAdminStatus,pbValue,tbl_idx);
                   }
                   else
                   {
                       if (dwTemp== INTERFACE_UP)
                       {
                           set_interface_status(INTERFACE_UP);
                           get_SNMP_value(ifAdminStatus,pbValue,tbl_idx);
                       }
                       else
                       {
                           if (dwTemp== INTERFACE_TEST)
                           {
                               set_interface_status(INTERFACE_TEST);
                               get_SNMP_value(ifAdminStatus,pbValue,tbl_idx);
                           }
                           else
                           {
                               iReturn =  SNMP_ERROR_BADVALUE_ERR;
                               get_SNMP_value(ifAdminStatus,pbValue,tbl_idx);
                           }
                       }
                   }
                   break;
       case  atIfIndex :
                   //Not supported, return current value
                   get_SNMP_dword(pbValue,&wDummy);
                   get_SNMP_value(atIfIndex,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

       case  atPhysAddress :
                   //Not supported, return current value
                   get_SNMP_value(atPhysAddress,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

       case  atNetAddress :
                   //Not supported,
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

       case  ipForwarding :
                   //Not supported, return current value
                   get_SNMP_value(ipForwarding,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case  ipDefaultTTL :
                   //Not supported, return current value
                   get_SNMP_value(ipDefaultTTL,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

       case   ipRouteDest:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteDest,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteIfIndex:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteIfIndex,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMetric1:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMetric1,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMetric2:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMetric2,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMetric3:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMetric3,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMetric4:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMetric4,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteNextHop:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteNextHop,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteType:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteType,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteAge:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteAge,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMask:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMask,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipRouteMetric:
                   //Not supported, return current value
                   get_SNMP_value(ipRouteMetric,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipNetToMediaIfIndex:
                   //Not supported, return current value
                   get_SNMP_value(ipNetToMediaIfIndex,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipNetToMediaPhyAddress:
                   //Not supported, return current value
                   get_SNMP_value(ipNetToMediaPhyAddress,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipNetToMediaNetAddress:
                   //Not supported, return current value
                   get_SNMP_value(ipNetToMediaNetAddress,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;
       case   ipNetToMediaType:
                   //Not supported, return current value
                   get_SNMP_value(ipNetToMediaType,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

       case  tcpConnState :
                   //Not supported, return current value
                   get_SNMP_value(tcpConnState,pbValue,tbl_idx);
                   iReturn = SNMP_ERROR_BADVALUE_ERR;
                   break;

      #ifdef DKXX_SNMP_DEMO
      /*Example for appending own SNMP variables: Handle DK40 IO port GET requests*/
      case dkxxOut0:

                        ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(0,ucDummy);
                        }
                        break;
      case dkxxOut1:    ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(1,ucDummy);
                        }
                        break;
      case dkxxOut2:    ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(2,ucDummy);
                        }
                        break;

      case dkxxOut3:    ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(3,ucDummy);
                        }
                        break;

      case dkxxOut4:    ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(4,ucDummy);
                        }
                        break;

      case dkxxOut5:
                        ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(5,ucDummy);
                        }
                        break;

      case dkxxOut6:
                        ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(6,ucDummy);
                        }
                        break;
      case dkxxOut7:
                        ucDummy = (unsigned char)get_SNMP_dword(pbValue,&wDummy);
                        if(ucDummy>1)
                        {
                          iReturn = SNMP_ERROR_BADVALUE_ERR;
                        }
                        else
                        {
                          set_dkxx_bit(7,ucDummy);
                        }
                        break;
      #endif

       default :  iReturn = SNMP_ERROR_BADVALUE_ERR;
                  break;

   }
   return iReturn;
}
/***************************************************************************************/
//Write modified config data to chip.ini
/***************************************************************************************/
static void SNMP_NEAR check_SNMP_cfgdata(void)
{
   if (ConfigDataChanged == TRUE)
   {
       BIOS_Set_Ini_String(SNMP_Section,SNMP_Contact,sSystemMibContact);
       BIOS_Set_Ini_String(SNMP_Section,SNMP_Name,sSystemMibName);
       BIOS_Set_Ini_String(SNMP_Section,SNMP_Location,sSystemMibLocation);
       if (mib_snmp.snmpEnableAuthTraps == TRAP_ENABLED)
       {
           BIOS_Set_Ini_String(SNMP_Section,SNMP_AuthTraps,"Enabled");
       }
       else
       {
           BIOS_Set_Ini_String(SNMP_Section,SNMP_AuthTraps,"Disabled");
       }

       BIOS_Set_Ini_String(SNMP_Section,SNMP_TrapCommunity,sTrapCommunity);
       BIOS_Set_Ini_String(SNMP_Section,SNMP_WriteCommunity,sWriteCommunity);
       BIOS_Set_Ini_String(SNMP_Section,SNMP_ReadCommunity,sReadCommunity);

       ConfigDataChanged = FALSE;
   }
}
/***************************************************************************************/
//Send SNMP trap
//To do: Inprove and simplify sending of traps
/***************************************************************************************/
int send_SNMP_trap(unsigned char bTrapNumber, unsigned char bSecTrapNumber)
{
   int iReturn= NO_ERR;

   if ((mib_snmp.snmpEnableAuthTraps == TRAP_ENABLED) && (trap_ip!=0L))
   {
       RTX_Reserve_Sem(trap_semaphoreID,0L);

       tTrapMessage.tSNMPHeadData.bVersion = SNMP_VERSION_1;
       strcpy (tTrapMessage.tSNMPHeadData.bCommunityString , sTrapCommunity);
       tTrapMessage.tTrapDataHead.bPDUType = SNMP_TRP_REQ_PDU; // Trap Message
       memcpy(tTrapMessage.tTrapDataHead.bEnterpriseMib,OIDSTART,8);

       ip_to_array(agent_ip,tTrapMessage.tTrapDataHead.bAgentIPAddress);

       tTrapMessage.tTrapDataHead.wStandardTrapType = bTrapNumber;
       tTrapMessage.tTrapDataHead.wSpecificTrapType = bSecTrapNumber;
       tTrapMessage.tTrapDataHead.dwTimeStamp = *sysuptime;

       insert_trap_oid(bTrapNumber,bSecTrapNumber, &tTrapMessage.tTrapDataHead.wNumberOfBindings, &tTrapMessage.tPUDBindings[0]);

       convert_SNMPtrap_to_BER( &tTrapMessage, bTrapSend);

       iReturn = send_BER_trap(bTrapSend,trap_ip, SNMP_TRAP_PORT);
       mib_snmp.snmpOutTraps++;

       RTX_Release_Sem(trap_semaphoreID);
   }
   return iReturn;
}
/***************************************************************************************/
//Send BER trap
/***************************************************************************************/
static int SNMP_NEAR send_BER_trap(unsigned char * abTrapMessage, unsigned long IPAddr, unsigned int wPort)
{
   int iReturn = NO_ERR;
   int  err;
   unsigned int wLength;
   unsigned int wDummy;
   struct sockaddr_in to;

   wLength = get_BER_send_length(&abTrapMessage[1],&wDummy);

   to.sin_family      = AF_INET;
   to.sin_addr.s_addr = IPAddr;
   to.sin_port        = htons(wPort);

   iReturn = sendto(snmp_socket,
                    (char far *)abTrapMessage,
                    wLength,MSG_BLOCKING,
                    (const struct sockaddr_in far * )&to
                    ,&err);

   if (err != 0)
   {
       iReturn = SNMP_TRAP_SEND_ERR;
   }

   return iReturn;
}
/***************************************************************************************/
/* Insert OID and Value bindings at trap message, identified by parameter
   bTrapNumber and bSecTrapNumber
   bTrapNumber constants are defined at snmphlp.h

   To do: Improve and simplify sending of traps

*/
/***************************************************************************************/
static int  SNMP_NEAR insert_trap_oid(unsigned char bTrapNumber, unsigned char bSecTrapNumber, unsigned int * wNumberBindings, tPDUBinding * ptBindings)
{
   int iReturn = NO_ERR;
   unsigned int wIndex=0;

   *wNumberBindings =0;

   //Default: Insert  "Undefined message" and OIDSTART
   insert_BER_string(ptBindings[0].abValueBinding,"Not defined",&wIndex);
   memcpy(ptBindings[0].abOIDBinding,OIDSTART,8);

   //Coldstart trap
   if ((bTrapNumber == SNMP_TRAP_COLDSTART)&&  (bSecTrapNumber==0))
   {
       memcpy(ptBindings[0].abOIDBinding,OIDSTART,8);
       insert_BER_string(ptBindings[0].abValueBinding,"!Coldstart",&wIndex);
       *wNumberBindings =1;
   }
   // Community error
   if ((bTrapNumber == SNMP_TRAP_AUTHFAIL)&&  (bSecTrapNumber==0))  {
       insert_BER_string(ptBindings[0].abValueBinding,"!Agent receive unexpect community",&wIndex);
       *wNumberBindings =1;
   }

   #ifdef DKXX_SNMP_DEMO
   //Example for insert an user specific trap
   //This trap will be send, if DK40/Dk50 output bit7 goes high (see endlessloop at main.c)
   if ((bTrapNumber == SNMP_TRAP_ENTERPRISESPECIFIC)  &&  (bSecTrapNumber==DKXX_TRAP_O_7))
   {
       memcpy(ptBindings[0].abOIDBinding,abDKXX_O_7_high,11);    //insert the Trap OID
       get_SNMP_value(dkxxOut7,ptBindings[0].abValueBinding,0);   //insert value of DKxx IO port Bit 7
       memcpy(ptBindings[1].abOIDBinding,abDKXX_O_7_high,11);    //insert the Trap OID
       insert_BER_string(ptBindings[1].abValueBinding,"!DKxx output 7 is high",&wIndex); //string message
       *wNumberBindings =2;
   }
   #endif
   return iReturn;
}
/***************************************************************************************/
//Process an incoming SNMP message
/***************************************************************************************/
static void SNMP_NEAR SNMP_message_input(struct sockaddr_in * tSend,
                                         unsigned char * rcvbuf,unsigned char * sndbuf)
{
   int iReturn= NO_ERR;

   if ((iReturn = convert_BER_to_SNMP(rcvbuf,&tReceivesMessage))== NO_ERR)
   {
       if (tReceivesMessage.tSNMPHeadData.bVersion != SNMP_VERSION_1)
       {
            mib_snmp.snmpInBadVersions++;
       }
       else
       {

           if (analyze_SNMP_message(&tReceivesMessage,&tSendMessage)== NO_ERR)
           {

              if (SNMP_convert_to_BER (&tSendMessage,sndbuf) == NO_ERR)
              {
                  send_BER_message(sndbuf,tSend);
              }
           }
       }
   }
   if (iReturn != NO_ERR)
   {
       if (iReturn == SNMP_IN_TO_BIG_ERR)
       {
            mib_snmp.snmpInTooBigs++;
       }
       else
       {
            mib_snmp.snmpInASNParseErrs++;
       }
   }
}
/***************************************************************************************/
//Some Set/Get functions
/***************************************************************************************/
int set_interface_status(int iNewStatus)
{
   int iReturn = NO_ERR;

   if (iInterface_Status != iNewStatus)
   {
       dwChangeInterfaceStatusTime =  *sysuptime;
       iInterface_Status = iNewStatus;
       if  (iInterface_Status == INTERFACE_DOWN)
       {
           iInterface_Status = INTERFACE_UP;

       }
   }
   return iReturn;
}
/*****************************************************************************/
#ifdef DKXX_SNMP_DEMO
/*****************************************************************************/
void set_dkxx_bit(unsigned char bit, unsigned char val /* 0 or 1 */)
{
   unsigned char old_outval;   //store DKXX evaluation board output values

   if(target==SC12)  //SC12, we assume a DK40 evaluation board
   {
      if(bit<7)
      {
         //read the current value
         old_outval = inportb(0x600);
         switch(val)
         {
                 case 0:
                         old_outval &= ~(1<<bit);
                         outportb(0x600,old_outval);

                         break;

                 case 1: 
                         old_outval |=  1<<bit;
                         outportb(0x600,old_outval);
                         break;

                 default: break;
         }
      }
      else
      {
        if(bit==7)  //Pio 13
        {
            hal_write_pio ( 13,val);
        }
      }
   }//if(target==SC12)..
   else //SC13, we assume a DK50 evaluation board
   {
         
         //read the current value
         old_outval = inportb(0x600);
         switch(val)
         {
                 case 0: old_outval &= ~(1<<bit);
                         outportb(0x600,old_outval);
                         break;

                 case 1: old_outval |=  1<<bit;
                         outportb(0x600,old_outval);
                         break;

                 default: break;
         }
   }
}

unsigned char get_dkxx_bit(unsigned char bit)
{
   if(target==SC12)  //we assume a DK40 evaluation board
   {
      if(bit<7)
      {
        // read the dk40/dk50 i/o port
        return (inportb(0x0600)&(1<<bit)) ? 1 : 0;
      }
      else
      {
        return hal_read_pio(13);
      }
   }
   else        //SC13, we assume a DK50 evaluation board
   {
      return (inportb(0x0600)&(1<<bit)) ? 1 : 0;
   }
}
/*****************************************************************************/
#endif /*DKXX_SNMP_DEMO*/
/*****************************************************************************/
// Allocate memory for dynamic table OID strings
/*****************************************************************************/
static int SNMP_NEAR alloc_oid_strings(int * old_max, int new_max, int how_many,
                                       int oid_strlength, char *oid_cptr[])
{
   int i;
   char * tmpptr;

   if(*old_max<new_max) //alloc only, if necessary
   {
      //Release old allocated memory
      if(oid_cptr[0]!=NULL)
      {
         helper_release_rtos_mem(FP_SEG(oid_cptr[0]));
      }

      tmpptr=MK_FP(helper_alloc_rtos_mem(new_max*oid_strlength*how_many),0);
      if(tmpptr==NULL)
      {
         helper_printf("\r\nSNMP memory allocation failed\r\n");
         return -1;
      }
      //Allocate memory for OID strings
      for(i=0;i<new_max*how_many;i++)
      {
         oid_cptr[i]=tmpptr;
         tmpptr+=oid_strlength;
      }
   }
   *old_max=new_max;
   return 0;
}

/*****************************************************************************/
// Building dymic mib-ii tables
/*****************************************************************************/
int build_dynamic_mib_tables(void)
{
   int i,j,k,ifs,avail,arp_route_entries;
   char abSTemp[6];
   static int old_max_dev=0;
   static int old_arp_route_entries=0;
   static int old_ifs=0;
   static int old_iproute=0;
   static int old_ipntm=0;
   static int old_udp=0;
   static int old_tcp=0;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Get current pointers to TCPIP internal device interface structs
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   max_dev=MAX_DEV;
   Get_SNMP_DataEx(7,(void *)&devIfMib,(int*)&max_dev);
   if(-1==alloc_oid_strings(&old_max_dev,max_dev,
                            ifSpecific-ifIndex + 1,
                            MAX_IFACE_OID_LEN,
                            iface_oid_str)
     )
   {
      return -1; //fatal memory allocation problem
   }

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building address translation table
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   arp_route_entries= GetARPRouteCacheEntries(arproute_cacheEntry);
   if(-1==alloc_oid_strings(&old_arp_route_entries,arp_route_entries,
                            atNetAddress - atIfIndex + 1,
                            MAX_AT_OID_LEN,
                            at_oid_str)
     )
   {
      return -1;  //Memory allocation problem
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Request all open sockets from TCPIP
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   avail=FindAllOpenSockets(&socks[0],MAX_SOCKETS);
   if(-1==alloc_oid_strings(&old_udp,avail,
                            udpLocalPort-udpLocalAddress + 1,
                            MAX_UDP_OID_LEN,
                            udp_oid_str)
     )
   {
      return -1; //fatal memory allocation problem
   }
   if(-1==alloc_oid_strings(&old_tcp,avail,
                            tcpConnRemPort-tcpConnState + 1,
                            MAX_TCP_OID_LEN,
                            tcp_oid_str)
     )
   {
      return -1; //fatal memory allocation problem
   }

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Request all interface entries
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ifs = Get_Iface_Entries(&iface_entry[0],MAX_DEV);
   if(-1==alloc_oid_strings(&old_ifs,ifs,
                            ipAdEntReasmMaxSize - ipAdEntAddr + 1,
                            MAX_IPADDR_OID_LEN,
                            ipaddr_oid_str)
     )
   {
      return -1; //fatal memory allocation problem
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building table of found device interface data
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   for(j=ifIndex,k=0;j<=ifSpecific;j++)
   {
      for(i=0;i<max_dev;i++)
      {
           strcpy(iface_oid_str[k],tMIB[j].abAddress);  //copy base OID string
           strcat(iface_oid_str[k], itoa((unsigned int)i+1,abSTemp,10)); //append table index
           iface_tble[k].abAddress =iface_oid_str[k];  //set abAddress pointer
           iface_tble[k].wChannel  = j;
           iface_tble[k].tbl_idx   = (unsigned char)i;    // table index for get_snmp_value
           iface_tble[k].baseOIDindex=2;
           iface_tble[k++].mibPointer = (tMib *)0L;       // null out next mib pointer
      }
   }
   //point to next mib root tree entry, see mib.c
   if(arp_route_entries)
   {
     iface_tble[k-1].mibPointer = &at_tble[0];
   }
   else
   {
     iface_tble[k-1].mibPointer = &tMIB[ipForwarding];
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building address translation table from requested arp cache
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   for(i=0,k=0;i<arp_route_entries;i++)
   {
       if(arproute_cacheEntry[i].flags & (LINKLAYER | CLONED_ARP))
       {
          for(j=atIfIndex;j<=atNetAddress;j++)
          {
              //1.3.6.1.2.1.3.1.1.x.ifIndex.1.A.B.C.D,  where A.B.C.D is IP address.
              strcpy(at_oid_str[k],tMIB[j].abAddress);  //copy base OID string
              strcat(at_oid_str[k], itoa((unsigned int)arproute_cacheEntry[i].ifIndex,abSTemp,10)); //append ifindex
              strcat(at_oid_str[k],".1");
              append_ip_to_oidstring(arproute_cacheEntry[i].arpIpAddress,at_oid_str[k]);
              at_tble[k].abAddress=at_oid_str[k];     //set abAddress pointer
              at_tble[k].wChannel   = j;
              at_tble[k].tbl_idx    = (unsigned char)i; //table index for get_snmp_value
              at_tble[k].baseOIDindex=3;
              at_tble[k++].mibPointer = (tMib *)0L;   //null out next mib pointer
          }
       }
   }
   //point to next mib root tree entry, see mib.c
   if(k)
   {
     at_tble[k-1].mibPointer = &tMIB[ipForwarding];
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building IP address  table
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   for(i=0,k=0;i<ifs;i++)
   {
       for(j=ipAdEntAddr;j<=ipAdEntReasmMaxSize;j++)
       {
          strcpy(ipaddr_oid_str[k],tMIB[j].abAddress);  //copy base OID string
          append_ip_to_oidstring(iface_entry[i].devIPAddr,ipaddr_oid_str[k]);
          ipaddr_tble[k].abAddress  = ipaddr_oid_str[k];     //set abAddress pointer
          ipaddr_tble[k].wChannel   = j;
          ipaddr_tble[k].tbl_idx    = (unsigned char)i;      //table index for get_snmp_value
          ipaddr_tble[k].baseOIDindex   = 5;
          ipaddr_tble[k++].mibPointer = (tMib *)0L;   //null out next mib pointer
       }//for(j
   }//for(i

   if(k)
   {
     ipaddr_tble[k-1].mibPointer = &iprt_tble[0];
   }
   else
   {
      tMIB[ipFragCreates].mibPointer = &iprt_tble[0];
   }
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //Building IP route table
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  if(-1==alloc_oid_strings(&old_iproute,arp_route_entries,
                            ipRouteInfo-ipRouteDest + 1,
                            MAX_RT_OID_LEN,
                            iprt_oid_str)
     )
   {
      return -1;  //fatal memory allocation problem
   }

   for(i=0,k=0;i<arp_route_entries;i++)
   {
       if(arproute_cacheEntry[i].flags & (LOCALROUTE | INDIRECT))
       {
          for(j=ipRouteDest;j<=ipRouteInfo;j++)
          {
              strcpy(iprt_oid_str[k],tMIB[j].abAddress);  //copy base OID string
              if(arproute_cacheEntry[i].flags & LOCALROUTE)
              {
                append_ip_to_oidstring(arproute_cacheEntry[i].ArpRouteGwayUnion.LocalNetRouteEntry.DevIpAddress
                                      ,iprt_oid_str[k]);
              }
              else
              {
                 append_ip_to_oidstring(arproute_cacheEntry[i].ArpRouteGwayUnion.GatewayEntry.GatewayIpAddress
                                      ,iprt_oid_str[k]);
              }
              iprt_tble[k].abAddress  = iprt_oid_str[k];     //set abAddress pointer
              iprt_tble[k].wChannel   = j;
              iprt_tble[k].tbl_idx    = (unsigned char)i;    //table index for get_snmp_value
              iprt_tble[k].baseOIDindex=6;
              iprt_tble[k++].mibPointer = (tMib *)0L;
          }//for(j
       }//if(arp
   }//for(i
   /*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      The IP route table has a general problem:
      Because of indexing mib only by RouteIPaddress,
      it's possible that duplicate mib OID come to existance
      => must remove duplicate entries and rebuild the ip route table
   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
   for(i=0;i<k-1;i++)
   {
      for(j=i+1;j<k;j++)
      {
        if(0==strcmp(iprt_oid_str[i],iprt_oid_str[j]))
        {
          //null out duplicate entries
          iprt_tble[j].abAddress  = (char *)0L;
        }
      }
   }
   for(i=0;i<k;i++)
   {
     if(iprt_tble[i].abAddress  == (char *)0L)
     {
        for(j=i+1;j<k;j++)
        {
           if(iprt_tble[j].abAddress  != (char *)0L)
           {
              memcpy(&iprt_tble[i],&iprt_tble[j],sizeof(tMib));
              iprt_tble[j].abAddress=(char *)0L;
              break;
           }
        }
     }
   }
   i=0;
   while(i<k)
   {
     if(iprt_tble[i].abAddress  == (char *)0L)
     {
       break;
     }
     i++;
   }
   //point to next mib root tree entry, see mib.c
   if(i)
   {
     iprt_tble[i-1].mibPointer = &ipntm_tble[0];
   }
   else
   {
      tMIB[ipFragCreates].mibPointer = &ipntm_tble[0];
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building ip net to media table
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   if(-1==alloc_oid_strings(&old_ipntm,arp_route_entries,
                            ipNetToMediaType-ipNetToMediaIfIndex + 1,
                            MAX_IPNTM_OID_LEN,
                            ipntm_oid_str)
     )
   {
      return -1; //fatal memory allocation problem
   }
   //Building ip_net_to_media table
   for(i=0,k=0;i<arp_route_entries;i++)
   {
       if(arproute_cacheEntry[i].flags & (LINKLAYER | CLONED_ARP))
       {
          for(j=ipNetToMediaIfIndex;j<=ipNetToMediaType;j++)
          {
              strcpy(ipntm_oid_str[k],tMIB[j].abAddress);  //copy base OID string
              strcat(ipntm_oid_str[k], itoa((unsigned int)arproute_cacheEntry[i].ifIndex,abSTemp,10)); //append ifindex
              append_ip_to_oidstring(arproute_cacheEntry[i].arpIpAddress,ipntm_oid_str[k]);
              ipntm_tble[k].abAddress  = ipntm_oid_str[k];     //set abAddress pointer
              ipntm_tble[k].wChannel   = j;
              ipntm_tble[k].tbl_idx    = (unsigned char)i;     //table index for get_snmp_value
              ipntm_tble[k].baseOIDindex=7;
              ipntm_tble[k++].mibPointer = (tMib *)0L;   //null out next mib pointer
          }
       }
   }
   //point to next mib root tree entry, see mib.c
   if(k)
   {
     ipntm_tble[k-1].mibPointer = &tMIB[ipRoutingDiscards];
   }
   else
   {
      tMIB[ipFragCreates].mibPointer = &tMIB[ipRoutingDiscards];
   }
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   //Building udp oid socket table
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   for(i=0,k=0;i<avail;i++)
   {
       if(socks[i].protocol==IP_UDP)
       {
         for(j=udpLocalAddress;j<=udpLocalPort;j++)
         {
             strcpy(udp_oid_str[k],tMIB[j].abAddress);  //copy base OID string
             append_ip_to_oidstring(socks[i].IfIpAddress,udp_oid_str[k]);
             strcat(udp_oid_str[k],".");
             append_port_to_oidstring((unsigned int)socks[i].localPort,udp_oid_str[k]);
             udp_tble[k].abAddress=udp_oid_str[k];     //set abAddress pointer
             udp_tble[k].wChannel   = j;
             udp_tble[k].tbl_idx    = (unsigned char)i;  //table index for get_snmp_value
             udp_tble[k].baseOIDindex=14;
             udp_tble[k++].mibPointer = (tMib *)0L;   //null out next mib pointer
         }
       }
  }
  //point to next mib root tree entry, see mib.c
  udp_tble[k-1].mibPointer = &tMIB[snmpInPkts];
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Build tcp socket oid table
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  for(i=0,k=0;i<avail;i++)
  {
       if(socks[i].protocol==IP_TCP)
       {
         for(j=tcpConnState;j<=tcpConnRemPort;j++)
         {
             strcpy(tcp_oid_str[k],tMIB[j].abAddress);  //copy base OID string
             append_ip_to_oidstring(socks[i].IfIpAddress,tcp_oid_str[k]);
             strcat(tcp_oid_str[k],".");
             append_port_to_oidstring((unsigned int)socks[i].localPort,tcp_oid_str[k]);
             append_ip_to_oidstring(socks[i].remoteIP,tcp_oid_str[k]);
             strcat(tcp_oid_str[k],".");
             append_port_to_oidstring((unsigned int)socks[i].remotePort,tcp_oid_str[k]);
             tcp_tble[k].abAddress    = tcp_oid_str[k];     //set abAddress pointer
             tcp_tble[k].wChannel     = j;
             tcp_tble[k].tbl_idx      = (unsigned char)i;              //table index for get_snmp_value
             tcp_tble[k].baseOIDindex = 11;
             tcp_tble[k++].mibPointer = (tMib *)0L;   //null out next mib pointer
         }
       }
  }
  //point to next mib root tree entry, see mib.c
  if(k)
  {
    tcp_tble[k-1].mibPointer = &tMIB[tcpInErrs];
  }
  else
  {
    tMIB[tcpRetransSegs].mibPointer= &tMIB[tcpInErrs];
  }
  return 0;
}
/*****************************************************************************/
//end snmpv1.c
