/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     timerout.c
* Function:   Example of using the hardware api with timer outputs.
*
*******************************************************************************
*
* $Header: TIMEROUT.C, 3, 31.03.2004 11:16:06, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
volatile unsigned long int_counter;
unsigned char timer;
unsigned long freq;

/******************************************************************************
* Interrupt handler
******************************************************************************/
void huge int_handler (void)
{
  int_counter++;
}

/***************************************************************************/
// Display command line options
/***************************************************************************/
void usage (void)
{
  printf("\nProgram enables TMROUT0 or TMROUT1 pin and sets a frequency");
  printf("\nUsage: timerout <Timer> <Freq> [IRQ_flag]");
  printf("\n Timer: 0..1");
  printf("\n Freq : value in Hz");
  printf("\n IRQ_flag: >0 interrupt when timer count reaches maxcount");
  printf("\n\n NOTE: Be careful with enabling interrupts. If the freq is");
  printf("\n       to high, the system is busy with the acknowledge of the");
  printf("\n       interrupt. The watchdog task will get no time to refresh");
  printf("\n       the watchdog and the system will reboot.");
}

/******************************************************************************
* Main
******************************************************************************/
void main (int argc, char *argv[])
{
   unsigned      i, maxcount;
   unsigned long timer_freq, maxcount_long;
   unsigned      use_irq;

   if (argc>=3)
   {
      timer   = atoi(argv[1]);
      freq    = atol(argv[2]);
      if (argc==4)
      {
        use_irq = atoi(argv[3]);
      }
      else
      {
        use_irq = 0;
      }

      if (timer>1)
      {
        usage();
        return;
      }
   }
   else
   {
     usage();
     return;
   }

   printf("Installing TMROUT%d ...", timer);
   int_counter = 0;

   // Get timer base freq.
   timer_freq = hal_get_frequencies(GET_FRQ_TIMER);

   printf("\nTimer base frequency: %lu Hz", timer_freq);
   printf("\nSet TMROUT%d frequency: %lu Hz", timer, freq);

   // Calculate output freq.
   maxcount_long = ((unsigned long)timer_freq/freq);

   if ((maxcount_long>65535L) || (maxcount_long == 0))
   {
     printf("\nFrequency not possible");
     return;
   }

   maxcount = (unsigned)maxcount_long;

   printf("\nClock divider: %u", maxcount);

   // Enable timers
   if (timer==0)
   {
     pfe_enable_timer(0x3); // enable TMROUT0
   }
   else
   {
     pfe_enable_timer(0xC); // enable TMROUT1
   }

   // Init timer
   if (use_irq)
   {
     hal_init_timer(timer, 3, maxcount);      // continous, enable interrupt
     hal_install_isr(8+timer, 1, int_handler);
   }
   else
   {
     hal_init_timer(timer, 1, maxcount);      // continous
   }

   // Start timer
   hal_start_timer(timer);

   if (use_irq)
   {
      for (i=0; i<60; i++)
      {
        // wait 1 sec
        RTX_Sleep_Time(1000);

        printf("\nInterrupts on Timer%d: %lu", timer, int_counter);
        int_counter = 0;
      }

      // Deinstall timer service interrupt routine
      hal_install_isr(8+timer, 0, NULL);
   }

   printf("\ndone");
}

